/*
 * pib_widget_pezold.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

/*
 * KnowledgeBase Archive
 * An Archive of Early Microsoft KnowledgeBase Articles
 *
 * Q94646: INFO: Using GetCharABCWidths() to Calculate Text Extents
 *   https://jeffpar.github.io/kbarchive/kb/094/Q94646/
 */
BOOL NEAR PASCAL IsBoldSimulated( HDC hDC )
{

  TEXTMETRIC                 tm ;
  OUTLINETEXTMETRIC          otm ;

  GetTextMetrics( hDC, &tm ) ;

  if (tm.tmWeight <= FW_NORMAL)
     return FALSE ;

  otm.otmSize = sizeof( OUTLINETEXTMETRIC ) ;

  /* Call GOTM() with the pointer to the buffer.  It will
   * fill in the buffer.
   */ 
  if (!GetOutlineTextMetrics( hDC, otm.otmSize, &otm ))
     return FALSE ;  // Probably not a TrueType font

       /* See the documentation on OUTLINETEXTMETRIC for a
        * description of the otmfsSelection member.
        */ 
  if (!(otm.otmfsSelection & 0x0020)) // bit 5 means bold
        return TRUE ;


#if 1 // [
  return FALSE;
#endif // ]
} /* IsBoldSimulated()  */ 

/****************************************************************
*  UINT WINAPI GetTextExtentABCPoint( HDC hDC, LPSTR lpsz,
*                                     int cb, LPSIZE lpSize )
*
*    Just like GetTextExtentPoint() but only works on TT fonts
*    and uses GetCharABCWidths().
*
*    Unlike GetTextExtent() and GetTextExtentPoint() which
*    return a BOOL, this function returns the absolute value
*    for the "A" value of the first character if it is
*    negative otherwise it returns 0.
*
****************************************************************/ 

UINT WINAPI GetTextExtentABCPointW( HDC hDC, LPCWSTR lpsz,

                                  const int cb, LPSIZE lpSize )

{

  int          A, C ;
  UINT         nRet = 0 ;
  static LPABC lpABC ;

#if 1 // [
  lpSize->cx=0;
#endif // ]

  GetTextExtentPointW( hDC, lpsz, cb, lpSize );

  if (lpABC ||
#if 0 // [
  (lpABC = (LPABC)GlobalAllocPtr( GHND, 256*sizeof(ABC) ))
#else // ] [
  (lpABC = (LPABC)calloc(256,sizeof(ABC) ))
#endif // ]
   )
  {
     if (GetCharABCWidthsW( hDC, 0, 255, lpABC ))
     {
        int         n ;

        /* We now have an array of ABC structures
         */ 
        for ( lpSize->cx = 0, n = 0 ; n < cb ; n++ )
        {
#if 0 // [
           lpSize->cx += (
                          lpABC[(int)lpsz[n]].abcA +
                          lpABC[(int)lpsz[n]].abcB +
                          lpABC[(int)lpsz[n]].abcC ) ;
#else // ] [
           lpSize->cx+=lpABC[(int)lpsz[n]].abcA ;
           lpSize->cx+=lpABC[(int)lpsz[n]].abcB ;
           lpSize->cx+=lpABC[(int)lpsz[n]].abcC ;
#endif // ]
        }

        if( cb )
        {
           A = lpABC[(int)lpsz[0]].abcA;

           if( A > 0 )
           {
              nRet = -A ;
              lpSize->cx -= A;
           }

           C = lpABC[(int)lpsz[cb-1]].abcC ;

           if ( C < 0 )
              lpSize->cx -= C ;    // two minus' make a plus
        }

        /* Bug in Windows 3.1.  If the tt font is embolded then
         * add one to the width for each char.
         */ 
        if ((BOOL)LOWORD( GetVersion() ) == 0x0A03 &&
            IsBoldSimulated( hDC ))
           lpSize->cx += cb ;
     }
     else
        GetTextExtentPointW( hDC, lpsz, cb, lpSize ) ;

     return nRet  ;
  }
  return nRet ;

} /* GetTextExtentABCPoint()  */ 
