/*
 * pib_control_check_box.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

static PibControlVmt vmt;

PibControl *pib_control_check_box_new(PibControlConfig *config, PibForm *form,
		HWND hDlg)
{
  PibControl *control=pib_control_common_new(sizeof control->check_box,
      &vmt,config,form);
  HWND hControl=GetDlgItem(hDlg,config->check_box.idc);

  if (!control) {
    DMESSAGE("allocating check box control");
    goto e_control;
  }

  control->check_box=config->check_box;

  SetWindowLongPtrW(
    hControl,          // HWND     hWnd,
    GWLP_USERDATA,        // int      nIndex,
    (LONG_PTR)control     // LONG_PTR dwNewLong
  );

  return control;
//cleanup:
  pib_control_common_destroy(control);
e_control:
  return NULL;
}

static void check_box_destroy(PibControl *control)
{
  pib_control_common_destroy(control);
}

static int check_box_idc_min(const PibControl *control)
{
  return control->check_box.idc;
}

static int check_box_idc_max(const PibControl *control)
{
  return control->check_box.idc;
}

static int check_box_set(PibControl *control, HWND hDlg)
{
  int val=pib_property_get_int(control->common.property,
      pib_control_cons(control));
  HWND hControl=GetDlgItem(hDlg,control->check_box.idc);

  SendMessageW(hControl,BM_SETCHECK,val?BST_CHECKED:BST_UNCHECKED,0);

  return 0;
}

static int check_box_get(PibControl *control, HWND hDlg, int idc)
{
  HWND hControl=GetDlgItem(hDlg,control->check_box.idc);
  int val=BST_CHECKED==SendMessageW(hControl,BM_GETCHECK,0,0)?1:0;

  pib_property_set_int(control->common.property,
      pib_control_trans(control),val);
  pib_control_changed(control,hDlg);

  return 0;
}

static void check_box_notify(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
  case WM_COMMAND:
    /*
     * in case we're notified with a WM_COMMAND we need to read-out the
     * check box.
     */
    check_box_get(control,hDlg,LOWORD(wParam));
    break;
  default:
    break;
  }
}

static void check_box_post(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      check_box_set(control,hDlg);
      break;
    default:
      break;
    }

    break;
  default:
    break;
  }
}

static PibControlVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=check_box_destroy,
  .idc_min=check_box_idc_min,
  .idc_max=check_box_idc_max,
  .get=check_box_get,
  .notify=check_box_notify,
  .post=check_box_post,
};
