/*
 * pib_control_common.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

PibControl *pib_control_common_new(size_t size, PibControlVmt *vmt,
    PibControlConfig *config, PibForm *form)
{
  PibControl *control=calloc(1,(sizeof control->common)+size);

  if (!control) {
    DMESSAGE("allocating control");
    goto e_control;
  }

  control->common.vmt=vmt;
  control->common.form=form;
  control->common.property
      =(PibProperty *)((char *)form->properties->min+config->property_offs);

  if (!control->common.property) {
    DVMESSAGE("something went wrong: %p",control->common.property);
    goto e_property;
  }
  else if (!control->common.property->vmt) {
    DVMESSAGE("something went wrong: %p %p",control->common.property->vmt);
    goto e_property_vmt;
  }

  control->common.cb.changed=config->changed;

  return control;
e_property_vmt:
e_property:
e_control:
  return NULL;
}

void pib_control_common_destroy(PibControl *control)
{
  free(control);
}

void pib_control_changed(PibControl *control, HWND hDlg)
{
  if (control&&control->common.cb.changed)
    control->common.cb.changed(control,hDlg);
}
