/*
 * pib_control_label.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

static PibControlVmt vmt;

PibControl *pib_control_label_new(PibControlConfig *config, PibForm *form,
    HWND hDlg)
{
  PibControl *control=pib_control_common_new(sizeof control->label,
      &vmt,config,form);

  if (!control) {
    DMESSAGE("creating control");
    goto e_control;
  }

  control->label=config->label;

  if (control->label.text)
    SetWindowTextW(GetDlgItem(hDlg,control->label.idc),control->label.text);

  return control;
//cleanup:
  pib_control_common_destroy(control);
e_control:
  return NULL;
}

static void label_destroy(PibControl *control)
{
  pib_control_common_destroy(control);
}

static int label_idc_min(const PibControl *control)
{
  return control->label.idc;
}

static int label_idc_max(const PibControl *control)
{
  return control->label.idc;
}

static int label_set(PibControl *control, HWND hDlg)
{
  pib_control_changed(control,hDlg);

  return 0;
}

static int label_get(PibControl *control, HWND hDlg, int idc)
{
  // not supported, i.e. succeed without doing something.
  return 0;
}

static void label_notify(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
#if 0 // [
  case WM_COMMAND:
    /*
     * in case we're notified with a WM_COMMAND we need to read-out the
     * radio group.
     */
    label_get(control,LOWORD(wParam));
    break;
#endif // ]
  default:
    break;
  }
}

static void label_post(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      label_set(control,hDlg);
      break;
    default:
      break;
    }

    break;
  default:
    break;
  }
}

static PibControlVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=label_destroy,
  .idc_min=label_idc_min,
  .idc_max=label_idc_max,
  .get=label_get,
  .notify=label_notify,
  .post=label_post,
};
