/*
 * pib_control_picture.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

static PibControlVmt vmt;
static WNDCLASSW *pib_picture_wndclassw(void);

PibControl *pib_control_picture_new(PibControlConfig *config,
    PibForm *form, HWND hDlg)
{
  PibControl *control=pib_control_common_new(sizeof control->picture,
      &vmt,config,form);
  WNDCLASSW *wndclassw=pib_picture_wndclassw();
  HWND hWnd;
  RECT rect;
  HGDIOBJ hFont;

  if (!control) {
    DMESSAGE("creating control");
    goto e_control;
  }

  control->picture.config=config->picture;
  hWnd=GetDlgItem(hDlg,control->picture.config.idc);
  GetWindowRect(hWnd,&rect);

  MapWindowPoints(
    NULL,                                 // HWND    hWndFrom,
    hDlg,                                 // HWND    hWndTo,
    (POINT *)&rect,                       // LPPOINT lpPoints,
    2                                     // UINT    cPoints
  );

  hFont=(HFONT)SendMessageW(hWnd,WM_GETFONT,0,0);
  DestroyWindow(hWnd);

  hWnd=CreateWindowExW(
    0,                                    // DWORD     dwExStyle,
    wndclassw->lpszClassName,             // LPCWSTR   lpClassName,
    NULL,                                 // LPCWSTR   lpWindowName,
    WS_BORDER|WS_VISIBLE|WS_CHILD/*|WS_TABSTOP*/,   // DWORD     dwStyle,
    rect.left,                            // int       X,
    rect.top,                             // int       Y,
    (rect.right-rect.left),               // int       nWidth,
    (rect.bottom-rect.top),               // int       nHeight,
    hDlg,                                 // HWND      hWndParent,
    (HMENU)config->picture.idc,   // HMENU     hMenu,
    plugin_hinstance(),                   // HINSTANCE hInstance,
    control                               // LPVOID    lpParam
  );

  if (!hWnd) {
    DMESSAGE("creating window");
    goto e_window;
  }

  SendMessageW(hWnd,WM_SETFONT,(WPARAM)hFont,TRUE);
  control->picture.hWnd=hWnd;

  return control;
//cleanup:
e_window:
  pib_control_common_destroy(control);
e_control:
  return NULL;
}

static void picture_destroy(PibControl *control)
{
  pib_control_common_destroy(control);
}

static int picture_idc_min(const PibControl *control)
{
  return control->picture.config.idc;
}

static int picture_idc_max(const PibControl *control)
{
  return control->picture.config.idc;
}

static int picture_set(PibControl *control, HWND hDlg)
{
  InvalidateRect(control->picture.hWnd,NULL,TRUE);

  return 0;
}

static int picture_get(PibControl *control, HWND hDlg, int idc)
{
  // not supported, i.e. succeed without doing something.
  return 0;
}

static void picture_notify(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
#if 0 // [
  case WM_COMMAND:
    /*
     * in case we're notified with a WM_COMMAND we need to read-out the
     * radio group.
     */
    picture_get(control,LOWORD(wParam));
    break;
#endif // ]
  default:
    break;
  }
}

static void picture_post(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      picture_set(control,hDlg);
      break;
    default:
      break;
    }

    break;
  default:
    break;
  }
}

static LRESULT CALLBACK pib_picture_window_procw(HWND hWnd, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  PibControl *control=(PibControl *)GetWindowLongPtrW(hWnd,GWLP_USERDATA);

  switch (uMsg) {
  case WM_CREATE:
    control=((CREATESTRUCTW *)lParam)->lpCreateParams;

    SetWindowLongPtrW(
      hWnd,                         // HWND     hWnd,
      GWLP_USERDATA,                // int      nIndex,
      (LONG_PTR)control             // LONG_PTR dwNewLong
    );

    break;
  case WM_PAINT:
    if (control->picture.config.paint) {
      control->picture.config.paint(control,
          control->picture.hWnd);
    }

    return 0;
  default:
    break;
  }

  return DefWindowProcW(
    hWnd,                           // HWND   hWnd,
    uMsg,                           // UINT   Msg,
    wParam,                         // WPARAM wParam,
    lParam                          // LPARAM lParam
  );
}

static WNDCLASSW *pib_picture_wndclassw(void)
{
  static WNDCLASSW wndclassw;

  if (!wndclassw.lpszClassName) {
#if 0 // [
    wndclassw.style=0;              // UINT      style;
#endif // ]
    wndclassw.lpfnWndProc=pib_picture_window_procw;
                                    // WNDPROC   lpfnWndProc;
#if 0 // [
    wndclassw.cbClsExtra=0;         // int       cbClsExtra;
    wndclassw.cbWndExtra=0;         // int       cbWndExtra;
#endif // ]
    wndclassw.hInstance=plugin_hinstance();
                                    // HINSTANCE hInstance;
#if 0 // [
    wndclassw.hIcon=NULL;           // HICON     hIcon;
    wndclassw.hCursor=NULL;         // HCURSOR   hCursor;
#endif // ]
#if 0 // [
    wndclassw.hbrBackground=(HBRUSH)GetStockObject(LTGRAY_BRUSH);
                                    // HBRUSH    hbrBackground;
#else // ] [
    wndclassw.hbrBackground=(HBRUSH)(COLOR_BTNFACE+1),
                                    // HBRUSH    hbrBackground;
#endif // ]
#if 0 // [
    wndclassw.lpszMenuName=NULL;    // LPCWSTR   lpszMenuName;
#endif // ]
    wndclassw.lpszClassName=L"PIB_PICTURE_CONTROL";
                                    // LPCWSTR   lpszClassName;
    RegisterClassW(&wndclassw);
  }

  return &wndclassw;
}

static PibControlVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=picture_destroy,
  .idc_min=picture_idc_min,
  .idc_max=picture_idc_max,
  .get=picture_get,
  .notify=picture_notify,
  .post=picture_post,
};
