/*
 * pib_control_progress_bar.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

static PibControlVmt vmt;

#define CHANGE_COLOR
#if defined (CHANGE_COLOR) // [
/*
 * How to change the progress bar color?
 *   https://stackoverflow.com/questions/25307515/how-to-change-the-progress-bar-color
 */
#endif // ]
PibControl *pib_control_progress_bar_new(PibControlConfig *config,
    PibForm *form, HWND hDlg)
{
  PibControl *control=pib_control_common_new(sizeof control->progress_bar,
      &vmt,config,form);
#if defined (CHANGE_COLOR) // [
  HWND hBar;
#endif // ]

  if (!control) {
    DMESSAGE("creating control");
    goto e_control;
  }

#if defined (CHANGE_COLOR) // [
	/*
	 * Changing a progress bar color in Windows 7
	 *   https://forum.powerbasic.com/forum/user-to-user-discussions/powerbasic-for-windows/51610-changing-a-progress-bar-color-in-windows-7
	 */
	//SendMessageW(hBar,PBM_SETSTATE,PBST_ERROR,0);
	//SendMessageW(hBar,PBM_SETSTATE,PBST_PAUSED,0);
	//SendMessageW(hBar,PBM_SETSTATE,PBS_MARQUEE,0);
	if (config->progress_bar.state) {
  	hBar=GetDlgItem(hDlg,config->progress_bar.idc);
DVWRITELN("### %d %p",config->progress_bar.idc,hBar);
		SendMessageW(hBar,PBM_SETSTATE,config->progress_bar.state,0);
	}
#endif // ]
  control->progress_bar=config->progress_bar;

  return control;
//cleanup:
  pib_control_common_destroy(control);
e_control:
  return NULL;
}

static void progress_bar_destroy(PibControl *control)
{
  pib_control_common_destroy(control);
}

static int progress_bar_idc_min(const PibControl *control)
{
  return control->progress_bar.idc;
}

static int progress_bar_idc_max(const PibControl *control)
{
  return control->progress_bar.idc;
}

static int progress_bar_set(PibControl *control, HWND hDlg)
{
  void *trans=pib_control_trans(control);
  int max_offs=control->progress_bar.max_offs;
  PibProperty *property_max
      =(void *)((char *)pib_control_form(control)->properties->min+max_offs);
  double max=pib_property_get_double(property_max,trans);
  double x=pib_property_get_double(control->common.property,trans);

  if (!max)
    goto e_max;

  SendMessageW(GetDlgItem(hDlg,control->progress_bar.idc),
      PBM_SETPOS,100.0*x/max,0);
  pib_control_changed(control,hDlg);

  return 0;
e_max:
  return -1;
}

static int progress_bar_get(PibControl *control, HWND hDlg, int idc)
{
  // not supported, i.e. succeed without doing something.
  return 0;
}

static void progress_bar_notify(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
#if 0 // [
  case WM_COMMAND:
    /*
     * in case we're notified with a WM_COMMAND we need to read-out the
     * radio group.
     */
    progress_bar_get(control,LOWORD(wParam));
    break;
#endif // ]
  default:
    break;
  }
}

static void progress_bar_post(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      progress_bar_set(control,hDlg);
      break;
    default:
      break;
    }

    break;
  default:
    break;
  }
}

static PibControlVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=progress_bar_destroy,
  .idc_min=progress_bar_idc_min,
  .idc_max=progress_bar_idc_max,
  .get=progress_bar_get,
  .notify=progress_bar_notify,
  .post=progress_bar_post,
};
