/*
 * pib_control_slider.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

static PibControlVmt vmt;

PibControl *pib_control_slider_new(PibControlConfig *config,
    PibForm *form, HWND hDlg)
{
  PibControl *control=pib_control_common_new(sizeof control->slider,
      &vmt,config,form);
  HWND hPibControl=GetDlgItem(hDlg,config->slider.idc);

  if (!control) {
    DMESSAGE("creating control");
    goto e_control;
  }

  control->slider=config->slider;

  SetWindowLongPtrW(
    hPibControl,          // HWND     hWnd,
    GWLP_USERDATA,        // int      nIndex,
    (LONG_PTR)control     // LONG_PTR dwNewLong
  );

  return control;
//cleanup:
  pib_control_common_destroy(control);
e_control:
  return NULL;
}

static void slider_destroy(PibControl *control)
{
  pib_control_common_destroy(control);
}

static int slider_idc_min(const PibControl *control)
{
  return control->slider.idc;
}

static int slider_idc_max(const PibControl *control)
{
  return control->slider.idc;
}

static double slider_max(PibControl *control, void *data)
{
  int max_offs=control->slider.max_offs;
  PibProperty *property_max
      =(void *)((char *)pib_control_form(control)->properties->min+max_offs);
  
  return pib_property_get_double(property_max,data);
}

static int slider_set(PibControl *control, HWND hDlg, int idc)
{
  void *trans=pib_control_trans(control);
  double max=slider_max(control,trans);
  double x=pib_property_get_double(control->common.property,trans);
  double pos;

  if (!max)
    goto e_max;

  pos=100.0*x/max+0.5;
  SendMessageW(GetDlgItem(hDlg,idc),TBM_SETPOS,TRUE,pos);

  return 0;
e_max:
  return -1;
}

static int slider_get(PibControl *control, HWND hDlg, int idc)
{
  void *trans=pib_control_trans(control);
  double max=slider_max(control,trans);
  double pos=SendMessageW(GetDlgItem(hDlg,idc),TBM_GETPOS,0,0);
  double x=max*pos/100.0;

  pib_property_set_double(control->common.property,trans,x);
  pib_control_changed(control,hDlg);

  return 0;
}

static void slider_notify(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
  case WM_NOTIFY:
    slider_get(control,hDlg,control->slider.idc);
    break;
  default:
    break;
  }
}

static void slider_post(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      slider_set(control,hDlg,control->slider.idc);
      break;
    default:
      break;
    }

    break;
  default:
    break;
  }
}

static PibControlVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=slider_destroy,
  .idc_min=slider_idc_min,
  .idc_max=slider_idc_max,
  .get=slider_get,
  .notify=slider_notify,
  .post=slider_post,
};
