/*
 * pib_control_sys_link.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

static PibControlVmt vmt;

//#define PIB_CONTROL_SYS_LINK_NEW_MAP
PibControl *pib_control_sys_link_new(PibControlConfig *config, PibForm *form,
    HWND hDlg)
{
  PibControl *control=pib_control_common_new(sizeof control->sys_link,
      &vmt,config,form);
  HWND hWnd;
  RECT rect;
  HGDIOBJ hFont;

  if (!control) {
    DMESSAGE("creating control");
    goto e_control;
  }

  control->sys_link=config->sys_link;
  hWnd=GetDlgItem(hDlg,control->sys_link.idc);
#if 0 // [
  GetClientRect(hWnd,&rect);
#else // ] [
  GetWindowRect(hWnd,&rect);

  MapWindowPoints(
    NULL,                                 // HWND    hWndFrom,
    hDlg,                                 // HWND    hWndTo,
    (POINT *)&rect,                       // LPPOINT lpPoints,
    2                                     // UINT    cPoints
  );
#endif // ]
  hFont=(HFONT)SendMessageW(hWnd,WM_GETFONT,0,0);
  DestroyWindow(hWnd);

  hWnd=CreateWindowExW(
    0,
    WC_LINK, 
    control->sys_link.text,
    WS_VISIBLE|WS_CHILD |WS_TABSTOP, 
    rect.left,
    rect.top,
    (rect.right-rect.left),               // int       nWidth,
    (rect.bottom-rect.top),               // int       nHeight,
    hDlg,
    (HMENU)control->sys_link.idc,
    plugin_hinstance(),
    NULL
  );

#if 0 // [
  /*
   * How to change text size in basic text window win32 c++
   *   https://stackoverflow.com/questions/17347324/how-to-change-text-size-in-basic-text-window-win32-c
   */
  hFont=CreateFontW(
#if 0 // [
    20,                       // int    cHeight,
#else // ] [
    12,                       // int    cHeight,
#endif // ]
    0,                        // int    cWidth,
    0,                        // int    cEscapement,
    0,                        // int    cOrientation,
    FW_DONTCARE,              // int    cWeight,
    FALSE,                    // DWORD  bItalic,
    FALSE,                    // DWORD  bUnderline,
    FALSE,                    // DWORD  bStrikeOut,
    ANSI_CHARSET,             // DWORD  iCharSet,
    OUT_DEFAULT_PRECIS,       // DWORD  iOutPrecision,
    CLIP_DEFAULT_PRECIS,      // DWORD  iClipPrecision,
    DEFAULT_QUALITY,          // DWORD  iQuality,
    DEFAULT_PITCH|FF_SWISS,   // DWORD  iPitchAndFamily,
#if 0 // [
    L"Arial"                  // LPCSTR pszFaceName
#else // ] [
    L"Ms Shell Dlg"           // LPCSTR pszFaceName
#endif // ]
    );
#endif // ]
  SendMessageW(hWnd,WM_SETFONT,(WPARAM)hFont,TRUE);

  return control;
//cleanup:
  pib_control_common_destroy(control);
e_control:
  return NULL;
}

static void sys_link_destroy(PibControl *control)
{
  pib_control_common_destroy(control);
}

static int sys_link_idc_min(const PibControl *control)
{
  return control->sys_link.idc;
}

static int sys_link_idc_max(const PibControl *control)
{
  return control->sys_link.idc;
}

static int sys_link_set(PibControl *control, HWND hDlg)
{
  pib_control_changed(control,hDlg);

  return 0;
}

static int sys_link_get(PibControl *control, HWND hDlg, int idc)
{
  // not supported, i.e. succeed without doing something.
  return 0;
}

static void sys_link_notify(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
#if 0 // [
  case WM_COMMAND:
    /*
     * in case we're notified with a WM_COMMAND we need to read-out the
     * radio group.
     */
    sys_link_get(control,LOWORD(wParam));
    break;
#endif // ]
  default:
    break;
  }
}

static void sys_link_post(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  switch (uMsg) {
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      sys_link_set(control,hDlg);
      break;
    default:
      break;
    }

    break;
  default:
    break;
  }
}

static PibControlVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=sys_link_destroy,
  .idc_min=sys_link_idc_min,
  .idc_max=sys_link_idc_max,
  .get=sys_link_get,
  .notify=sys_link_notify,
  .post=sys_link_post,
};
