/*
 * pib_control_tab.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>
//#include <stdlib.h>
#include <uxtheme.h>

static PibControlVmt vmt;

static void tab_page_adjust(HWND hDlg, HWND hTab, HWND hPage,
    RECT *rect_page)
{
	RECT rect_tab;
	long delta;

  GetClientRect(
    hTab,                  	// HWND   hWnd,
    &rect_tab               // LPRECT lpRect
  );

  TabCtrl_AdjustRect(
    hTab,                   // hwnd,
    FALSE,                  // bLarger,
    &rect_tab               // prc
  );

	////////
  GetClientRect(
    hPage,                  // HWND   hWnd,
    rect_page               // LPRECT lpRect
  );


	delta=(rect_tab.right-rect_tab.left)-(rect_page->right-rect_page->left);
	delta>>=1;
	rect_page->left+=delta;
	rect_page->right+=delta;

	delta=(rect_tab.bottom-rect_tab.top)-(rect_page->bottom-rect_page->top);
	delta>>=1;
	rect_page->top+=delta;
	rect_page->bottom+=delta;
#if 0 // [
	rect_page->top+=rect_tab.top;
	rect_page->bottom+=rect_tab.top;
#endif // ]

	////////
  TabCtrl_AdjustRect(
    hTab,                   // hwnd,
    FALSE,                  // bLarger,
    rect_page               // prc
  );

  /////////////////////////////////////////////////////////////////////////////
  // MapWindowPoints function: The MapWindowPoints function converts (maps)
  // a set of points from a coordinate space relative to one window to a
  // coordinate space relative to another window.
  // HWND hWndFrom: A handle to the window from which points are converted.
  //   If this parameter is NULL or HWND_DESKTOP, the points are presumed
  //   to be in screen coordinates.
  // HWND hWndTo: A handle to the window to which points are converted.
  //   If this parameter is NULL or HWND_DESKTOP, the points are converted
  //   to screen coordinates.
  // LPPOINT lpPoints: A pointer to an array of POINT structures that
  //   contain the set of points to be converted. The points are in device
  //   units. This parameter can also point to a RECT structure, in which
  //   case the cPoints parameter should be set to 2.
  // UINT cPoints: The number of POINT structures in the array pointed to by
  //   the lpPoints parameter.
  MapWindowPoints(
    hTab,                   // _In_    HWND    hWndFrom,
    hDlg,                   // _In_    HWND    hWndTo,
    (LPPOINT)rect_page,     // _Inout_ LPPOINT lpPoints,
    2                       // _In_    UINT    cPoints
  );
}

static int tab_page_create(PibControl *control, PibForm *form,
    HWND hDlg, HWND hTab, int n)
{
  enum {
    SWP_FLAGS=SWP_DEFERERASE
        |SWP_NOSENDCHANGING
        |SWP_NOACTIVATE
        |SWP_NOSIZE
        |SWP_NOZORDER
        |SWP_NOREDRAW
  };

  PibFormVmt *form_vmt=control->tab.config.form_vmts[n];

  PibFormInit init={
    .form_vmt=form_vmt,
    .form=&control->tab.pages[n].form,
    .properties=form->properties,
    .data=form->data,
  };

#if defined (PIB_GEN) // [
  winampGeneralPurposePlugin *gen=winampGetGeneralPurposePlugin();
#endif // ]
  HWND *hPage=&control->tab.pages[n].hPage;
  TCITEMW tci;
  RECT rect;

  /////////////////////////////////////////////////////////////////////////////
  memset(&tci,0,sizeof tci);
  tci.mask=TCIF_TEXT;
  tci.pszText=form_vmt->label;
  SendMessageW(hTab,TCM_INSERTITEMW,n,(LPARAM)&tci);

  /////////////////////////////////////////////////////////////////////////////
  *hPage=CreateDialogParamW(
#if defined (PIB_GEN) // [
    gen->hDllInstance,                // HINSTANCE hInstance,
#else // ] [
    plugin_hinstance(),               // HINSTANCE hInstance,
#endif // ]
    MAKEINTRESOURCEW(form_vmt->idd),  // LPCWSTR   lpTemplateName,
    hDlg,                             // HWND      hWndParent,
    form_vmt->dlgproc,                // DLGPROC   lpDialogFunc,
    (LPARAM)&init                     // LPARAM    dwInitParam
  );

  if (!*hPage) {
    DMESSAGE("creating dialog");
    goto e_dialog;
  }

  EnableThemeDialogTexture(
    *hPage,                           // _In_ HWND  hwnd,
    ETDT_ENABLETAB                    // _In_ DWORD dwFlags
  );

  /////////////////////////////////////////////////////////////////////////////
  tab_page_adjust(hDlg,hTab,*hPage,&rect);

  SetWindowPos(
    *hPage,                           // HWND hWnd,
    HWND_NOTOPMOST,                   // HWND hWndInsertAfter,
    rect.left,                        // int  X,
    rect.top,                         // int  Y,
    rect.right-rect.left,             // int  cx,
    rect.bottom-rect.top,             // int  cy,
    SWP_FLAGS                         // UINT uFlags
  );

  return 0;
//cleanup:
e_dialog:
  return -1;
}

PibControl *pib_control_tab_new(PibControlConfig *config,
    PibForm *form, HWND hDlg)
{
  enum {
    SIZE_TAB_CONTROL=sizeof ((PibControl *)NULL)->tab,
    SIZE_PAGE=sizeof ((PibControl *)NULL)->tab.pages[0],
  };

  size_t nitems=config->tab.npages;
  size_t size=SIZE_TAB_CONTROL+(nitems-1)*SIZE_PAGE;
  HWND hTab=GetDlgItem(hDlg,config->tab.idc);
#if 0 // [
  PibControl *control=control_common_new(size,&vmt,config,form);
#else // ] [
  PibControl *control;
#endif // ]
  int n;

  control=pib_control_common_new(size,&vmt,config,form);

  if (!control) {
    DMESSAGE("allocating tab control");
    goto e_control;
  }

  control->tab.n=-1;
  control->tab.config=config->tab;

  for (n=0;n<nitems;++n) {
    if (tab_page_create(control,form,hDlg,hTab,n)<0) {
      DMESSAGE("creating page");
      goto e_page;
    }
  }

  SetWindowLongPtrW(
    hTab,                 // HWND     hWnd,
    GWLP_USERDATA,        // int      nIndex,
    (LONG_PTR)control     // LONG_PTR dwNewLong
  );

  return control;
//cleanup:
e_page:
  pib_control_common_destroy(control);
e_control:
  return NULL;
}

static void tab_destroy(PibControl *control)
{
  // TODO: destroy pages!
  pib_control_common_destroy(control);
}

static int tab_idc_min(const PibControl *control)
{
  return control->tab.config.idc;
}

static int tab_idc_max(const PibControl *control)
{
  return control->tab.config.idc;
}

static void tab_switch_tab(PibControl *control, int page, HWND hDlg)
{
  enum {
    SWP_FLAGS=SWP_NOACTIVATE
        |SWP_DEFERERASE
        |SWP_NOMOVE
        |SWP_NOSIZE
        |SWP_NOZORDER
  };

  HWND hTab=GetDlgItem(hDlg,control->tab.config.idc);
  int *n=&control->tab.n;

  if (0<=*n&&*n!=page) {
    SetWindowPos(
      control->tab.pages[*n].hPage,
                                // HWND hWnd,
      HWND_NOTOPMOST,           // HWND hWndInsertAfter,
      0,                        // int  X,
      0,                        // int  Y,
      0,                        // int  cx,
      0,                        // int  cy,
      SWP_HIDEWINDOW|SWP_FLAGS  // UINT uFlags
    );
  }

  *n=page;

  SetWindowPos(
    control->tab.pages[*n].hPage,
                                // HWND hWnd,
    HWND_NOTOPMOST,             // HWND hWndInsertAfter,
    0,                          // int  X,
    0,                          // int  Y,
    0,                          // int  cx,
    0,                          // int  cy,
    SWP_SHOWWINDOW|SWP_FLAGS    // UINT uFlags
  );

  SendMessageW(hTab,TCM_SETCURSEL,*n,0);
  pib_property_set_int(control->common.property,pib_control_trans(control),*n);
}

static int tab_set(PibControl *control, HWND hDlg)
{
  int n=pib_property_get_int(control->common.property,
      pib_control_trans(control));

  tab_switch_tab(control,n,hDlg);

  return 0;
}

static int tab_get(PibControl *control, HWND hDlg, int idc)
{
  HWND hTab=GetDlgItem(hDlg,control->tab.config.idc);
  int n=SendMessageW(hTab,TCM_GETCURSEL,0,0);

  pib_property_set_int(control->common.property,pib_control_trans(control),n);

  return 0;
}

static void tab_notify(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  int idc=control->tab.config.idc;

  switch (uMsg) {
  case WM_NOTIFY:
DWRITELN("WM_NOTIFY");
    if (idc==wParam&&TCN_SELCHANGE==((NMHDR *)lParam)->code)
      tab_switch_tab(control,TabCtrl_GetCurSel(GetDlgItem(hDlg,idc)),
          hDlg);

    break;
  default:
    break;
  }
}

static void tab_post(PibControl *control, HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  int i;

  switch (uMsg) {
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      tab_set(control,hDlg);
      break;
    default:
      break;
    }

    break;
  default:
    break;
  }

#if defined (PIB_SEND_MESSAGE) // [
  for (i=0;i<control->tab.config.npages;++i)
    SendMessageW(control->tab.pages[i].hPage,uMsg,wParam,lParam);
#else // ] [
  for (i=0;i<control->tab.config.npages;++i)
    PostMessageW(control->tab.pages[i].hPage,uMsg,wParam,lParam);
#endif // ]
}

static PibControlVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=tab_destroy,
  .idc_min=tab_idc_min,
  .idc_max=tab_idc_max,
  .get=tab_get,
  .notify=tab_notify,
  .post=tab_post,
};
