/*
 * properties.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

int pib_property_common_create(PibProperty *property, PibPropertyVmt *vmt,
    const wchar_t *app, const wchar_t *key, int offs,
    pib_property_callback_t *changed, void *data)
{
  property->vmt=vmt;
  property->app=app;
  property->key=key;
  property->offs=offs;
  property->cb.changed=changed;
  property->cb.data=data;

  return 0;
}

void pib_property_common_destroy(PibProperty *property)
{
}

void pib_property_fire_changed(const PibProperty *property,
    const PibProperties *properties)
{
  if (property->cb.changed)
    property->cb.changed(property,properties,property->cb.data);
}

int pib_properties_create(PibProperties *properties, PibProperty *min,
    PibProperty *max, void *cons, void *trans, const wchar_t *path)
{
  properties->path=path;
  properties->min=min;
  properties->max=max;
  properties->cons=cons;
  properties->trans=trans;
  pib_properties_read(properties);

  return 0;
}

void pib_properties_destroy(PibProperties *properties)
{
}

void pib_properties_read(PibProperties *properties)
{
  PibProperty *cur;

  for (cur=properties->min;cur<properties->max;++cur) {
    if (cur->app&&cur->key)
      pib_property_read(cur,properties);
  }
}

void pib_properties_write(PibProperties *properties)
{
  PibProperty *cur;

  for (cur=properties->min;cur<properties->max;++cur) {
    if (cur->app&&cur->key)
      pib_property_write(cur,properties);
  }
}
