/*
 * pib_property_double.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

static PibPropertyVmt vmt;

int pib_property_double_create_ex(PibProperty *property, const wchar_t *app,
    const wchar_t *key, int offs,
    pib_property_callback_t *changed, void *data)
{
  return pib_property_common_create(property,&vmt,app,key,offs,changed,data);
}

static void pib_property_double_read(const PibProperty *property,
    PibProperties *properties)
{
  enum { SIZE=64 };
  const wchar_t *path=properties->path;
  double *trans=(double *)(((char *)properties->trans)+property->offs);
  double *cons=(double *)(((char *)properties->cons)+property->offs);
  wchar_t buf[SIZE];

  swprintf(buf,SIZE,L"%lf",*trans);

  GetPrivateProfileStringW(
    property->app,      // LPCWSTR lpAppName,
    property->key,      // LPCWSTR lpKeyName,
    buf,                // LPCWSTR lpDefault,
    buf,                // LPWSTR  lpReturnedString,
    SIZE,               // DWORD   nSize,
    path                // LPCWSTR lpFileName
  );

  swscanf(buf,L"%lf",trans);
  *cons=*trans;
DVWRITELNW("!!! %s.%s cons:%f trans:%f",property->app,property->key,*cons,*trans);
}

static void pib_property_double_write(const PibProperty *property,
    PibProperties *properties)
{
  enum { SIZE=64 };
  const wchar_t *path=properties->path;
  double *trans=(double *)(((char *)properties->trans)+property->offs);
  double *cons=(double *)(((char *)properties->cons)+property->offs);
  wchar_t buf[SIZE];

DVWRITELNW("!!! %s.%s cons:%f trans:%f",property->app,property->key,*cons,*trans);
  if (*cons!=*trans) {
    pib_property_fire_changed(property,properties);
    swprintf(buf,SIZE,L"%lf",*trans);

    WritePrivateProfileStringW(
      property->app,      // LPCWSTR lpAppName,
      property->key,      // LPCWSTR lpKeyName,
      buf,                // LPCWSTR lpString,
      path                // LPCWSTR lpFileName
    );

    *cons=*trans;
  }
}

static int pib_property_double_get_int(PibProperty *property, void *data)
{
  return *(double *)(((char *)data)+property->offs);
}

static void pib_property_double_set_int(PibProperty *property, void *data,
    int i)
{
  *(double *)(((char *)data)+property->offs)=i;
}

static long pib_property_double_get_long(PibProperty *property, void *data)
{
  return *(double *)(((char *)data)+property->offs);
}

static void pib_property_double_set_long(PibProperty *property, void *data,
    long l)
{
  *(double *)(((char *)data)+property->offs)=l;
}

static double pib_property_double_get_double(PibProperty *property, void *data)
{
  return *(double *)(((char *)data)+property->offs);
}

static void pib_property_double_set_double(PibProperty *property, void *data,
    double x)
{
  *(double *)(((char *)data)+property->offs)=x;
}

static PibPropertyVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=pib_property_common_destroy,
  .read=pib_property_double_read,
  .write=pib_property_double_write,
  .get_int=pib_property_double_get_int,
  .set_int=pib_property_double_set_int,
  .get_long=pib_property_double_get_long,
  .set_long=pib_property_double_set_long,
  .get_double=pib_property_double_get_double,
  .set_double=pib_property_double_set_double,
};
