/*
 * pib_trace.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

#if defined (DEBUG) // [
//#define TRACE_REDIRECT

///////////////////////////////////////////////////////////////////////////////
static struct {
  int bInitialized;
  CRITICAL_SECTION cs;
#if ! defined (TRACE_REDIRECT) // [
  int bConsole;
#endif // ]
} trace;

///////////////////////////////////////////////////////////////////////////////
int pib_trace_create(void)
{
  if (trace.bInitialized)
    goto e_initialized;

#if ! defined (TRACE_REDIRECT) // [
  if (trace.bConsole)
    goto e_console;
#endif // ]

  InitializeCriticalSection(&trace.cs);
#if ! defined (TRACE_REDIRECT) // [
  trace.bConsole=AllocConsole();

  if (trace.bConsole) {
#endif // ]
    freopen("conin$", "r",stdin);
    freopen("conout$","w",stdout);
#if defined (TRACE_REDIRECT) // [
    freopen("D:\\gen-rand.log","w",stderr);
#else // ] [
    freopen("conout$","w",stderr);
#endif // ]
    DWRITELN("Console allocated");
#if ! defined (TRACE_REDIRECT) // [
  }
#endif // ]
#if ! defined (TRACE_REDIRECT) // [
e_console:
#endif // ]
e_initialized:
  return 0;
}

void pib_trace_destroy(void)
{
#if ! defined (TRACE_REDIRECT) // [
  fputs("Hit some key ...",stderr);
  fflush(stderr);
  fgetc(stdin);

  if (trace.bConsole)
    FreeConsole();
#endif // ]

  DeleteCriticalSection(&trace.cs);
}

///////////////////////////////////////////////////////////////////////////////
void pib_trace_writeln(const char *path, int line, const char *func,
    const char *format)
{
  pib_trace_create();
  EnterCriticalSection(&trace.cs); // [
  fputs(format,stderr);
  fprintf(stderr,". [%s:%d:%s]\n",pib_basenamea(path),line,func);
  fflush(stderr);
  LeaveCriticalSection(&trace.cs); // ]
}

void pib_trace_writelnw(const char *path, int line, const char *func,
    const wchar_t *format)
{
  pib_trace_create();
  EnterCriticalSection(&trace.cs); // [
  fputws(format,stderr);
  fprintf(stderr,". [%s:%d:%s]\n",pib_basenamea(path),line,func);
  fflush(stderr);
  LeaveCriticalSection(&trace.cs); // ]
}

void pib_trace_vwriteln(const char *path, int line, const char *func,
    const char *format, ...)
{
  va_list ap;

  pib_trace_create();
  EnterCriticalSection(&trace.cs); // [
  va_start(ap,format);
  vfprintf(stderr,format,ap);
  va_end(ap);
  fprintf(stderr,". [%s:%d:%s]\n",pib_basenamea(path),line,func);
  fflush(stderr);
  LeaveCriticalSection(&trace.cs); // ]
}

void pib_trace_vwritelnw(const char *path, int line, const char *func,
    const wchar_t *format, ...)
{
  va_list ap;

  pib_trace_create();
  EnterCriticalSection(&trace.cs); // [
  va_start(ap,format);
  vfwprintf(stderr,format,ap);
  va_end(ap);
  fprintf(stderr,". [%s:%d:%s]\n",pib_basenamea(path),line,func);
  fflush(stderr);
  LeaveCriticalSection(&trace.cs); // ]
}

void pib_trace_markln(const char *path, int line, const char *func)
{
  pib_trace_create();
  EnterCriticalSection(&trace.cs); // [
  fprintf(stderr,"[%s:%d:%s]\n",pib_basenamea(path),line,func);
  fflush(stderr);
  LeaveCriticalSection(&trace.cs); // ]
}
#endif // ]
