/*
 * pib_widget.h
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#ifndef __PIB_WIDGET_H__ // [
#define __PIB_WIDGET_H__

#include <pib.h>

///////////////////////////////////////////////////////////////////////////////
#define PIB_WIDGET_REALIZER2
#if defined (PIB_WIDGET_REALIZER2) // [
#define PIB_WIDGET_REALIZER2_MORPH
#define PIB_WIDGET_REALIZER2_GEOMETRY
#define PIB_WIDGET_REALIZER_CORE_VMT
#endif // ]
#if ! defined (PIB_WIDGET_REALIZER2) // [
//#define PIB_WIDGET_INTITALIZER_ARRANGE
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
/*
 * we need to create a dialog by means of "DialogBoxIndirectParamW" in order
 * to have it always on top of it's parent window. unfortunately, it's close
 * to impossible to arrange the controls of the dialog by means of "dialog
 * box units (dlu)". that's why we have two phases in realizing a dialog:
 *
 * phase 1:  create the dialogs's controls all in the upper left corner (0,0)
 *   whith extension (1,1).
 * phaes 2:  initialize and arrange the the dialog's controls from within
 *   "DlgProc".
 */
//#define PIB_WIDGET_INTITALIZER_ARRANGE_CORRECT
#define PIB_WIDGET_INTITALIZER_POSITION_DOUBLE
#define PIB_WIDGET_PIXELS
#endif // ]
#endif // ]
//#define PIB_WIDGET_ID_PHYSICAL_EX
//#define PIB_WIDGET_GRID

///////////////////////////////////////////////////////////////////////////////
typedef struct _PibWidgetBuf PibWidgetBuf;
typedef struct _PibWidgetVmt PibWidgetVmt;
typedef struct _PibWidget PibWidget;
typedef enum _EPibWidgetMode EPibWidgetMode;
typedef struct _PibWidgetMode PibWidgetMode;
typedef struct _PibWidgetNode PibWidgetNode;
typedef struct _PibWidgetList PibWidgetList;
typedef struct _PibWidgetPhysical PibWidgetPhysical;
typedef const struct _PibWidgetComboboxMap PibWidgetComboboxMap;
typedef struct _PibWidgetCombobox PibWidgetCombobox;
#if defined (PIB_WIDGET_REALIZER2) // [
typedef struct _PibWidgetTreeview PibWidgetTreeview;
#endif // ]
typedef struct _PibWidgetBox PibWidgetBox;
#if defined (PIB_WIDGET_GRID) // [
typedef struct _PibWidgetGridEdge PibWidgetGridEdge;
typedef struct _PibWidgetGridVertex PibWidgetGridVertex;
typedef struct _PibWidgetGrid PibWidgetGrid;
#endif // ]
typedef struct _PibWidgetAcceptorVmt PibWidgetAcceptorVmt;
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE)// [
typedef enum _EPibWidgetPetzold EPibWidgetPetzold;
typedef struct _PibWidgetPetzold PibWidgetPetzold;
#endif // ]
#if defined (PIB_WIDGET_PIXELS) // [
typedef enum _EPibWidgetInitializer EPibWidgetInitializer;
#endif // ]
#if defined (PIB_WIDGET_REALIZER2) // [
typedef struct _PibWidgetTurtle PibWidgetTurtle;
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
typedef const struct _PibWidgetRealizerCoreVmt PibWidgetRealizerCoreVmt;
#endif // ]
typedef struct _PibWidgetRealizerCore PibWidgetRealizerCore;
#endif // ]
typedef struct _PibWidgetRealizer PibWidgetRealizer;
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
typedef struct _PibWidgetAcceptorParam PibWidgetAcceptorParam;
typedef struct _PibWidgetSizer PibWidgetSizer;
#endif // ]
#endif // ]
typedef struct _PibWidgetAcceptor PibWidgetAcceptor;
#if defined (PIB_WIDGET_REALIZER2) // [
typedef const struct _PibWidgetRealizerMap PibWidgetRealizerMap;
#endif // ]

///////////////////////////////////////////////////////////////////////////////
/*
 * NOTE: needs to be linked with a resource file at least containing a
 *   mainfest.xml!
 */
struct _PibWidgetBuf {
  size_t size;
  WORD idc;

  union {
    char *str;
    wchar_t *wcs;
    short *s;
    WORD *w;
    DWORD *dw;
    DLGTEMPLATE *head;
    DLGITEMTEMPLATE *item;
  };
};

void pib_widget_buf_head(
    PibWidgetBuf *buf,
    DWORD style,
    DWORD dwExtendedStyle,
    short x,
    short y,
    short cx,
    short cy,
    const wchar_t *menu,
    const wchar_t *cls,
    const wchar_t *title,
    WORD point_size,
    const wchar_t *type_face);
void pib_widget_buf_item(
    PibWidgetBuf *buf,
    DWORD style,
    DWORD dwExtendedStyle,
    short x,
    short y,
    short cx,
    short cy,
    const wchar_t *cls,
    const wchar_t *title);

///////////////////////////////////////////////////////////////////////////////
struct _PibWidgetVmt {
  const char *id;
  size_t size;
  const PibWidgetVmt *prev;
  void (*destroy)(PibWidget *widget);
#if defined (PIB_WIDGET_REALIZER2) // [
  void (*extent)(PibWidget *widget, PibWidgetAcceptor *acceptor, SIZE *size);
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
  void (*extenti)(PibWidget *widget, PibWidgetPetzold *petzold, SIZE *size);
#endif // ]
#if defined (PIB_WIDGET_PETZOLD2) // ] [
  void (*extent)(PibWidget *widget, PibWidgetPetzold *petzold, SIZE *size);
#elif defined (PIB_WIDGET_EXTENT_POINT_SIZE) // ] [
  void (*extent)(PibWidget *widget, int point_size, SIZE *size);
#else // ] [
  void (*extent)(PibWidget *widget, HDC hDc, int point_size, SIZE *size);
#endif // ]
#endif // ]
  SIZE *(*geometry)(PibWidget *widget);
  PibWidgetList *(*list)(PibWidget *widget);
  void (*accept)(PibWidget *widget, PibWidgetAcceptor *acceptor);
};

#define pib_widget_destroy(widget) \
  ((widget)->vmt->destroy(widget))
#define pib_widget_accept(widget,acceptor) \
  ((widget)->vmt->accept(widget,acceptor))

////////
struct _PibWidget {
#define _PIB_WIDGET \
  const PibWidgetVmt *vmt; \
  int count;
  _PIB_WIDGET
};

#define PIB_WIDGET(widget) ((PibWidget *)pib_widget_cast(widget, \
    "PIB_WIDGET",__FILE__,__LINE__,__func__))

PibWidgetVmt *pib_widget_vmt(void);
PibWidget *pib_widget_new(PibWidgetVmt *vmt);

PibWidget *pib_widget_ref(PibWidget *widget);
void pib_widget_unref(PibWidget *widget);

#if defined (PIB_WIDGET_REALIZER2) // [
int pib_widget_run(PibWidget *widget, int border, const wchar_t *title,
    WORD point_size, const wchar_t *type_face);
#else // ] [
int pib_widget_run(PibWidget *widget, const wchar_t *title, WORD point_size,
    const wchar_t *type_face);
#endif // ]
void *pib_widget_cast(void *widget, const char *id, const char *file,
    int line, const char *func);

///////////////////////////////////////////////////////////////////////////////
enum _EPibWidgetMode {
  PIB_WIDGET_NONE,
  PIB_WIDGET_BEGIN,
  PIB_WIDGET_CENTER,
  PIB_WIDGET_END,
};

struct _PibWidgetMode {
  EPibWidgetMode x,y;
};

struct _PibWidgetNode {
#define _PIB_WIDGET_NODE \
  _PIB_WIDGET \
  PibWidgetNode *prev,*next; \
  PibWidgetList *list; \
  PibWidgetMode mode; \
  PibWidget *ref;
  _PIB_WIDGET_NODE
};

#define PIB_WIDGET_ID_NODE \
  "PIB_WIDGET_NODE"
#define PIB_WIDGET_NODE(w) ((PibWidgetNode *)pib_widget_cast(w,\
    PIB_WIDGET_ID_NODE,__FILE__,__LINE__,__func__))

#define pib_widget_node_new(list,mx,my,ref) \
  pib_widget_node_new_ex(pib_widget_node_vmt(),list,mx,my,ref)
PibWidget *pib_widget_node_new_ex(PibWidgetVmt *vmt, PibWidgetList *list,
    EPibWidgetMode mx, EPibWidgetMode my, PibWidget *ref);
PibWidgetVmt *pib_widget_node_vmt(void);

struct _PibWidgetList {
#define _PIB_WIDGET_LIST \
  _PIB_WIDGET \
  int nchildren; \
  PibWidgetNode *head,*tail;
  _PIB_WIDGET_LIST
};

#define PIB_WIDGET_ID_LIST \
  "PIB_WIDGET_LIST"
#define PIB_WIDGET_LIST(w) ((PibWidgetList *)pib_widget_cast(w,\
  PIB_WIDGET_ID_LIST,__FILE__,__LINE__,__func__))

PibWidgetVmt *pib_widget_list_vmt(void);

PibWidget *pib_widget_list_new_ex(PibWidgetVmt *vmt);
PibWidget *pib_widget_list_new(void);

void pib_widget_list_append(PibWidgetList *list, EPibWidgetMode mx,
    EPibWidgetMode my, PibWidget *widget);

///////////////////////////////////////////////////////////////////////////////
/*
 * physical widgets com in several flavors:
 *   * button,
 *   * treeview, and
 *   * combobox.
 */
struct _PibWidgetPhysical {
#define _PIB_WIDGET_PHYSICAL \
  _PIB_WIDGET \
  SIZE geometry; \
  const wchar_t *title; \
  int idc; \
  HWND hWnd;
  _PIB_WIDGET_PHYSICAL
};

#define PIB_WIDGET_ID_PHYSICAL \
  "PIB_WIDGET_PHYSICAL"
#define PIB_WIDGET_PHYSICAL(w) ((PibWidgetPhysical *)pib_widget_cast(w,\
  PIB_WIDGET_ID_PHYSICAL,__FILE__,__LINE__,__func__))

PibWidgetVmt *pib_widget_physical_vmt(void);
PibWidget *pib_widget_physical_new(PibWidgetVmt *vmt, long cx, long cy,
    const wchar_t *title);

////////
PibWidgetVmt *pib_widget_button_vmt(void);

#define PIB_WIDGET_ID_BUTTON \
  "PIB_WIDGET_BUTTON"
#define PIB_WIDGET_BUTTON(w) ((PibWidgetPhysical *)pib_widget_cast(w,\
  PIB_WIDGET_ID_BUTTON,__FILE__,__LINE__,__func__))
#define pib_widget_button_new(title) \
  pib_widget_physical_new(pib_widget_button_vmt(),-1,-1,title)

////////
#if defined (PIB_WIDGET_REALIZER2) // [
struct _PibWidgetTreeview {
#define _PIB_WIDGET_TREEVIEW \
  _PIB_WIDGET_PHYSICAL \
  SIZE design;
  _PIB_WIDGET_TREEVIEW
};

PibWidgetVmt *pib_widget_treeview_vmt(void);
PibWidget *pib_widget_treeview_new(long cx, long cy);

#define PIB_WIDGET_ID_TREEVIEW \
  "PIB_WIDGET_TREEVIEW"
#define PIB_WIDGET_TREEVIEW(w) ((PibWidgetTreeview *)pib_widget_cast(w,\
    PIB_WIDGET_ID_TREEVIEW,__FILE__,__LINE__,__func__))
#else // ] [
PibWidgetVmt *pib_widget_treeview_vmt(void);

#define PIB_WIDGET_ID_TREEVIEW \
  "PIB_WIDGET_TREEVIEW"
#define PIB_WIDGET_TREEVIEW(w) ((PibWidgetPhysical *)pib_widget_cast(w,\
  PIB_WIDGET_ID_TREEVIEW,__FILE__,__LINE__,__func__))
#define pib_widget_treeview_new(cx,cy) \
  pib_widget_physical_new(pib_widget_treeview_vmt(),cx,cy,NULL)
#endif // ]

////////
struct _PibWidgetComboboxMap {
  const wchar_t *label;
  int id;
};

struct _PibWidgetCombobox {
#define _PIB_WIDGET_COMBOBOX \
  _PIB_WIDGET_PHYSICAL \
  PibWidgetComboboxMap *map;
  _PIB_WIDGET_COMBOBOX
};

PibWidgetVmt *pib_widget_combobox_vmt(void);
PibWidget *pib_widget_combobox_new(PibWidgetComboboxMap *map);

#define PIB_WIDGET_ID_COMBOBOX \
  "PIB_WIDGET_COMBOBOX"
#define PIB_WIDGET_COMBOBOX(w) ((PibWidgetCombobox *)pib_widget_cast(w,\
    PIB_WIDGET_ID_COMBOBOX,__FILE__,__LINE__,__func__))

////////
PibWidgetVmt *pib_widget_trackbar_vmt(void);

#define PIB_WIDGET_ID_TRACKBAR \
  "PIB_WIDGET_TRACKBAR"
#define PIB_WIDGET_TRACKBAR(w) ((PibWidgetPhysical *)pib_widget_cast(w,\
  PIB_WIDGET_ID_TRACKBAR,__FILE__,__LINE__,__func__))
#define pib_widget_trackbar_new(title,width) \
  pib_widget_physical_new(pib_widget_trackbar_vmt(),title,width,15)

////////
PibWidgetVmt *pib_widget_progress_vmt(void);

#define PIB_WIDGET_ID_PROGRESS \
  "PIB_WIDGET_PROGRESS"
#define PIB_WIDGET_PROGRESS(w) ((PibWidgetPhysicalEx *)pib_widget_cast(w,\
  PIB_WIDGET_ID_PROGRESS,__FILE__,__LINE__,__func__))
#define pib_widget_progress_new(title,width) \
  pib_widget_physical_ex_new(pib_widget_progress_vmt(),title,width,14)

///////////////////////////////////////////////////////////////////////////////
struct _PibWidgetBox {
#define _PIB_WIDGET_BOX \
  _PIB_WIDGET_LIST \
  int border,delta; \
  SIZE geometry; \
  int nvar;
  _PIB_WIDGET_BOX
};

////////
#define PIB_WIDGET_ID_BOX \
  "PIB_WIDGET_BOX"
#define PIB_WIDGET_BOX(w) ((PibWidgetBox *)pib_widget_cast(w,\
  PIB_WIDGET_ID_BOX,__FILE__,__LINE__,__func__))
#define pib_widget_box_append(box,mx,my,widget) \
  pib_widget_list_append(PIB_WIDGET_LIST(box),mx,my,widget)

PibWidgetVmt *pib_widget_box_vmt(void);
PibWidget *pib_widget_box_new(PibWidgetVmt *vmt, int border, int delta);

////////
PibWidgetVmt *pib_widget_hbox_vmt(void);

#define PIB_WIDGET_ID_HBOX \
  "PIB_WIDGET_HBOX"
#define PIB_WIDGET_HBOX(w) ((PibWidgetBox *)pib_widget_cast(w,\
  PIB_WIDGET_ID_HBOX,__FILE__,__LINE__,__func__))
#define pib_widget_hbox_new(border,delta) \
  pib_widget_box_new(pib_widget_hbox_vmt(),border,delta)

////////
PibWidgetVmt *pib_widget_vbox_vmt(void);

#define PIB_WIDGET_ID_VBOX \
  "PIB_WIDGET_VBOX"
#define PIB_WIDGET_VBOX(w) ((PibWidgetBox *)pib_widget_cast(w,\
  PIB_WIDGET_ID_VBOX,__FILE__,__LINE__,__func__))
#define pib_widget_vbox_new(border,delta) \
  pib_widget_box_new(pib_widget_vbox_vmt(),border,delta)

#if defined (PIB_WIDGET_GRID) // [
///////////////////////////////////////////////////////////////////////////////
struct _PibWidgetGridEdge {
#define _PIB_WIDGET_GRID_EDGE \
  _PIB_WIDGET_NODE
  _PIB_WIDGET_GRID_EDGE
};

struct _PibWidgetGridVertex {
#define _PIB_WIDGET_GRID_VERTEX \
  _PIB_WIDGET \
  PibWidgetGridEdge *left,*right; \
  PibWidgetGridEdge *prev,*next; \
  PibWidget *ref;
  _PIB_WIDGET
  _PIB_WIDGET_GRID_VERTEX
};

struct _PibWidgetGrid {
#define _PIB_WIDGET_GRID \
  _PIB_WIDGET \
  int border,delta; \
  SIZE geometry; \
  PibWidgetGridEdge *first,*last; \
  PibWidgetGridEdge *head,*tail;
  _PIB_WIDGET_GRID
};

PibWidgetGridVertex *pib_widget_graph_insert(PibWidgetGraph *graph,
    PibWidget *widget, PibWidgetVertex *prev, PibWidgetVertex *next,
    PibWidgetVertex *left, PibWidgetVertex *right);
#endif // ]

///////////////////////////////////////////////////////////////////////////////
struct _PibWidgetAcceptorVmt {
  const char *id;

  void (*destroy)(PibWidgetAcceptor *acceptor);

  void (*dispatch_button)(PibWidgetAcceptor *acceptor, PibWidget *widget);
  void (*dispatch_treeview)(PibWidgetAcceptor *acceptor, PibWidget *widget);
  void (*dispatch_combobox)(PibWidgetAcceptor *acceptor, PibWidget *widget);
  void (*dispatch_hbox)(PibWidgetAcceptor *acceptor, PibWidget *widget);
  void (*dispatch_vbox)(PibWidgetAcceptor *acceptor, PibWidget *widget);
};

////////
#if defined (PIB_WIDGET_PETZOLD1) // [
void pib_widget_petzold_justify(HDC hDc, LPCWSTR text, SIZE *extent);
#elif defined (PIB_WIDGET_INTITALIZER_ARRANGE) // ] [
enum _EPibWidgetPetzold {
  PIB_WIDGET_PETZOLD_LASTCHAR=127,
};

struct _PibWidgetPetzold {
  HWND hDlg;
  HFONT hFont;
  HDC hDc;

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE_CORRECT) // [
  struct {
    HBITMAP hBitmap;
    HDC hDc;
  } compatible;
#endif // ]
};

int pib_widget_petzold_create(PibWidgetPetzold *petzold, HWND hDlg,
    HFONT hFont);
void pib_widget_petzold_destroy(PibWidgetPetzold *petzold);

/*
 * should calculate the extent in pixels (*not* in dlu.)
 */
void pib_widget_petzold_extent(PibWidgetPetzold *petzold, LPCWSTR pText,
    SIZE *extent);
#elif defined (PIB_WIDGET_PETZOLD2) // ] [
enum _EPibWidgetPetzold {
  PIB_WIDGET_PETZOLD_LASTCHAR=127,
};

struct _PibWidgetPetzold {
#if defined (PIB_WIDGET_PETZOLD2_CREATE_COMPATIBLE) // [
  struct {
#if ! defined (PIB_WIDGET_INTITALIZER_ARRANGE_CORRECT) // [
    HDC hDc;
#endif // ]
    HBITMAP hBitmap;
  } screen;
#endif // ]

  HDC hDc;

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
  HWND hDlg;
#else // ] [
  struct {
    HFONT hCur;
    HFONT hPrev;
  } font;
#endif // ]

  double cy;
  double widths[PIB_WIDGET_PETZOLD_LASTCHAR+1];
};

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
int pib_widget_petzold_create(PibWidgetPetzold *petzold, HWND hDlg);
#else // ] [
int pib_widget_petzold_create(PibWidgetPetzold *petzold, int point_size,
    const wchar_t *type_face);
#endif // ]
void pib_widget_petzold_destroy(PibWidgetPetzold *petzold);

void pib_widget_petzold_extent(PibWidgetPetzold *petzold, LPCWSTR pText,
    SIZE *extent);
void pib_widget_petzold_justify(PibWidgetPetzold *petzold, LPCWSTR pText,
    SIZE *extent);
#endif // ]

////////
#if defined (PIB_WIDGET_PIXELS) // [
enum _EPibWidgetInitializer {
  PIB_WIDGET_INITIALIZER_UNIT=1000,
};
#endif // ]

#if defined (PIB_WIDGET_REALIZER2) // [
struct _PibWidgetTurtle {
  double x,y;
};

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
struct _PibWidgetRealizerCoreVmt {
  void (*destroy)(PibWidgetRealizerCore *core);
  int (*realizing)(PibWidgetAcceptor *acceptor);
  void (*dispatch_treeview)(PibWidgetAcceptor *acceptor, PibWidget *widget);
  void (*dispatch_combobox)(PibWidgetAcceptor *acceptor, PibWidget *widget);
  void (*dispatch_button)(PibWidgetAcceptor *acceptor, PibWidget *widget);
};
#endif // ]

struct _PibWidgetRealizerCore {
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  PibWidgetRealizerCoreVmt *vmt;
#endif // ]
#if defined (PIB_WIDGET_REALIZER2_GEOMETRY) // [
  SIZE geometry;
#endif // ]
  double border;
  HWND hDlg;
  HDC hDc;

  struct {
    HFONT hCur,hPrev;
  } font;

  struct {
    double cx;
    double cy;
  } unit;

  PibWidgetTurtle turtle;
};

#if 0 && defined (PIB_WIDGET_REALIZER2_GEOMETRY) // [
int pib_widget_realizer_core_create(PibWidgetRealizerCore *core, long cx,
    long cy, long border, WORD point_size, const wchar_t *type_face);
#else // ] [
int pib_widget_realizer_core_create(PibWidgetRealizerCore *core, long border,
    WORD point_size, const wchar_t *type_face);
#endif // ]
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
void pib_widget_realizer_core_dispatch_box(PibWidgetAcceptor *acceptor,
    PibWidget *widget, PibWidgetRealizerMap *map);
#else // ] [
void pib_widget_realizer_core_destroy(PibWidgetRealizerCore *core);
#endif // ]

#if defined (PIB_WIDGET_REALIZER2_GEOMETRY) // [
int pib_widget_sizer_core_create(PibWidgetRealizerCore *core, long cx,
    long cy, long border, HWND hDlg, HFONT hFont);
#else // ] [
int pib_widget_sizer_core_create(PibWidgetRealizerCore *core, long border,
    HWND hDlg, HFONT hFont);
#endif // ]
#if ! defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
void pib_widget_sizer_core_destroy(PibWidgetRealizerCore *core);
#endif // ]
#endif // ]

struct _PibWidgetRealizer {
#if ! defined (PIB_WIDGET_REALIZER2_MORPH) // [
  double border;
#endif // ]
  const wchar_t *title;
  WORD point_size;
  const wchar_t *type_face;
#if ! defined (PIB_WIDGET_REALIZER2_MORPH) // [
  HDC hDc;

  struct {
    HFONT hCur,hPrev;
  } font;

  struct {
    double cx;
    double cy;
  } unit;

  PibWidgetTurtle turtle;
#endif // ]
  PibWidgetBuf buf;
  DLGTEMPLATE *head;
};
#endif // ]

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
struct _PibWidgetAcceptorParam {
  int border;
  PibWidget *root;
};

struct _PibWidgetSizer {
  PibWidget *root;
  int realizing;
};
#endif // ]

struct _PibWidgetAcceptor {
  const PibWidgetAcceptorVmt *vmt;
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetRealizerCore core;
#endif // ]

  union {
#if defined (PIB_WIDGET_REALIZER2) // [
    PibWidgetRealizer realizer;
#else // ] [
    struct {
      const wchar_t *title;
      WORD point_size;
      const wchar_t *type_face;
      PibWidgetBuf *buf;
      long x,y;

#if defined (PIB_WIDGET_PETZOLD2) // [
      PibWidgetPetzold petzold;
#elif ! defined (PIB_WIDGET_EXTENT_POINT_SIZE) // [
      struct {
        HFONT hCur;
        HFONT hPrev;
      } font;

      HDC hDc;
#endif // ]
      SIZE bounds;

      struct {
        PibWidgetNode *node[20];
        int tos;
      } stack;
    } realizer;
#endif // ]

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    PibWidgetSizer sizer;
#else // ] [
    struct {
#if ! defined (PIB_WIDGET_REALIZER2) // [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
      PibWidgetPetzold petzold;
      int realizing;
      SIZE bounds;
#if defined (PIB_WIDGET_PIXELS) // [
      SIZE unit;
#endif // ]
#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
      double x,y;
#else // ] [
      long x,y;
#endif // ]

      struct {
        PibWidgetNode *node[20];
        int tos;
      } stack;
#endif // ]
#endif // ]
      HWND hDlg;
    } initializer;
#endif // ]
  };
};

#if defined (PIB_WIDGET_REALIZER2) // [
int pib_widget_realizer_create(PibWidgetAcceptor *acceptor, long border,
    const wchar_t *title, WORD point_size, const wchar_t *type_face);
void pib_widget_realizer_phase1(PibWidgetAcceptor *acceptor,
    PibWidget *widget);
void pib_widget_realizer_phase2(PibWidgetAcceptor *acceptor,
    PibWidget *widget);
#else // ] [
void pib_widget_realizer_create(PibWidgetAcceptor *acceptor,
    const wchar_t *title, WORD point_size, const wchar_t *type_face,
    PibWidgetBuf *buf);
void pib_widget_realizer_reset(PibWidgetAcceptor *acceptor);
#endif // ]

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if defined (PIB_WIDGET_REALIZER2_GEOMETRY) // [
int pib_widget_sizer_create(PibWidgetAcceptor *acceptor, long cx, long cy,
    long border, HWND hDlg, HFONT hFont, PibWidget *root);
#else // ] [
int pib_widget_sizer_create(PibWidgetAcceptor *acceptor, int border,
    HWND hDlg, HFONT hFont, PibWidget *root);
#endif // ]

void pib_widget_sizer_phase1(PibWidgetAcceptor *acceptor, PibWidget *widget);
void pib_widget_sizer_phase2(PibWidgetAcceptor *acceptor, PibWidget *widget);
#else // ] [
void pib_widget_initializer_create(PibWidgetAcceptor *acceptor, HWND hDlg,
    HFONT hFont);
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
void pib_widget_initializer_reset(PibWidgetAcceptor *acceptor);
#endif // ]
#endif // ]

#if defined (PIB_WIDGET_REALIZER2) // [
///////////////////////////////////////////////////////////////////////////////
struct _PibWidgetRealizerMap {
  double *(*tx)(PibWidgetTurtle *turtle);
  double *(*ty)(PibWidgetTurtle *turtle);
  long *(*cx)(SIZE *size);
  long *(*cy)(SIZE *size);
  EPibWidgetMode *(*mx)(PibWidgetMode *mode);
  EPibWidgetMode *(*my)(PibWidgetMode *mode);
};
#endif // ]

///////////////////////////////////////////////////////////////////////////////
/*
 * KnowledgeBase Archive
 * An Archive of Early Microsoft KnowledgeBase Articles
 *
 * Q94646: INFO: Using GetCharABCWidths() to Calculate Text Extents
 *   https://jeffpar.github.io/kbarchive/kb/094/Q94646/
 */
BOOL NEAR PASCAL IsBoldSimulated(HDC hDC);
UINT WINAPI GetTextExtentABCPointW(HDC hDC, LPCWSTR lpsz, const int cb,
    LPSIZE lpSize);

#endif // ]
