/*
 * pib_widget.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

//#define PIB_DEBUG_CAST
PibWidget *pib_widget_new(PibWidgetVmt *vmt)
{
  PibWidget *widget=calloc(1,vmt->size);

  if (!widget) {
    DMESSAGE("allocating widget");
    goto e_widget;
  }

  widget->vmt=vmt;
  widget->count=0;

  return widget;
//cleanup:
  free(widget);
e_widget:
  return NULL;
}

PibWidget *pib_widget_ref(PibWidget *widget)
{
  ++widget->count;

  return widget;
}

void pib_widget_unref(PibWidget *widget)
{
  if (!widget->count) {
DWRITELN("something went wrong");
  }
  else if (!--widget->count)
    widget->vmt->destroy(widget);
}

void *pib_widget_cast(void *widget, const char *id, const char *file,
    int line, const char *func)
{
  const PibWidgetVmt *vmt=widget?((PibWidget *)widget)->vmt:NULL;

  while (vmt) {
#if defined (PIB_DEBUG_CAST) // [
DVWRITELN("\"%s\" \"%s\"",vmt->id,id);
#endif // ]
    if (!strcmp(vmt->id,id))
      return widget;

    vmt=vmt->prev;
  }

  DVMESSAGE("illegal cast: %s:%d:%s",file,line,func);

  return NULL;
}

static void _pib_widget_destroy(PibWidget *widget)
{
DVWRITELN("~%s",widget->vmt->id);
  free(widget);
}

#if defined (PIB_WIDGET_REALIZER2) // [
static void _pib_widget_extent(PibWidget *widget, PibWidgetAcceptor *acceptor,
    SIZE *size)
{
  DVMESSAGE("method not available for %s",widget->vmt->id);
}
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
static void _pib_widget_extenti(PibWidget *widget, PibWidgetPetzold *petzold,
    SIZE *size)
{
  DVMESSAGE("method not available for %s",widget->vmt->id);
}
#endif // ]

#if defined (PIB_WIDGET_PETZOLD2) // [
static void _pib_widget_extent(PibWidget *widget, PibWidgetPetzold *petzold,
    SIZE *size)
#elif defined (PIB_WIDGET_EXTENT_POINT_SIZE) // ] [
static void _pib_widget_extent(PibWidget *widget, int point_size, SIZE *size)
#else // ] [
static void _pib_widget_extent(PibWidget *widget, HDC hDc, int point_size,
    SIZE *size)
#endif // ]
{
  DVMESSAGE("method not available for %s",widget->vmt->id);
}
#endif // ]

static SIZE *_pib_widget_geometry(PibWidget *widget)
{
  DVMESSAGE("method not available for %s",widget->vmt->id);

  return NULL;
}

static void _pib_widget_accept(PibWidget *widget, PibWidgetAcceptor *acceptor)
{
  DVMESSAGE("%s not available for %s",acceptor->vmt->id,widget->vmt->id);
}

static PibWidgetList *_pib_widget_list(PibWidget *widget)
{
  return NULL;
}

PibWidgetVmt *pib_widget_vmt(void)
{
  static PibWidgetVmt vmt;

  if (!vmt.destroy) {
    vmt.id="PIB_WIDGET";
    vmt.size=sizeof (PibWidget);
    vmt.destroy=_pib_widget_destroy;
#if defined (PIB_WIDGET_REALIZER2) // [
    vmt.extent=_pib_widget_extent;
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
    vmt.extenti=_pib_widget_extenti;
#endif // ]
    vmt.extent=_pib_widget_extent;
#endif // ]
    vmt.geometry=_pib_widget_geometry;
    vmt.list=_pib_widget_list;
    vmt.accept=_pib_widget_accept;
  }

  return &vmt;
}
