/*
 * pib_widget_buf.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

#if 0 // [
static short *pib_widget_buf_short_inc(PibWidgetBuf *buf)
{
  char *str=buf->str;
  short *s=buf->s;

  if (str) {
    ++buf->s;
    buf->size+=buf->str-str;
  }
  else
    buf->size+=sizeof *buf->s;

  return s;
}
#endif // ]

static WORD *pib_widget_buf_word_inc(PibWidgetBuf *buf)
{
  char *str=buf->str;
  WORD *w=buf->w;

  if (str) {
    ++buf->w;
    buf->size+=buf->str-str;
  }
  else
    buf->size+=sizeof *buf->w;

  return w;
}

#if 0 // [
static DWORD *pib_widget_buf_dword_inc(PibWidgetBuf *buf)
{
  char *str=buf->str;
  DWORD *dw=buf->dw;

  if (str) {
    ++buf->dw;
    buf->size+=buf->str-str;
  }
  else
    buf->size+=sizeof *buf->dw;

  return dw;
}
#endif // ]

static DLGTEMPLATE *pib_widget_buf_head_inc(PibWidgetBuf *buf)
{
  char *str=buf->str;
  DLGTEMPLATE *head=buf->head;

  if (str) {
    ++buf->head;
    buf->size+=buf->str-str;
  }
  else
    buf->size+=sizeof *buf->head;

  return head;
}

static DLGITEMTEMPLATE *pib_widget_buf_item_inc(PibWidgetBuf *buf)
{
  char *str=buf->str;
  DLGITEMTEMPLATE *item=buf->item;

  if (str) {
    ++buf->item;
    buf->size+=buf->str-str;
  }
  else
    buf->size+=sizeof *buf->item;

  return item;
}

static void pib_widget_buf_align(PibWidgetBuf *buf, size_t size)
{
  size_t delta=buf->size%size;

  if (delta) {
    delta=size-delta;
    buf->size+=delta;

    if (buf->str)
      buf->str+=delta;
  }
}

static void pib_widget_buf_wcs(PibWidgetBuf *buf, const wchar_t *wcs,
    const char *id)
{
  char *str;
  size_t size;

  if (buf->str) {
    str=buf->str;

    if (wcs) {
      wcscpy(buf->wcs,wcs);
      buf->wcs+=wcslen(wcs)+1;
    }
    else
      *buf->wcs++=L'\0';

    size=buf->str-str;
  }
  else
    size=((wcs?wcslen(wcs):0)+1)*sizeof *buf->wcs;

  buf->size+=size;
}

void pib_widget_buf_head(
    PibWidgetBuf *buf,
    DWORD style,
    DWORD dwExtendedStyle,
    short x,
    short y,
    short cx,
    short cy,
    const wchar_t *menu,
    const wchar_t *cls,
    const wchar_t *title,
    WORD point_size,
    const wchar_t *typeface)
{
  enum { IDC_MIN=4000 };

  if (!buf->str)
    buf->idc=IDC_MIN;

  buf->size=0;

  if ((point_size&&typeface)||(DS_SHELLFONT&style))
    style|=DS_SETFONT;
  else
    style&=~DS_SETFONT;

  if (buf->str) {
    buf->head->style=style;
    buf->head->dwExtendedStyle=dwExtendedStyle;
    buf->head->cdit=buf->idc-IDC_MIN;
    buf->head->x=x;
    buf->head->y=y;
    buf->head->cx=cx;
    buf->head->cy=cy;
  }

  buf->idc=IDC_MIN;
  pib_widget_buf_head_inc(buf);
  /*
   * The menu, class, title, and font arrays must be aligned on WORD
   * boundaries.
   *
   * Immediately following the DLGTEMPLATE structure is a menu array that
   * identifies a menu resource for the dialog box. If the first element
   * of this array is 0x0000, the dialog box has no menu and the array has
   * no other elements.
   */
  pib_widget_buf_align(buf,sizeof(WORD));
  pib_widget_buf_wcs(buf,menu,"head::menu");
  /*
   * Following the menu array is a class array that identifies the window
   * class of the control. If the first element of the array is 0x0000,
   * the system uses the predefined dialog box class for the dialog box
   * and the array has no other elements. 
   */
  pib_widget_buf_align(buf,sizeof(WORD));
  pib_widget_buf_wcs(buf,cls,"head::class");
  /*
   * Following the class array is a title array that specifies a
   * null-terminated Unicode string that contains the title of the dialog
   * box. If the first element of this array is 0x0000, the dialog box
   * has no title and the array has no other elements.
   */
  pib_widget_buf_align(buf,sizeof(WORD));
  pib_widget_buf_wcs(buf,title,"head::title");
  /*
   * The 16-bit point size value and the typeface array follow the title
   * array, but only if the style member specifies the DS_SETFONT style.
   * The point size value specifies the point size of the font to use for
   * the text in the dialog box and its controls. The typeface array is a
   * null-terminated Unicode string specifying the name of the typeface
   * for the font.
   */
  if (point_size&&typeface) {
    if (buf->str)
      *pib_widget_buf_word_inc(buf)=point_size;
    else
      pib_widget_buf_word_inc(buf);

    pib_widget_buf_align(buf,sizeof(WORD));
    pib_widget_buf_wcs(buf,typeface,"head::font");
  }
  else if (DS_SHELLFONT&style) {
    /*
     * Why does DS_SHELLFONT = DS_FIXEDSYS | DS_SETFONT?
     *   https://devblogs.microsoft.com/oldnewthing/?p=36513
     * #define DS_SHELLFONT        (DS_SETFONT | DS_FIXEDSYS)
     *
     * It might be a mysterious coincidence that DS_FIXEDSYS exactly gives
     * the geometry we'd like to have but if Raymond's saying ...
     */
    if (buf->str)
      *pib_widget_buf_word_inc(buf)=DS_FIXEDSYS;
    else
      pib_widget_buf_word_inc(buf);

    /*
     * The typeface array is a null-terminated Unicode string specifying
     * the name of the typeface for the font. 
     */
    pib_widget_buf_align(buf,sizeof(WORD));
    pib_widget_buf_wcs(buf,L"MS Shell Dlg","head::title");
  }
}

void pib_widget_buf_item(
    PibWidgetBuf *buf,
    DWORD style,
    DWORD dwExtendedStyle,
    short x,
    short y,
    short cx,
    short cy,
    const wchar_t *cls,
    const wchar_t *title)
{
  /*
   * Each DLGITEMTEMPLATE structure in the template must be aligned on a
   * DWORD boundary. 
   */
  pib_widget_buf_align(buf,sizeof(DWORD));

  if (buf->str) {
    buf->item->style= style;
    buf->item->dwExtendedStyle=dwExtendedStyle;
    buf->item->x=x;
    buf->item->y=y;
    buf->item->cx=cx;
    buf->item->cy=cy;
    buf->item->id=buf->idc;
  }

  pib_widget_buf_item_inc(buf);
  /*
   * The class and title arrays must be aligned on WORD boundaries. The
   * creation data array must be aligned on a WORD boundary.
   *
   * Immediately following each DLGITEMTEMPLATE structure is a class array
   * that specifies the window class of the control. If the first element
   * of this array is any value other than 0xFFFF, the system treats the
   * array as a null-terminated Unicode string that specifies the name of
   * a registered window class.
   */
  pib_widget_buf_align(buf,sizeof(WORD));
  pib_widget_buf_wcs(buf,cls,"item:class");
  /*
   * Following the class array is a title array that contains the initial
   * text or resource identifier of the control.
   */
  pib_widget_buf_align(buf,sizeof(WORD));
  pib_widget_buf_wcs(buf,title,"item:title");
  /*
   * The creation data array begins at the next WORD boundary after the
   * title array.
   */
  pib_widget_buf_align(buf,sizeof(WORD));
  pib_widget_buf_wcs(buf,NULL,"item:data");
  ++buf->idc;
}
