/*
 * pib_widget_button.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

#if defined (PIB_WIDGET_REALIZER2) // [
static void _pib_widget_button_extent(PibWidget *widget,
    PibWidgetAcceptor *acceptor, SIZE *size)
{
  const wchar_t *title=PIB_WIDGET_PHYSICAL(widget)->title;
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  double cx=(50.0-4.0)*acceptor->core.unit.cx/4;
  double cy=14.0*acceptor->core.unit.cy/8;
#else // ] [
  double cx=(50.0-4.0)*acceptor->realizer.unit.cx/4;
  double cy=14.0*acceptor->realizer.unit.cy/8;
#endif // ]

#if 0 // [
  GetTextExtentABCPointW(realizer->hDc,title,wcslen(title),size);
#else // ] [
  GetTextExtentPoint32W(
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    acceptor->core.hDc,       // HDC     hdc,
#else // ] [
    acceptor->realizer.hDc,   // HDC     hdc,
#endif // ]
    title,                    // LPCWSTR lpString,
    wcslen(title),            // int     c,
    size                      // LPSIZE  psizl
  );
#endif // ]

  if (size->cx<cx+0.5)
    size->cx=cx+0.5;
  else {
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    size->cx=(double)size->cx+4.0*acceptor->core.unit.cx/4+0.5;
#else // ] [
    size->cx=(double)size->cx+4.0*acceptor->realizer.unit.cx/4+0.5;
#endif // ]
  }

  size->cy=cy+0.5;
}
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
#if 0 // [
/*
 * Reverse MapDialogRect()
 *   https://groups.google.com/forum/#!topic/comp.os.ms-windows.programmer.win32/fT3gXGbwISc
 */
void MapDialogRectReverse(HWND hDlg, RECT *rect)
{
  // First we get the dialog units for your dialog box
  RECT calibrate;

  calibrate.left=0;
  calibrate.top=0;
  calibrate.right=4;
  calibrate.bottom=8;
  MapDialogRect(hDlg,&calibrate);
#if 0 // [
  baseunitX=calibrate.right;
  baseunitY=calibrate.bottom;
#endif // ]

  // Now we do the conversion
  rect->left=MulDiv(rect->left,4,calibrate.right);
  rect->right=MulDiv(rect->right,4,calibrate.right);
  rect->top=MulDiv(rect->top, 8,calibrate.bottom);
  rect->bottom=MulDiv(rect->bottom,8,calibrate.bottom);
}
#endif // ]

static void _pib_widget_button_extenti(PibWidget *widget,
    PibWidgetPetzold *petzold, SIZE *size)
{
#if defined (DEBUG) // [
  HFONT hFont;
  LOGFONTW lf;
#endif // ]
  RECT rect;

#if defined (DEBUG) // [
  /*
   * The GetCurrentObject function retrieves a handle to an object of
   * the specified type that has been selected into the specified device
   * context (DC).
   */
  hFont=GetCurrentObject(
    petzold->hDc,           // HDC  hdc,
    OBJ_FONT                // UINT type
  );

  /*
   * The GetObject function retrieves information for the specified
   * graphics object.
   */
  GetObjectW (
     hFont,                 // HANDLE h,
     sizeof (LOGFONTW),     // int    c,
     &lf                    // LPVOID pv
  );
DVWRITELNW(">>> \"%s\"",lf.lfFaceName);
#endif // ]

  pib_widget_petzold_extent(petzold,PIB_WIDGET_PHYSICAL(widget)->title,size);

  /*
   * the minimum button geometry.
   */
  rect.left=0;              // DLU
  rect.top=0;               // DLU
  rect.right=50;            // DLU
  rect.bottom=14;           // DLU

  /*
   * The MapDialogRect function assumes that the initial coordinates in
   * the RECT structure represent dialog box units. 
   */
  MapDialogRect(
    petzold->hDlg,          // HWND   hDlg,
    &rect                   // LPRECT lpRect
  );

  if (size->cx<rect.right-rect.left)
    size->cx=rect.right-rect.left;

  size->cy=rect.bottom-rect.top;
}
#endif // ]

#if defined (PIB_WIDGET_PETZOLD2) // [
static void _pib_widget_button_extent(PibWidget *widget,
    PibWidgetPetzold *petzold, SIZE *size)
#elif defined (PIB_WIDGET_EXTENT_POINT_SIZE) // ] [
static void _pib_widget_button_extent(PibWidget *widget, int point_size,
    SIZE *size)
#else // ] [
static void _pib_widget_button_extent(PibWidget *widget, HDC hDc,
    int point_size, SIZE *size)
#endif // ]
{
  PibWidgetPhysical *physical=PIB_WIDGET_PHYSICAL(widget);
#if defined (PIB_WIDGET_PETZOLD2) // [
  pib_widget_petzold_extent(petzold,physical->title,size);

DVWRITELNW("\"%s\" cx:%ld",physical->title,size->cx);
  if (size->cx<50)
    size->cx=50;

  size->cy=14;
#else // ] [
  int len=wcslen(physical->title);

#if defined (PIB_WIDGET_EXTENT_POINT_SIZE) // [
  if (point_size<10)
    ++len;

  len<<=2;
  size->cx=len<48?50:len+2;
  size->cy=14;
#else // ] [
  SIZE extent;

  GetTextExtentPoint32W(
    hDc,                      // HDC     hdc,
    physical->title,          // LPCWSTR lpString,
    len,                      // int     c,
    &extent                   // LPSIZE  psizl
  );
DVWRITELNW("*** map \"%s\": %d -> %d",physical->title,len<<2,extent.cx);

  size->cx=extent.cx<48?50:extent.cx+2;
  size->cy=14;
#endif // ]
#endif // ]
}
#endif // ]

static void _pib_widget_button_accept(PibWidget *widget,
    PibWidgetAcceptor *acceptor)
{
  acceptor->vmt->dispatch_button(acceptor,widget);
}

PibWidgetVmt *pib_widget_button_vmt(void)
{
  static PibWidgetVmt vmt;
  PibWidgetVmt *prev;

  if (!vmt.destroy) {
    prev=pib_widget_physical_vmt();
    vmt=*prev;
    vmt.id=PIB_WIDGET_ID_BUTTON;
    vmt.prev=prev;
#if defined (PIB_WIDGET_REALIZER2) // [
    vmt.extent=_pib_widget_button_extent;
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
    vmt.extenti=_pib_widget_button_extenti;
#endif // ]
    vmt.extent=_pib_widget_button_extent;
#endif // ]
    vmt.accept=_pib_widget_button_accept;
  }

  return &vmt;
}
