/*
 * pib_widget_combobox.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

PibWidget *pib_widget_combobox_new(PibWidgetComboboxMap *map)
{
  PibWidget *widget=pib_widget_physical_new(pib_widget_combobox_vmt(),
      -1,-1,NULL);

  PIB_WIDGET_COMBOBOX(widget)->map=map;

  return widget;
}

#if defined (PIB_WIDGET_REALIZER2) // [
static void _pib_widget_combobox_extent(PibWidget *widget,
    PibWidgetAcceptor *acceptor, SIZE *size)
{
  PibWidgetComboboxMap *map;
  SIZE extent;

  size->cx=0;
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if 0 // [
  size->cy=13.0*acceptor->core.unit.cy/8+0.5;
#else // ] [
  size->cy=0;//13.0*acceptor->core.unit.cy/8+0.5;
#endif // ]
#else // ] [
  size->cy=13.0*acceptor->realizer.unit.cy/8;
#endif // ]

  if (PIB_WIDGET_COMBOBOX(widget)->map) {
    for (map=PIB_WIDGET_COMBOBOX(widget)->map;map->label;++map) {
      GetTextExtentPoint32W(
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
        acceptor->core.hDc,       // HDC     hdc,
#else // ] [
        acceptor->realizer.hDc,   // HDC     hdc,
#endif // ]
        map->label,               // LPCWSTR lpString,
        wcslen(map->label),       // int     c,
        &extent                   // LPSIZE  psizl
      );

      if (size->cx<extent.cx)
        size->cx=extent.cx;

      if (size->cy<extent.cy)
        size->cy=extent.cy;
    }
  }

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if 1 // [
  /*
   * Not that I am aware of as the width of a vertical scroll bar is a
   * system-wide defined constant. Use
   *   GetSystemMetrics(SM_CXVSCROLL);
   * to obtain the value.
   *
   * http://forums.codeguru.com/showthread.php?542405-Combo-Box-Height
   */
  size->cx=(double)size->cx+1.5*GetSystemMetrics(SM_CXVSCROLL)+0.5;
#else // ] [
  size->cx=(double)size->cx+16.0*acceptor->core.unit.cx/4+0.5;
#endif // ]
  size->cy=(double)extent.cy+5.0*acceptor->core.unit.cy/8+0.5;
#else // ] [
  size->cx=(double)size->cx+16.0*acceptor->realizer.unit.cx/4+0.5;
#endif // ]
}
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
static void _pib_widget_combobox_extenti(PibWidget *widget,
    PibWidgetPetzold *petzold, SIZE *size)
{
  PibWidgetComboboxMap *map;
  SIZE extent;

  size->cx=0;
  size->cy=13;

  if (PIB_WIDGET_COMBOBOX(widget)->map) {
    for (map=PIB_WIDGET_COMBOBOX(widget)->map;map->label;++map) {
      pib_widget_petzold_extent(petzold,map->label,&extent);

      if (size->cx<extent.cx)
        size->cx=extent.cx;
    }
  }

  size->cx+=14;
}
#endif // ]

#if defined (PIB_WIDGET_PETZOLD2) // [
static void _pib_widget_combobox_extent(PibWidget *widget,
    PibWidgetPetzold *petzold, SIZE *size)
#elif defined (PIB_WIDGET_EXTENT_POINT_SIZE) // [
static void _pib_widget_combobox_extent(PibWidget *widget, int point_size,
    SIZE *size)
#else // ] [
static void _pib_widget_combobox_extent(PibWidget *widget, HDC hDc,
    int point_size, SIZE *size)
#endif // ]
{
#if ! defined (PIB_WIDGET_PETZOLD2) // [
  int max=0,len;
#endif // ]
  PibWidgetCombobox *combobox=PIB_WIDGET_COMBOBOX(widget);
  PibWidgetComboboxMap *map;
#if defined (PIB_WIDGET_PETZOLD2) \
    || ! defined (PIB_WIDGET_EXTENT_POINT_SIZE) // [
  SIZE extent;
#endif // ]

  size->cx=0;
  size->cy=0;

  if (combobox->map) {
    for (map=combobox->map;map->label;++map) {
#if defined (PIB_WIDGET_PETZOLD2) // [
      pib_widget_petzold_extent(petzold,map->label,&extent);

      if (size->cx<extent.cx)
        size->cx=extent.cx;

      if (size->cy<extent.cy)
        size->cy=extent.cy;
#else // ] [
      len=wcslen(map->label);

      if (max<len) {
        max=len;

#if ! defined (PIB_WIDGET_EXTENT_POINT_SIZE) // [
        GetTextExtentPoint32W(
          hDc,              // HDC     hdc,
          map->label,       // LPCWSTR lpString,
          len,              // int     c,
          &extent           // LPSIZE  psizl
        );

      	if (size->cx<extent.cx)
        	size->cx=extent.cx;

      	if (size->cy<extent.cy)
        	size->cy=extent.cy;
DVWRITELNW("*** map \"%s\": %d -> %d",map->label,len<<2,extent.cx);
#endif // ]
      }
#endif // ]
    }

  }

#if defined (PIB_WIDGET_PETZOLD2) // [
  size->cx+=14;
  size->cy=13;
DVWRITELN("cy:%ld",size->cy);
#elif defined (PIB_WIDGET_EXTENT_POINT_SIZE) // ] [
  if (point_size<10)
    ++max;

  max<<=2;
  size->cx=14+max;
  size->cy=13;
#else // ] [
  size->cx=14+extent.cx;
  size->cy=14;
#endif // ]
}
#endif // ]

static void _pib_widget_combobox_accept(PibWidget *widget,
    PibWidgetAcceptor *acceptor)
{
  acceptor->vmt->dispatch_combobox(acceptor,widget);
}

PibWidgetVmt *pib_widget_combobox_vmt(void)
{
  static PibWidgetVmt vmt;
  PibWidgetVmt *prev;

  if (!vmt.destroy) {
    prev=pib_widget_physical_vmt();
    vmt=*prev;
    vmt.id=PIB_WIDGET_ID_COMBOBOX;
    vmt.size=sizeof (PibWidgetCombobox);
    vmt.prev=prev;
#if defined (PIB_WIDGET_REALIZER2) // [
    vmt.extent=_pib_widget_combobox_extent;
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
    vmt.extenti=_pib_widget_combobox_extenti;
#endif // ]
    vmt.extent=_pib_widget_combobox_extent;
#endif // ]
    vmt.accept=_pib_widget_combobox_accept;
  }

  return &vmt;
}
