/*
 * pib_widget_initializer.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

#if ! defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
typedef const struct _PibWidgetRealizerMap PibWidgetRealizerMap;

struct _PibWidgetRealizerMap {
#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
  double *(*x)(PibWidgetAcceptor *acceptor);
  double *(*y)(PibWidgetAcceptor *acceptor);
#else // ] [
  long *(*x)(PibWidgetAcceptor *acceptor);
  long *(*y)(PibWidgetAcceptor *acceptor);
#endif // ]
  long *(*cx)(SIZE *size);
  long *(*cy)(SIZE *size);
  EPibWidgetMode *(*mx)(PibWidgetMode *mode);
  EPibWidgetMode *(*my)(PibWidgetMode *mode);
};

static PibWidgetRealizerMap pib_map_affine;
static PibWidgetRealizerMap pib_map_twisted;
#endif // ]
static PibWidgetAcceptorVmt vmt;

void pib_widget_initializer_create(PibWidgetAcceptor *acceptor, HWND hDlg,
    HFONT hFont)
{
#if defined (PIB_WIDGET_PIXELS) // [
  RECT rect;
#endif // ]

  memset(acceptor,0,sizeof *acceptor);
  acceptor->vmt=&vmt;
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
#if defined (PIB_WIDGET_PIXELS) // [
  rect.left=0;
  rect.top=0;
  rect.right=PIB_WIDGET_INITIALIZER_UNIT;
  rect.bottom=PIB_WIDGET_INITIALIZER_UNIT;

  MapDialogRect(
    hDlg,       // HWND   hDlg,
    &rect       // LPRECT lpRect
  );

  acceptor->initializer.unit.cx=rect.right-rect.left;
  acceptor->initializer.unit.cy=rect.bottom-rect.top;
DVWRITELN("unit.cx:%ld unit.cy:%ld",acceptor->initializer.unit.cx,acceptor->initializer.unit.cy);
#endif // ]
  pib_widget_petzold_create(&acceptor->initializer.petzold,hDlg,hFont);
#else // ] [
  acceptor->initializer.hDlg=hDlg;
#endif // ]
}

static void pib_widget_initializer_destroy(PibWidgetAcceptor *acceptor)
{
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
  pib_widget_petzold_destroy(&acceptor->initializer.petzold);
#endif // ]
}

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
void pib_widget_initializer_reset(PibWidgetAcceptor *acceptor)
{
DWRITELN("==================================");
  if (!acceptor->initializer.realizing)
    ++acceptor->initializer.realizing;

  acceptor->initializer.bounds.cx=acceptor->initializer.x;
  acceptor->initializer.bounds.cy=acceptor->initializer.y;
  acceptor->initializer.x=0;
  acceptor->initializer.y=0;
}
#endif // ]

//#define PIB_WIDGET_INITIALIZER_MAP_WINDOW
static void pib_widget_initializer_dispatch_physical(
    PibWidgetAcceptor *acceptor, PibWidget *widget)
{
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
  SIZE *geometry=widget->vmt->geometry(widget);
#if defined (PIB_WIDGET_INITIALIZER_MAP_WINDOW) // [
  RECT rect;
#endif // ]

  if (acceptor->initializer.realizing) {
#if defined (PIB_WIDGET_INITIALIZER_MAP_WINDOW) // [
    rect.left=acceptor->initializer.x;
    rect.top=acceptor->initializer.y;
    rect.right=acceptor->initializer.x+geometry->cx;
    rect.bottom=acceptor->initializer.y+geometry->cy;

    MapWindowPoints(
      acceptor->initializer.petzold.hDlg, // HWND    hWndFrom,
      NULL,                               // HWND    hWndTo,
      (POINT *)&rect,                     // LPPOINT lpPoints,
      2                                   // UINT    cPoints
    );
#endif // ]

    SetWindowPos(
      PIB_WIDGET_PHYSICAL(widget)->hWnd,  // HWND hWnd,
      HWND_NOTOPMOST,                     // HWND hWndInsertAfter,
#if defined (PIB_WIDGET_INITIALIZER_MAP_WINDOW) // [
      rect.left,                          // int  X,
      rect.top,                           // int  Y,
      rect.right-rect.left,               // int  cx,
      rect.bottom-rect.top,               // int  cy,
#else // ] [
      acceptor->initializer.x,               // int  X,
      acceptor->initializer.y,               // int  Y,
      geometry->cx,                       // int  cx,
      geometry->cy,                       // int  cy,
#endif // ]
      SWP_NOACTIVATE|SWP_NOZORDER         // UINT uFlags
    );
  }
  else {
    PIB_WIDGET_PHYSICAL(widget)->hWnd=GetDlgItem(
      acceptor->initializer.petzold.hDlg, // HWND hDlg,
      PIB_WIDGET_PHYSICAL(widget)->idc    // int  nIDDlgItem
    );

    SetWindowLongPtrW(
      PIB_WIDGET_PHYSICAL(widget)->hWnd,  // HWND     hWnd,
      GWLP_USERDATA,                      // int      nIndex,
      (LONG_PTR)widget                    // LONG_PTR dwNewLong
    );

DVWRITELN(">>> \"%s\" %ld %ld",widget->vmt->id,geometry->cx,geometry->cy);
    widget->vmt->extenti(widget,&acceptor->initializer.petzold,geometry);
DVWRITELN("<<< \"%s\" %ld %ld",widget->vmt->id,geometry->cx,geometry->cy);
  }

DVWRITELN("@@@ \"%s\" cx:%ld cy:%ld",widget->vmt->id,geometry->cx,geometry->cy);
  acceptor->initializer.x+=geometry->cx;
  acceptor->initializer.y+=geometry->cy;
#else // ] [
    PIB_WIDGET_PHYSICAL(widget)->hWnd=GetDlgItem(
      acceptor->initializer.hDlg,         // HWND hDlg,
      PIB_WIDGET_PHYSICAL(widget)->idc    // int  nIDDlgItem
    );

DVWRITELN("@@@ %p %d: %p",acceptor->initializer.hDlg,PIB_WIDGET_PHYSICAL(widget)->idc,PIB_WIDGET_PHYSICAL(widget)->hWnd);
    SetWindowLongPtrW(
      PIB_WIDGET_PHYSICAL(widget)->hWnd,  // HWND     hWnd,
      GWLP_USERDATA,                      // int      nIndex,
      (LONG_PTR)widget                    // LONG_PTR dwNewLong
    );
#endif // ]
}

static void pib_widget_initializer_dispatch_combobox(
    PibWidgetAcceptor *acceptor, PibWidget *widget)
{
  PibWidgetCombobox *combobox=PIB_WIDGET_COMBOBOX(widget);
  PibWidgetComboboxMap *map;

  pib_widget_initializer_dispatch_physical(acceptor,widget);

  if (combobox->map&&combobox->map->label) {
DWRITELN("##################################");
    for (map=combobox->map;map->label;++map) {
DVWRITELNW("### %p \"%s\"",combobox->hWnd,map->label);
      SendMessageW(combobox->hWnd,CB_INSERTSTRING,-1,(LPARAM)map->label);
    }

    SendMessageW(combobox->hWnd,CB_SETCURSEL,0,0);
  }
}

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
static void pib_widget_initializer_dispatch_button(
    PibWidgetAcceptor *acceptor, PibWidget *widget)
{
  pib_widget_initializer_dispatch_physical(acceptor,widget);
}

static void pib_widget_initializer_dispatch_treeview(
    PibWidgetAcceptor *acceptor, PibWidget *widget)
{
  pib_widget_initializer_dispatch_physical(acceptor,widget);
}
#endif // ]

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
static void pib_widget_initializer_push(PibWidgetAcceptor *acceptor,
    PibWidgetNode *node)
{
  enum {
    SIZE_TOTAL=sizeof acceptor->initializer.stack.node,
    SIZE_ELEM=sizeof acceptor->initializer.stack.node[0],
    NELEMS=SIZE_TOTAL/SIZE_ELEM,
  };

  if (NELEMS<=acceptor->initializer.stack.tos) {
    DMESSAGE("overflow");
    return;
  }

  acceptor->initializer.stack.node[acceptor->initializer.stack.tos]=node;
  ++acceptor->initializer.stack.tos;
}

static PibWidgetNode *pib_widget_initializer_pop(PibWidgetAcceptor *acceptor)
{
  if (!acceptor->initializer.stack.tos) {
    DMESSAGE("underflow");
    return NULL;
  }

  return acceptor->initializer.stack.node[--acceptor->initializer.stack.tos];
}

//#define PIB_WIDGET_INITIALIZER_BORDER
#if defined (PIB_WIDGET_INITIALIZER_BORDER) // [
#if defined (PIB_WIDGET_PIXELS) // [
static void  pib_widget_initializer_border(PibWidgetAcceptor *acceptor,
    PibWidgetRealizerMap *map, SIZE *border)
#else // ] [
static int pib_widget_initializer_border(PibWidgetAcceptor *acceptor)
#endif // ]
{
#if defined (PIB_WIDGET_PIXELS) // [
  SIZE *unit=&acceptor->initializer.unit;
#else // ] [
  int border=0;
#endif // ]
  int tos=acceptor->initializer.stack.tos;
  PibWidgetNode *node;
  PibWidgetList *list;
  
#if defined (PIB_WIDGET_PIXELS) // [
  border->cx=0;
  border->cy=0;
#endif // ]

  while (tos) {
    --tos;
    node=acceptor->initializer.stack.node[tos];
    list=node->vmt->list(PIB_WIDGET(node));

    if (!list)
      break;

#if defined (PIB_WIDGET_PIXELS) // [
  *map->cx(border)+=MulDiv(PIB_WIDGET_BOX(list)->border,*map->cx(unit),
      PIB_WIDGET_INITIALIZER_UNIT);
  *map->cy(border)+=MulDiv(PIB_WIDGET_BOX(list)->border,*map->cy(unit),
      PIB_WIDGET_INITIALIZER_UNIT);
#else // ] [
    border+=PIB_WIDGET_BOX(list)->border;
#endif // ]
  }

#if ! defined (PIB_WIDGET_PIXELS) // [
  return border;
#endif // ]
}
#endif // ]

static void pib_widget_initializer_dispatch_box(PibWidgetAcceptor *acceptor,
    PibWidget *widget, PibWidgetRealizerMap *map)
#else // ] [
static void pib_widget_initializer_dispatch_box(
    PibWidgetAcceptor *acceptor, PibWidget *widget)
#endif // ]
{
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
  PibWidgetList *list=widget->vmt->list(widget);
  SIZE *bounds=list?list->vmt->geometry(PIB_WIDGET(list))
      :&acceptor->initializer.bounds;
  SIZE *geometry=widget->vmt->geometry(widget);
#if ! defined (PIB_WIDGET_PIXELS) // [
  long border=PIB_WIDGET_BOX(widget)->border;
  long delta=PIB_WIDGET_BOX(widget)->delta;
#endif // ]
#if defined (PIB_WIDGET_PIXELS) // [
  SIZE *unit=&acceptor->initializer.unit;
  SIZE border;
  SIZE delta;
#endif // ]
#if  1 // [
  long x,y,cx,dy;
#else // ] [
  long x,cx,dy;
#endif // ]
  int realizing=acceptor->initializer.realizing;
#endif // ]
  PibWidgetNode *cur;

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
  if (realizing&&PIB_WIDGET_BOX(widget)->nvar) {
#if defined (PIB_WIDGET_PIXELS) // [
    dy=*map->cy(bounds)-*map->cy(geometry);
DVWRITELN("*** \"%s\" DY:%ld",widget->vmt->id,dy);

    if (dy<0) {
      DVMESSAGE("something went wrong (dy:%ld)",dy);
      return;
    }

#if defined (PIB_WIDGET_INITIALIZER_BORDER) // [
    pib_widget_initializer_border(acceptor,map,&border);
#else // ] [
    *map->cx(&border)=MulDiv(PIB_WIDGET_BOX(widget)->border,*map->cx(unit),
        PIB_WIDGET_INITIALIZER_UNIT);
    *map->cy(&border)=MulDiv(PIB_WIDGET_BOX(widget)->border,*map->cy(unit),
        PIB_WIDGET_INITIALIZER_UNIT);
#endif // ]
    dy-=*map->cy(&border)<<1;
#else // ] [
    dy=*map->cy(bounds)-*map->cy(geometry);
    dy-=pib_widget_initializer_border(acceptor)<<1;
#endif // ]
DVWRITELN("### \"%s\" y:%ld",widget->vmt->id,*map->y(acceptor));
    *map->y(acceptor)-=dy>>1;
DVWRITELN("### \"%s\" y:%ld",widget->vmt->id,*map->y(acceptor));
    dy/=PIB_WIDGET_BOX(widget)->nvar;
  }
  else
    dy=0;

#if defined (PIB_WIDGET_PIXELS) // [
  *map->cx(&border)=MulDiv(PIB_WIDGET_BOX(widget)->border,*map->cx(unit),
      PIB_WIDGET_INITIALIZER_UNIT);
  *map->cy(&border)=MulDiv(PIB_WIDGET_BOX(widget)->border,*map->cy(unit),
      PIB_WIDGET_INITIALIZER_UNIT);
if (realizing) DVWRITELN("border.cx:%ld border.cy:%ld",*map->cx(&border),*map->cy(&border));
  *map->cx(&delta)=MulDiv(PIB_WIDGET_BOX(widget)->delta,*map->cx(unit),
      PIB_WIDGET_INITIALIZER_UNIT);
  *map->cy(&delta)=MulDiv(PIB_WIDGET_BOX(widget)->delta,*map->cy(unit),
      PIB_WIDGET_INITIALIZER_UNIT);
  *map->x(acceptor)+=*map->cx(&border);
DVWRITELN("### \"%s\" y:%ld",widget->vmt->id,*map->y(acceptor));
  *map->y(acceptor)+=*map->cy(&border);
DVWRITELN("### \"%s\" y:%ld",widget->vmt->id,*map->y(acceptor));
#else // ] [
  *map->x(acceptor)+=border;
  *map->y(acceptor)+=border;
#endif // ]
#if 1 // [
  // we need to remember "y".
  y=*map->y(acceptor);
DVWRITELN("### \"%s\" y:%ld",widget->vmt->id,y);

  if (y<0) {
    DVMESSAGE("something went wrong (y:%ld)",y);
    return;
  }
#endif // ]
#endif // ]

  for (cur=PIB_WIDGET_BOX(widget)->head;cur;cur=cur->next) {
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
    // remember "x".
    x=*map->x(acceptor); // {
DVWRITELN("### \"%s\" y:%ld",widget->vmt->id,*map->y(acceptor));

    if (realizing) {
      // when realizing, the box's bounds (cx,cy) are already known. anyway,
      // still we need to position the child within the box in front of
      // calling accept().
      cx=*map->cx(widget->vmt->geometry(widget));
      cx-=*map->cx(cur->vmt->geometry(PIB_WIDGET(cur)));

      switch (*map->mx(&cur->mode)) {
      case PIB_WIDGET_NONE:
        break;
      case PIB_WIDGET_BEGIN:
        break;
      case PIB_WIDGET_CENTER:
        *map->x(acceptor)+=cx>>1;
        break;
      case PIB_WIDGET_END:
        *map->x(acceptor)+=cx;
        break;
      default:
        break;
      }

      switch (*map->my(&cur->mode)) {
      case PIB_WIDGET_NONE:
        break;
      case PIB_WIDGET_BEGIN:
        break;
      case PIB_WIDGET_CENTER:
#if 1 // [
        *map->y(acceptor)
            +=(dy-*map->cy(cur->vmt->geometry(PIB_WIDGET(cur))))>>1;
#else // ] [
        *map->y(acceptor)+=(dy>>1);
#endif // ]
        break;
      case PIB_WIDGET_END:
        *map->y(acceptor)+=dy;
        break;
      default:
        DMESSAGE("something went wrong");
        break;
      }
    }

    ////////
    pib_widget_initializer_push(acceptor,cur);
#endif // ]
    cur->vmt->accept(PIB_WIDGET(cur),acceptor);
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
    pib_widget_initializer_pop(acceptor);

    ////////
    if (realizing) {
      switch (*map->my(&cur->mode)) {
      case PIB_WIDGET_NONE:
        break;
      case PIB_WIDGET_BEGIN:
        *map->y(acceptor)+=dy;
        break;
      case PIB_WIDGET_CENTER:
#if 1 // [
        *map->y(acceptor)
            +=(dy-*map->cy(cur->vmt->geometry(PIB_WIDGET(cur))))>>1;
#else // ] [
        *map->y(acceptor)+=dy>>1;
#endif // ]
        break;
      case PIB_WIDGET_END:
        break;
      default:
        DMESSAGE("something went wrong");
        break;
      }
    }
    else {
      // we just have to consider (cx,cy). (x,y) is already advanced by
      // the call to "accept()" - indeed we make use of it.
      //
      // in x-direction, we take into account just the maximum.
      cx=*map->x(acceptor)-x;

      if (*map->cx(geometry)<cx)
        *map->cx(geometry)=cx;

      // in y-direction, the box is growing by each step.
      *map->cy(geometry)=*map->y(acceptor)-y;

      switch (*map->my(&cur->mode)) {
      case PIB_WIDGET_NONE:
        break;
      case PIB_WIDGET_BEGIN:
      case PIB_WIDGET_CENTER:
      case PIB_WIDGET_END:
        ++PIB_WIDGET_BOX(widget)->nvar;
        break;
      default:
        DMESSAGE("something went wrong");
        break;
      }
    }

#if defined (PIB_WIDGET_PIXELS) // [
    if (cur->next) {
DVWRITELN("*** \"%s\" CY:%ld (%d)",widget->vmt->id,*map->cy(&delta),PIB_WIDGET_BOX(widget)->delta);
      *map->y(acceptor)+=*map->cy(&delta);
    }
#else // ] [
    if (cur->next)
      *map->y(acceptor)+=delta;
#endif // ]

    // restore "x".
    *map->x(acceptor)=x; // }
#endif // ]
  }

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
  // "x" is still restored to it's initial value, hence in x-direction we
  // need to add "cx".
  *map->x(acceptor)+=*map->cx(geometry);
#if defined (PIB_WIDGET_PIXELS) // [
if (realizing) DVWRITELN("??? border.cx:%ld border.cy:%ld",*map->cx(&border),*map->cy(&border));
#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
if (realizing) DVWRITELN("??? \"%s\" cx:%.0lf cy:%.0lf",widget->vmt->id,*map->x(acceptor)+0.5,*map->y(acceptor)+0.5);
#else // ] [
if (realizing) DVWRITELN("??? \"%s\" cx:%ld cy:%ld",widget->vmt->id,*map->x(acceptor),*map->y(acceptor));
#endif // ]
  *map->x(acceptor)+=*map->cx(&border);
  *map->y(acceptor)+=*map->cy(&border);
#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
if (realizing) DVWRITELN("??? \"%s\" cx:%.0lf cy:%.0lf",widget->vmt->id,*map->x(acceptor)+0.5,*map->y(acceptor)+0.5);
#else // ] [
if (realizing) DVWRITELN("??? \"%s\" cx:%ld cy:%ld",widget->vmt->id,*map->x(acceptor),*map->y(acceptor));
#endif // ]
#else // ] [
  *map->x(acceptor)+=border;
  *map->y(acceptor)+=border;
#endif // ]
#endif // ]
}

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
#if 0 // [
static void pib_widget_initializer_dispatch_hbox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
  pib_widget_initializer_dispatch_box(acceptor,widget,&pib_map_twisted);
}

static void pib_widget_initializer_dispatch_vbox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
  pib_widget_initializer_dispatch_box(acceptor,widget,&pib_map_affine);
}
#else // ] [
static void pib_widget_initializer_dispatch_hbox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
  pib_widget_initializer_dispatch_box(acceptor,widget,&pib_map_twisted);
}

static void pib_widget_initializer_dispatch_vbox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
  pib_widget_initializer_dispatch_box(acceptor,widget,&pib_map_affine);
}
#endif // ]
#endif // ]

static PibWidgetAcceptorVmt vmt={
  .id="PIB_INITIALIZER",
  .destroy=pib_widget_initializer_destroy,
  .dispatch_combobox=pib_widget_initializer_dispatch_combobox,
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
  .dispatch_treeview=pib_widget_initializer_dispatch_treeview,
  .dispatch_button=pib_widget_initializer_dispatch_button,
  .dispatch_hbox=pib_widget_initializer_dispatch_hbox,
  .dispatch_vbox=pib_widget_initializer_dispatch_vbox,
#else // ] [
  .dispatch_treeview=pib_widget_initializer_dispatch_physical,
  .dispatch_button=pib_widget_initializer_dispatch_physical,
  .dispatch_hbox=pib_widget_initializer_dispatch_box,
  .dispatch_vbox=pib_widget_initializer_dispatch_box,
#endif // ]
};

#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
////////
#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
static double *_pib_map_affine_x(PibWidgetAcceptor *acceptor)
#else // ] [
static long *_pib_map_affine_x(PibWidgetAcceptor *acceptor)
#endif // ]
{
  return &acceptor->initializer.x;
}

#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
static double *_pib_map_affine_y(PibWidgetAcceptor *acceptor)
#else // ] [
static long *_pib_map_affine_y(PibWidgetAcceptor *acceptor)
#endif // ]
{
  return &acceptor->initializer.y;
}

static long *_pib_map_affine_cx(SIZE *size)
{
  return &size->cx;
}

static long *_pib_map_affine_cy(SIZE *size)
{
  return &size->cy;
}

static EPibWidgetMode *_pib_map_affine_mx(PibWidgetMode *mode)
{
  return &mode->x;
}

static EPibWidgetMode *_pib_map_affine_my(PibWidgetMode *mode)
{
  return &mode->y;
}

static PibWidgetRealizerMap pib_map_affine={
  .x=_pib_map_affine_x,
  .y=_pib_map_affine_y,
  .cx=_pib_map_affine_cx,
  .cy=_pib_map_affine_cy,
  .mx=_pib_map_affine_mx,
  .my=_pib_map_affine_my,
};

////////
#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
static double *_pib_map_twisted_x(PibWidgetAcceptor *acceptor)
#else // ] [
static long *_pib_map_twisted_x(PibWidgetAcceptor *acceptor)
#endif // ]
{
  return &acceptor->initializer.y;
}

#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
static double *_pib_map_twisted_y(PibWidgetAcceptor *acceptor)
#else // ] [
static long *_pib_map_twisted_y(PibWidgetAcceptor *acceptor)
#endif // ]
{
  return &acceptor->initializer.x;
}

static long *_pib_map_twisted_cx(SIZE *size)
{
  return &size->cy;
}

static long *_pib_map_twisted_cy(SIZE *size)
{
  return &size->cx;
}

static EPibWidgetMode *_pib_map_twisted_mx(PibWidgetMode *mode)
{
  return &mode->y;
}

static EPibWidgetMode *_pib_map_twisted_my(PibWidgetMode *mode)
{
  return &mode->x;
}

static PibWidgetRealizerMap pib_map_twisted={
  .x=_pib_map_twisted_x,
  .y=_pib_map_twisted_y,
  .cx=_pib_map_twisted_cx,
  .cy=_pib_map_twisted_cy,
  .mx=_pib_map_twisted_mx,
  .my=_pib_map_twisted_my,
};
#endif // ]
#endif // ]
