/*
 * pib_widget_list.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

PibWidget *pib_widget_list_new_ex(PibWidgetVmt *vmt)
{
  PibWidget *widget=pib_widget_new(vmt);

  if (!widget) {
    DMESSAGE("creating list");
    goto e_list;
  }

  return widget;
//cleanup:
  pib_widget_vmt()->destroy(widget);
e_list:
  return NULL;
}

PibWidget *pib_widget_list_new(void)
{
  return pib_widget_list_new_ex(pib_widget_list_vmt());
}

void pib_widget_list_append(PibWidgetList *list, EPibWidgetMode mx,
    EPibWidgetMode my, PibWidget *widget)
{
  PibWidget *node=pib_widget_node_new(list,mx,my,widget);

  if (!node) {
    DMESSAGE("creating node");
    goto e_node;
  }

  if (list->tail) {
    PIB_WIDGET_NODE(node)->prev=list->tail;
    list->tail->next=PIB_WIDGET_NODE(node);
  }
  else
    list->head=PIB_WIDGET_NODE(node);

  list->tail=PIB_WIDGET_NODE(node);
  pib_widget_ref(node);
  ++list->nchildren;
e_node:
  return;
}

static void _pib_widget_list_destroy(PibWidget *widget)
{
  PibWidgetNode *tail=PIB_WIDGET_LIST(widget)->tail;
  PibWidgetNode *cur;

  while (tail) {
    cur=tail;
    tail=tail->prev;
    pib_widget_unref(PIB_WIDGET(cur));
  }

  pib_widget_vmt()->destroy(widget);
}

PibWidgetVmt *pib_widget_list_vmt(void)
{
  static PibWidgetVmt vmt;
  PibWidgetVmt *prev;

  if (!vmt.destroy) {
    prev=pib_widget_vmt();
    vmt=*prev;
    vmt.id=PIB_WIDGET_ID_LIST;
    vmt.size=sizeof (PibWidgetList);
    vmt.prev=prev;
    vmt.destroy=_pib_widget_list_destroy;
  }

  return &vmt;
}
