/*
 * pib_widget_node.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

PibWidget *pib_widget_node_new_ex(PibWidgetVmt *vmt, PibWidgetList *list,
    EPibWidgetMode mx, EPibWidgetMode my, PibWidget *ref)
{
  PibWidget *widget=pib_widget_new(vmt);

  if (!widget) {
    DMESSAGE("creating node");
    goto e_node;
  }

  PIB_WIDGET_NODE(widget)->prev=NULL;
  PIB_WIDGET_NODE(widget)->next=NULL;
  PIB_WIDGET_NODE(widget)->list=list;
  PIB_WIDGET_NODE(widget)->mode.x=mx;
  PIB_WIDGET_NODE(widget)->mode.y=my;
  pib_widget_ref(PIB_WIDGET_NODE(widget)->ref=ref);

  return widget;
//cleanup:
  pib_widget_vmt()->destroy(widget);
e_node:
  return NULL;
}

static void _pib_widget_node_destroy(PibWidget *widget)
{
  pib_widget_unref(PIB_WIDGET_NODE(widget)->ref);
  pib_widget_vmt()->destroy(widget);
}

#if defined (PIB_WIDGET_REALIZER2) // [
static void _pib_widget_node_extent(PibWidget *widget,
    PibWidgetAcceptor *acceptor, SIZE *size)
{
  PibWidget *ref=PIB_WIDGET_NODE(widget)->ref;

  ref->vmt->extent(ref,acceptor,size);
}
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
static void _pib_widget_node_extenti(PibWidget *widget,
    PibWidgetPetzold *petzold, SIZE *size)
{
  DVMESSAGE("method not available for %s",widget->vmt->id);
}
#endif // ]

#if defined (PIB_WIDGET_PETZOLD2) // [
static void _pib_widget_node_extent(PibWidget *widget,
    PibWidgetPetzold *petzold, SIZE *size)
#elif defined (PIB_WIDGET_EXTENT_POINT_SIZE) // ] [
static void _pib_widget_node_extent(PibWidget *widget, int point_size,
    SIZE *size)
#else // ] [
static void _pib_widget_node_extent(PibWidget *widget, HDC hDc,
    int point_size, SIZE *size)
#endif // ]
{
  PibWidget *ref=PIB_WIDGET_NODE(widget)->ref;

#if defined (PIB_WIDGET_PETZOLD2) // [
  ref->vmt->extent(ref,petzold,size);
#elif defined (PIB_WIDGET_EXTENT_POINT_SIZE) // ] [
  ref->vmt->extent(ref,point_size,size);
#else // ] [
  ref->vmt->extent(ref,hDc,point_size,size);
#endif // ]
}
#endif // ]

static SIZE *_pib_widget_node_geometry(PibWidget *widget)
{
  PibWidget *ref=PIB_WIDGET_NODE(widget)->ref;

  return ref->vmt->geometry(ref);
}

static void _pib_widget_node_accept(PibWidget *widget,
    PibWidgetAcceptor *acceptor)
{
  PibWidget *ref=PIB_WIDGET_NODE(widget)->ref;

  ref->vmt->accept(ref,acceptor);
}

static PibWidgetList *_pib_widget_node_list(PibWidget *widget)
{
  return PIB_WIDGET_NODE(widget)->list;
}

PibWidgetVmt *pib_widget_node_vmt(void)
{
  static PibWidgetVmt vmt;
  PibWidgetVmt *prev;

  if (!vmt.destroy) {
    prev=pib_widget_vmt();
    vmt=*prev;
    vmt.id=PIB_WIDGET_ID_NODE;
    vmt.size=sizeof (PibWidgetNode);
    vmt.prev=prev;
    vmt.destroy=_pib_widget_node_destroy;
#if defined (PIB_WIDGET_REALIZER2) // [
    vmt.extent=_pib_widget_node_extent;
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
    vmt.extenti=_pib_widget_node_extenti;
#endif // ]
    vmt.extent=_pib_widget_node_extent;
#endif // ]
    vmt.geometry=_pib_widget_node_geometry;
    vmt.list=_pib_widget_node_list;
    vmt.accept=_pib_widget_node_accept;
  }

  return &vmt;
}
