/*
 * pib_widget_node_proxy.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

#if defined (PIB_WIDGET_NODE_PROXY) // [
PibWidget *pib_widget_node_proxy_new_ex(PibWidgetVmt *vmt, PibWidget *ref)
{
  PibWidget *widget=pib_widget_new(vmt);

  if (!widget) {
    DMESSAGE("creating node_proxy");
    goto e_node_proxy;
  }

  PIB_WIDGET_NODE_PROXY(widget)->prev=NULL;
  PIB_WIDGET_NODE_PROXY(widget)->next=NULL;
  pib_widget_ref(PIB_WIDGET_NODE_PROXY(widget)->ref=ref);

  return widget;
//cleanup:
  pib_widget_vmt()->destroy(widget);
e_node_proxy:
  return NULL;
}

PibWidget *pib_widget_node_proxy_new(PibWidget *ref)
{
  return pib_widget_node_proxy_new_ex(pib_widget_node_proxy_vmt(),ref);
}

static void _pib_widget_node_proxy_destroy(PibWidget *widget)
{
  pib_widget_unref(PIB_WIDGET_NODE_PROXY(widget)->ref);
  pib_widget_vmt()->destroy(widget);
}

#if defined (PIB_WIDGET_EXTENT_POINT_SIZE) // [
static void _pib_widget_node_proxy_extent(PibWidget *widget, int point_size,
    SIZE *size)
#else // ] [
static void _pib_widget_node_proxy_extent(PibWidget *widget, HDC hDc, SIZE *size)
#endif // ]
{
  PibWidget *ref=PIB_WIDGET_NODE_PROXY(widget)->ref;

#if defined (PIB_WIDGET_EXTENT_POINT_SIZE) // [
  ref->vmt->extent(ref,point_size,size);
#else // ] [
  ref->vmt->extent(ref,hDc,size);
#endif // ]
}

static SIZE *_pib_widget_node_proxy_geometry(PibWidget *widget)
{
  PibWidget *ref=PIB_WIDGET_NODE_PROXY(widget)->ref;

  return ref->vmt->geometry(ref);
}

static void _pib_widget_node_proxy_accept(PibWidget *widget,
    PibWidgetAcceptor *acceptor)
{
  PibWidget *ref=PIB_WIDGET_NODE_PROXY(widget)->ref;

  ref->vmt->accept(ref,acceptor);
}

PibWidgetVmt *pib_widget_node_proxy_vmt(void)
{
  static PibWidgetVmt vmt;
  PibWidgetVmt *prev;

  if (!vmt.destroy) {
    prev=pib_widget_vmt();
    vmt=*prev;
    vmt.id=PIB_WIDGET_ID_NODE_PROXY;
    vmt.size=sizeof (PibWidgetNodeProxy);
    vmt.prev=prev;
    vmt.destroy=_pib_widget_node_proxy_destroy;
    vmt.extent=_pib_widget_node_proxy_extent;
    vmt.geometry=_pib_widget_node_proxy_geometry;
    vmt.accept=_pib_widget_node_proxy_accept;
  }

  return &vmt;
}
#endif // ]
