/*
 * pib_widget_physical.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

PibWidget *pib_widget_physical_new(PibWidgetVmt *vmt, long cx, long cy,
    const wchar_t *title)
{
  PibWidget *widget=pib_widget_new(vmt);

  if (!widget) {
    DMESSAGE("creating physical");
    goto e_physical;
  }

  PIB_WIDGET_PHYSICAL(widget)->geometry.cx=cx;
  PIB_WIDGET_PHYSICAL(widget)->geometry.cy=cy;
  PIB_WIDGET_PHYSICAL(widget)->title=title;

  return widget;
//cleanup:
  pib_widget_vmt()->destroy(widget);
e_physical:
  return NULL;
}

static SIZE *_pib_widget_physical_geometry(PibWidget *widget)
{
  return &PIB_WIDGET_PHYSICAL(widget)->geometry;
}

PibWidgetVmt *pib_widget_physical_vmt(void)
{
  static PibWidgetVmt vmt;
  PibWidgetVmt *prev;

  if (!vmt.destroy) {
    prev=pib_widget_vmt();
    vmt=*prev;
    vmt.id=PIB_WIDGET_ID_PHYSICAL;
    vmt.size=sizeof (PibWidgetPhysical);
    vmt.prev=prev;
    vmt.geometry=_pib_widget_physical_geometry;
  }

  return &vmt;
}
