/*
 * pib_widget_realizer.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

#if defined (PIB_WIDGET_REALIZER2) // [
#if 0 // [
typedef const struct _PibWidgetRealizerMap PibWidgetRealizerMap;

struct _PibWidgetRealizerMap {
  double *(*tx)(PibWidgetTurtle *turtle);
  double *(*ty)(PibWidgetTurtle *turtle);
  long *(*cx)(SIZE *size);
  long *(*cy)(SIZE *size);
  EPibWidgetMode *(*mx)(PibWidgetMode *mode);
  EPibWidgetMode *(*my)(PibWidgetMode *mode);
};
#endif // ]

static PibWidgetAcceptorVmt vmt;
static PibWidgetRealizerMap pib_map_affine;
static PibWidgetRealizerMap pib_map_twisted;

int pib_widget_realizer_create(PibWidgetAcceptor *acceptor, long border,
    const wchar_t *title, WORD point_size, const wchar_t *type_face)
{
#if ! defined (PIB_WIDGET_REALIZER2_MORPH) // [
  static const wchar_t TEXT[]
      //=L"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
      //=L"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
      //=L"ABCDEFGHIJKLMNOPQRSTUVWXYZ";
      =L"0123456789";

  enum {
    TEXT_LEN=(sizeof TEXT)/(sizeof TEXT[0])-1
  };
#endif // ]

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetRealizerCore *core=&acceptor->core;
#endif // ]
  PibWidgetRealizer *realizer=&acceptor->realizer;
#if ! defined (PIB_WIDGET_REALIZER2_MORPH) // [
  SIZE size;
  LOGFONTW lf;
#endif // ]

  memset(acceptor,0,sizeof *acceptor);
  acceptor->vmt=&vmt;

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
DVWRITELN("border:%d",border);
  if (pib_widget_realizer_core_create(core,border,point_size,type_face)<0) {
    DMESSAGE("creating core");
    goto e_core;
  }
#else // ] [
  realizer->border=border;
#endif // ]
  realizer->title=title;
  realizer->point_size=point_size;
  realizer->type_face=type_face;

#if ! defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if 0 // [
  /*
   * The low-order word of the return value contains the horizontal dialog
   * box base unit, and the high-order word contains the vertical dialog
   * box base unit.
   */
  realizer->unit.cx=LOWORD(baseunit);
  realizer->unit.cy=HIWORD(baseunit);
#endif // ]

  /*
   * The CreateCompatibleDC function creates a memory device context (DC)
   * compatible with the specified device.
   *
   * When you no longer need the memory DC, call the DeleteDC function.
   */
  realizer->hDc=CreateCompatibleDC(NULL);

  if (!realizer->hDc) {
    DMESSAGE("getting device context");
    goto e_hdc;
  }

  /*
   * The CreateFontIndirect function creates a logical font that has the
   * specified characteristics. The font can subsequently be selected as
   * the current font for any device context.
   *
   * When you no longer need the font, call the DeleteObject function
   * to delete it.
   */
  memset(&lf,0,sizeof lf);
  /*
   * lfHeight > 0:  The font mapper transforms this value into device units
   *   and matches it against the cell height of the available fonts.
   * lfHeight < 0:  The font mapper transforms this value into device units
   *   and matches its absolute value against the character height of the
   *   available fonts.
   */
  lf.lfHeight=-point_size,
  wcscpy(lf.lfFaceName,type_face);
  realizer->font.hCur=CreateFontIndirectW(&lf);

  if (!realizer->font.hCur) {
    DMESSAGE("creating font");
    goto e_font_cur;
  }

  realizer->font.hPrev=(HFONT)SelectObject(realizer->hDc,realizer->font.hCur);

  if (!realizer->font.hPrev) {
    DMESSAGE("selecting font");
    goto e_font_prev;
  }

#if 1 // [
  /*
   * Q145994: HOWTO: Calculate Dialog Units When Not Using the System Font
   *   https://jeffpar.github.io/kbarchive/kb/145/Q145994/
   */
  GetTextExtentPoint32W(
    realizer->hDc,      // HDC     hdc,
    TEXT,               // LPCWSTR lpString,
    TEXT_LEN,           // int     c,
    &size               // LPSIZE  psizl
  );

  //realizer->unit.cx=((double)size.cx/(TEXT_LEN>>1)+1)/2;
  realizer->unit.cx=(double)size.cx/TEXT_LEN;
#if 0 // [
  realizer->unit.cx=((double)size.cx/52+1)/2;
DVWRITELN("unit.cx:%.2lf",realizer->unit.cx);
#endif // ]
  realizer->unit.cy=point_size;
#endif // ]
#endif // ]

  return 0;
//cleanup:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  core->vmt->destroy(core);
#else // ] [
  pib_widget_realizer_core_destroy(core);
#endif // ]
e_core:
#else // ] [
  SelectObject(realizer->hDc,realizer->font.hPrev);
e_font_prev:
  DeleteObject(realizer->font.hCur);
e_font_cur:
  DeleteDC(realizer->hDc);
e_hdc:
#endif // ]
  return -1;
}

#define PIB_WIDGET_REALIZER_OVERLAPPEDWINDOW
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
static void pib_widget_realizer_reset(PibWidgetAcceptor *acceptor)
#else // ] [
static void pib_widget_realizer_reset(PibWidgetRealizer *realizer)
#endif // ]
{
  enum {
    STYLE=DS_3DLOOK
#if 0 // [
				|DS_SETFONT
        |DS_SHELLFONT
#endif // ]
        |DS_3DLOOK
        |DS_CENTER
#if defined (PIB_WIDGET_REALIZER_OVERLAPPEDWINDOW) // [
        |WS_OVERLAPPEDWINDOW
#else // ] [
        |DS_MODALFRAME
#endif // ]
        |DS_SYSMODAL
        |WS_CAPTION
        |WS_VISIBLE
        |WS_POPUP
        |WS_SYSMENU
  };

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetRealizerCore *core=&acceptor->core;
  PibWidgetRealizer *realizer=&acceptor->realizer;
#endif // ]

  pib_widget_buf_head(
    &realizer->buf,             // WidgetBuf *buf,
    STYLE,                      // DWORD style,
    0,                          // DWORD dwExtendedStyle,
    0,                          // short x,
    0,                          // short y,
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    core->turtle.x*4/core->unit.cx+0.5,
                                // short cx,
    core->turtle.y*8/core->unit.cy+0.5,
                                // short cy,
#else // ] [
    realizer->turtle.x*4/realizer->unit.cx+0.5,
                                // short cx,
    realizer->turtle.y*8/realizer->unit.cy+0.5,
                                // short cy,
#endif // ]
    NULL,                       // const wchar_t *menu,
    NULL,                       // const wchar_t *cls,
    realizer->title,            // const wchar_t *title,
    realizer->point_size,       // WORD point_size,
    realizer->type_face         // const wchar_t *type_face
  );
}

void pib_widget_realizer_phase1(PibWidgetAcceptor *acceptor, PibWidget *widget)
{
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetRealizerCore *core=&acceptor->core;
#else // ] [
  PibWidgetRealizer *realizer=&acceptor->realizer;
#endif // ]

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  pib_widget_realizer_reset(acceptor);
  core->turtle.x=core->border*core->unit.cx/4;
  core->turtle.y=core->border*core->unit.cy/8;
DVWRITELN("border: %.2lf turtle.x:%.2lf turtle.y:%.2lf",core->border,core->turtle.x,core->turtle.y);
  pib_widget_accept(widget,acceptor);
  core->turtle.x+=core->border*core->unit.cx/4;
  core->turtle.y+=core->border*core->unit.cy/8;
#else // ] [
  pib_widget_realizer_reset(realizer);
  realizer->turtle.x=realizer->border*realizer->unit.cx/4;
  realizer->turtle.y=realizer->border*realizer->unit.cy/8;
  pib_widget_accept(widget,acceptor);
  realizer->turtle.x+=realizer->border*realizer->unit.cx/4;
  realizer->turtle.y+=realizer->border*realizer->unit.cy/8;
#endif // ]
}

void pib_widget_realizer_phase2(PibWidgetAcceptor *acceptor, PibWidget *widget)
{
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetRealizerCore *core=&acceptor->core;
#endif // ]
  PibWidgetRealizer *realizer=&acceptor->realizer;
  size_t size1,size2;

  /////////////////////////////////////////////////////////////////////////////
  size1=realizer->buf.size;
  realizer->head=malloc(size1);

  if (!realizer->head) {
    DMESSAGE("allocating");
    goto e_head;
  }

  realizer->buf.head=realizer->head;

  /////////////////////////////////////////////////////////////////////////////
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  pib_widget_realizer_reset(acceptor);
  core->turtle.x=core->border*core->unit.cx/4;
  core->turtle.y=core->border*core->unit.cy/8;
  pib_widget_accept(widget,acceptor);
  core->turtle.x+=core->border*core->unit.cx/4;
  core->turtle.y+=core->border*core->unit.cy/8;
#else // ] [
  pib_widget_realizer_reset(realizer);
  realizer->turtle.x=realizer->border*realizer->unit.cx/4;
  realizer->turtle.y=realizer->border*realizer->unit.cy/8;
  pib_widget_accept(widget,acceptor);
  realizer->turtle.x+=realizer->border*realizer->unit.cx/4;
  realizer->turtle.y+=realizer->border*realizer->unit.cy/8;
#endif // ]
  size2=realizer->buf.size;

#if 1 // [
  if (size2!=size1||!realizer->head->cdit) {
    DVMESSAGE("something went wrong: size1:%u size2:%u cdit:%d",
        size2,size1,realizer->head->cdit);
    goto e_size;
  }
#else // ] [
  if (size2!=size1) {
    DVMESSAGE("something went wrong: size1:%u size2:%u",
        size2,size1,realizer->head->cdit);
    goto e_size;
  }
#endif // ]
e_size:
e_head:
  return;
}

////////
static void pib_widget_realizer_destroy(PibWidgetAcceptor *acceptor)
{
  PibWidgetRealizer *realizer=&acceptor->realizer;

  if (realizer->head)
    free(realizer->head);

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  acceptor->core.vmt->destroy(&acceptor->core);
#else // ] [
  pib_widget_realizer_core_destroy(&acceptor->core);
#endif // ]
#else // ] [
  SelectObject(realizer->hDc,realizer->font.hPrev);
  DeleteObject(realizer->font.hCur);
  DeleteDC(realizer->hDc);
#endif // ]
}

#if ! defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
static void pib_widget_realizer_dispatch_physical(PibWidgetAcceptor *acceptor,
    PibWidget *widget, DWORD style, wchar_t *cls)
{
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetRealizerCore *core=&acceptor->core;
#endif // ]
  PibWidgetRealizer *realizer=&acceptor->realizer;
  const wchar_t *title=PIB_WIDGET_PHYSICAL(widget)->title;
  SIZE *geometry=widget->vmt->geometry(widget);

  if (!realizer->head) {
    widget->vmt->extent(widget,acceptor,geometry);
    PIB_WIDGET_PHYSICAL(widget)->idc=realizer->buf.idc;
  }

  pib_widget_buf_item(
    &realizer->buf,                 // WidgetBuf *buf,
    style,                          // DWORD style,
    WS_EX_LEFT,                     // DWORD dwExtendedStyle,
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    core->turtle.x*4/core->unit.cx+0.5,
                                    // short x,
    core->turtle.y*8/core->unit.cy+0.5,
                                    // short y,
    (double)geometry->cx*4/core->unit.cx+0.5,
                                    // short cx,
    (double)geometry->cy*8/core->unit.cy+0.5,
                                    // short cy,
#else // ] [
    realizer->turtle.x*4/realizer->unit.cx+0.5,
                                    // short x,
    realizer->turtle.y*8/realizer->unit.cy+0.5,
                                    // short y,
    (double)geometry->cx*4/realizer->unit.cx+0.5,
                                    // short cx,
    (double)geometry->cy*8/realizer->unit.cy+0.5,
                                    // short cy,
#endif // ]
    cls,                            // wchar_t *cls,
    title                           // wchar_t *title
  );

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  core->turtle.x+=geometry->cx;
  core->turtle.y+=geometry->cy;
#else // ] [
  realizer->turtle.x+=geometry->cx;
  realizer->turtle.y+=geometry->cy;
#endif // ]
}
#endif // ]

static void pib_widget_realizer_dispatch_treeview(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  acceptor->core.vmt->dispatch_treeview(acceptor,widget);
#else // ] [
  enum { STYLE=WS_VISIBLE|WS_TABSTOP|WS_BORDER|TVS_INFOTIP|TVS_NOSCROLL };

  pib_widget_realizer_dispatch_physical(acceptor,widget,STYLE,WC_TREEVIEWW);
#endif // ]
}

static void pib_widget_realizer_dispatch_button(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  acceptor->core.vmt->dispatch_button(acceptor,widget);
#else // ] [
  enum { STYLE=WS_VISIBLE|WS_TABSTOP|BS_PUSHBUTTON };

  pib_widget_realizer_dispatch_physical(acceptor,widget,STYLE,WC_BUTTONW);
#endif // ]
}

static void pib_widget_realizer_dispatch_combobox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  acceptor->core.vmt->dispatch_combobox(acceptor,widget);
#else // ] [
  enum { STYLE=WS_VISIBLE|WS_TABSTOP|CBS_DROPDOWNLIST|CBS_HASSTRINGS };

  pib_widget_realizer_dispatch_physical(acceptor,widget,STYLE,WC_COMBOBOXW);
#endif // ]
}

#if ! defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
static void pib_widget_realizer_dispatch_box(PibWidgetAcceptor *acceptor,
    PibWidget *widget, PibWidgetRealizerMap *map)
{
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetRealizerCore *core=&acceptor->core;
#endif // ]
  PibWidgetRealizer *realizer=&acceptor->realizer;
  SIZE *geometry=widget->vmt->geometry(widget);
  int realizing UNUSED=realizer->buf.head?1:0;
  long delta=PIB_WIDGET_BOX(widget)->delta;
  PibWidgetNode *cur;
  double x,y,cx;

  if (!realizing) {
    // initialize the box's size.
    *map->cx(geometry)=0;
    *map->cy(geometry)=0;
  }

  // remember "y".
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  y=*map->ty(&core->turtle);
#else // ] [
  y=*map->ty(&realizer->turtle);
#endif // ]

  for (cur=PIB_WIDGET_LIST(widget)->head;cur;cur=cur->next) {
    // remember "x".
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    x=*map->tx(&core->turtle); // {
#else // ] [
    x=*map->tx(&realizer->turtle); // {
#endif // ]

    if (realizing) {
      cx=*map->cx(widget->vmt->geometry(widget));
      cx-=*map->cx(cur->vmt->geometry(PIB_WIDGET(cur)));

      switch (*map->mx(&cur->mode)) {
      case PIB_WIDGET_NONE:
        break;
      case PIB_WIDGET_BEGIN:
        break;
      case PIB_WIDGET_CENTER:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
        *map->tx(&core->turtle)+=cx/2.0;
#else // ] [
        *map->tx(&realizer->turtle)+=cx/2.0;
#endif // ]
        break;
      case PIB_WIDGET_END:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
        *map->tx(&core->turtle)+=cx;
#else // ] [
        *map->tx(&realizer->turtle)+=cx;
#endif // ]
        break;
      default:
        break;
      }
    }

    cur->vmt->accept(PIB_WIDGET(cur),acceptor);

    if (realizing) {
      switch (*map->mx(&cur->mode)) {
      case PIB_WIDGET_NONE:
        break;
      case PIB_WIDGET_BEGIN:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
        *map->tx(&core->turtle)+=cx;
#else // ] [
        *map->tx(&realizer->turtle)+=cx;
#endif // ]
        break;
      case PIB_WIDGET_CENTER:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
        *map->tx(&core->turtle)+=cx/2.0;
#else // ] [
        *map->tx(&realizer->turtle)+=cx/2.0;
#endif // ]
        break;
      case PIB_WIDGET_END:
        break;
      default:
        DMESSAGE("something went wrong");
        break;
      }
    }

    // in x-direction, we take into account just the maximum.
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    cx=*map->tx(&core->turtle)-x;
#else // ] [
    cx=*map->tx(&realizer->turtle)-x;
#endif // ]

    if (*map->cx(geometry)<cx)
      *map->cx(geometry)=cx+0.5;

    // in y-direction, the box is growing by each step.
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    *map->cy(geometry)=(*map->ty(&core->turtle)-y)+0.5;
#else // ] [
    *map->cy(geometry)=(*map->ty(&realizer->turtle)-y)+0.5;
#endif // ]

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    if (cur->next)
      *map->ty(&core->turtle)+=delta;

    // restore "x".
    *map->tx(&core->turtle)=x; // }
#else // ] [
    if (cur->next)
      *map->ty(&realizer->turtle)+=delta;

    // restore "x".
    *map->tx(&realizer->turtle)=x; // }
#endif // ]
  }

  // "x" is still restored to it's initial value, hence in x-direction we
  // need to add "cx".
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  *map->tx(&core->turtle)+=*map->cx(geometry);
#else // ] [
  *map->tx(&realizer->turtle)+=*map->cx(geometry);
#endif // ]
}
#endif // ]

static void pib_widget_realizer_dispatch_hbox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  pib_widget_realizer_core_dispatch_box(acceptor,widget,&pib_map_twisted);
#else // ] [
  pib_widget_realizer_dispatch_box(acceptor,widget,&pib_map_twisted);
#endif // ]
}

static void pib_widget_realizer_dispatch_vbox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  pib_widget_realizer_core_dispatch_box(acceptor,widget,&pib_map_affine);
#else // ] [
  pib_widget_realizer_dispatch_box(acceptor,widget,&pib_map_affine);
#endif // ]
}

////////
static PibWidgetAcceptorVmt vmt={
  .id="PIB_REALIZER",
  .destroy=pib_widget_realizer_destroy,
  .dispatch_combobox=pib_widget_realizer_dispatch_combobox,
  .dispatch_treeview=pib_widget_realizer_dispatch_treeview,
  .dispatch_button=pib_widget_realizer_dispatch_button,
  .dispatch_hbox=pib_widget_realizer_dispatch_hbox,
  .dispatch_vbox=pib_widget_realizer_dispatch_vbox,
};

////////
static double *_pib_map_affine_tx(PibWidgetTurtle *turtle)
{
  return &turtle->x;
}

static double *_pib_map_affine_ty(PibWidgetTurtle *turtle)
{
  return &turtle->y;
}

static long *_pib_map_affine_cx(SIZE *size)
{
  return &size->cx;
}

static long *_pib_map_affine_cy(SIZE *size)
{
  return &size->cy;
}

static EPibWidgetMode *_pib_map_affine_mx(PibWidgetMode *mode)
{
  return &mode->x;
}

static EPibWidgetMode *_pib_map_affine_my(PibWidgetMode *mode)
{
  return &mode->y;
}

static PibWidgetRealizerMap pib_map_affine={
  .tx=_pib_map_affine_tx,
  .ty=_pib_map_affine_ty,
  .cx=_pib_map_affine_cx,
  .cy=_pib_map_affine_cy,
  .mx=_pib_map_affine_mx,
  .my=_pib_map_affine_my,
};

////////
static double *_pib_map_twisted_tx(PibWidgetTurtle *turtle)
{
  return &turtle->y;
}

static double *_pib_map_twisted_ty(PibWidgetTurtle *turtle)
{
  return &turtle->x;
}

static long *_pib_map_twisted_cx(SIZE *size)
{
  return &size->cy;
}

static long *_pib_map_twisted_cy(SIZE *size)
{
  return &size->cx;
}

static EPibWidgetMode *_pib_map_twisted_mx(PibWidgetMode *mode)
{
  return &mode->y;
}

static EPibWidgetMode *_pib_map_twisted_my(PibWidgetMode *mode)
{
  return &mode->x;
}

static PibWidgetRealizerMap pib_map_twisted={
  .tx=_pib_map_twisted_tx,
  .ty=_pib_map_twisted_ty,
  .cx=_pib_map_twisted_cx,
  .cy=_pib_map_twisted_cy,
  .mx=_pib_map_twisted_mx,
  .my=_pib_map_twisted_my,
};
#endif // ]
