/*
 * pib_widget_run.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

static INT_PTR CALLBACK pib_widget_dlgproc(HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam);

#if defined (PIB_WIDGET_REALIZER2) // [
int pib_widget_run(PibWidget *widget, int border, const wchar_t *title,
    WORD point_size, const wchar_t *type_face)
#else // ] [
int pib_widget_run(PibWidget *widget, const wchar_t *title, WORD point_size,
    const wchar_t *type_face)
#endif // ]
{
  int err=-1;
#if ! defined (PIB_WIDGET_REALIZER2) // [
  PibWidgetBuf buf;
#endif // ]
  PibWidgetAcceptor realizer;
  DLGTEMPLATE *head;
#if ! defined (PIB_WIDGET_REALIZER2) // [
  size_t size1,size2;
#endif // ]
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetAcceptorParam param;
#endif // ]

#if defined (PIB_WIDGET_REALIZER2) // [
  pib_widget_realizer_create(
    &realizer,            // PibWidgetAcceptor *acceptor,
    border,               // int border,
    title,                // const wchar_t *title,
    point_size,           // WORD point_size,
    type_face             // const wchar_t *type_face,
  );

  pib_widget_realizer_phase1(&realizer,widget);
  pib_widget_realizer_phase2(&realizer,widget);
#else // ] [
  memset(&buf,0,sizeof buf);

  pib_widget_realizer_create(
    &realizer,            // PibWidgetAcceptor *acceptor,
    title,                // const wchar_t *title,
    point_size,           // WORD point_size,
    type_face,            // const wchar_t *type_face,
    &buf                  // PibWidgetBuf *buf
  );

  pib_widget_accept(widget,&realizer);

  size1=buf.size;
  buf.head=head=malloc(size1);

	if (!head) {
		DMESSAGE("allocating DLGTEMPLATE");
		goto e_head;
	}

  pib_widget_realizer_reset(&realizer);
  pib_widget_accept(widget,&realizer);

  size2=buf.size;

  if (size2!=size1) {
    DVMESSAGE("something went wrong: %u %u",size2,size1);
    goto e_size;
  }
#endif // ]

#if defined (PIB_WIDGET_REALIZER2) // [
  head=realizer.realizer.head;
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  param.border=border;
  param.root=widget;
#endif // ]
#endif // ]
DVWRITELN("%p",head);
  err=DialogBoxIndirectParamW(
    NULL,                 // HINSTANCE       hInstance,
    head,                 // LPCDLGTEMPLATEW hDialogTemplate,
    NULL,                 // HWND            hWndParent,
    pib_widget_dlgproc,   // DLGPROC         lpDialogFunc,
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    (LPARAM)&param        // LPARAM          dwInitParam
#else // ] [
    (LPARAM)widget        // LPARAM          dwInitParam
#endif // ]
  );
DVWRITELN("%p",head);

#if ! defined (PIB_WIDGET_REALIZER2) // [
e_size:
  free(head);
e_head:
#endif // ]
  realizer.vmt->destroy(&realizer);

  return err;
}

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#define PIB_WIDGET_DLGPROC_SIZER
#endif // ]
static INT_PTR CALLBACK pib_widget_dlgproc(HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if defined (PIB_WIDGET_DLGPROC_SIZER) // [
  PibWidgetAcceptor *acceptor
      =(PibWidgetAcceptor *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);
  PibWidgetAcceptorParam *param;

  struct {
    RECT window;
    RECT client;
  } rect;
#endif // ]
#else // ] [
  PibWidget *widget=(PibWidget *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);
  PibWidgetPhysical *physical;
#endif // ]
  HFONT hFont;
#if ! defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetAcceptor initializer;
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
  struct {
    RECT window;
    RECT client;
  } rect;
#endif // ]
#endif // ]

#if defined (DEBUG) // [
  LOGFONTW lf;
#endif // ]

  switch (uMsg) {
  case WM_INITDIALOG:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if defined (PIB_WIDGET_DLGPROC_SIZER) // [
    param=(PibWidgetAcceptorParam *)lParam;
    hFont=(HFONT)GetWindowLongPtrW(hDlg,GWLP_USERDATA);
    acceptor=calloc(1,sizeof *acceptor);
#if defined (PIB_WIDGET_REALIZER2_GEOMETRY) // [
    pib_widget_sizer_create(acceptor,-1,-1,param->border,hDlg,hFont,
        param->root);
#else // ] [
    pib_widget_sizer_create(acceptor,param->border,hDlg,hFont,param->root);
#endif // ]

    SetWindowLongPtrW(
      hDlg,               									// HWND     hWnd,
      GWLP_USERDATA,      									// int      nIndex,
      (LONG_PTR)acceptor  									// LONG_PTR dwNewLong
    );

    pib_widget_sizer_phase1(acceptor,param->root);
    pib_widget_sizer_phase2(acceptor,param->root);

    GetWindowRect(hDlg,&rect.window);

    MapWindowPoints(
      NULL,                                 // HWND    hWndFrom,
      hDlg,                                 // HWND    hWndTo,
      (LPPOINT)&rect.window,                // LPPOINT lpPoints,
      2                                     // UINT    cPoints
    );

    GetClientRect(hDlg,&rect.client);
		rect.window.right-=rect.client.right-rect.client.left;
		rect.window.bottom-=rect.client.bottom-rect.client.top;

    rect.client.left+=(rect.client.right-rect.client.left)>>1;
    rect.client.right=rect.client.left+0.5*acceptor->core.turtle.x+0.5;
    rect.client.right+=rect.window.right;
    rect.client.left-=0.5*acceptor->core.turtle.x+0.5;
    rect.client.left+=rect.window.left;

    rect.client.top+=(rect.client.bottom-rect.client.top)>>1;
    rect.client.bottom=rect.client.top+0.5*acceptor->core.turtle.y+0.5;
    rect.client.bottom+=rect.window.bottom;
    rect.client.top-=0.5*acceptor->core.turtle.y+0.5;
    rect.client.top+=rect.window.top;

    MapWindowPoints(
      hDlg,                                 // HWND    hWndFrom,
      NULL,                                 // HWND    hWndTo,
      (LPPOINT)&rect.client,                // LPPOINT lpPoints,
      2                                     // UINT    cPoints
    );

    SetWindowPos(
      hDlg,                                 // HWND hWnd,
      HWND_NOTOPMOST,                       // HWND hWndInsertAfter,
      rect.client.left,                     // int  X,
      rect.client.top,                      // int  Y,
      rect.client.right-rect.client.left,   // int  cx,
      rect.client.bottom-rect.client.top,   // int  cy,
      SWP_NOACTIVATE|SWP_NOZORDER           // UINT uFlags
    );
#endif // ]
#else // ] [
    hFont=(HFONT)GetWindowLongPtrW(hDlg,GWLP_USERDATA);
    widget=(PibWidget *)lParam;

    SetWindowLongPtrW(
      hDlg,               // HWND     hWnd,
      GWLP_USERDATA,      // int      nIndex,
      lParam              // LONG_PTR dwNewLong
    );

    pib_widget_initializer_create(&initializer,hDlg,hFont);
DWRITELN("||||||||||||||||||||||||||||||||||");
    widget->vmt->accept(widget,&initializer);
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
DWRITELN("||||||||||||||||||||||||||||||||||");
    pib_widget_initializer_reset(&initializer);
DWRITELN("||||||||||||||||||||||||||||||||||");
    widget->vmt->accept(widget,&initializer);
DWRITELN("||||||||||||||||||||||||||||||||||");

    GetWindowRect(hDlg,&rect.window);
    GetClientRect(hDlg,&rect.client);

#if 0 // [
    if (initializer.initializer.x<rect.right-rect.left) {
DVWRITELN("!!! %d (%ld)",rect.right-rect.left,initializer.initializer.x);
      rect.left+=initializer.initializer.x>>1;
DVWRITELN("!!! %d (%ld)",rect.right-rect.left,initializer.initializer.x);
      rect.right-=initializer.initializer.x>>1;
DVWRITELN("!!! %d (%ld)",rect.right-rect.left,initializer.initializer.x);
    }
    else {
DVWRITELN("!!! %d (%ld)",rect.right-rect.left,initializer.initializer.x);
      rect.left-=initializer.initializer.x>>1;
DVWRITELN("!!! %d (%ld)",rect.right-rect.left,initializer.initializer.x);
      rect.right+=initializer.initializer.x>>1;
DVWRITELN("!!! %d (%ld)",rect.right-rect.left,initializer.initializer.x);
    }
#else // ] [
      rect.client.left+=(rect.client.right-rect.client.left)>>1;
#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
      rect.client.right=rect.client.left+0.5*initializer.initializer.x+0.5;
      rect.client.left-=0.5*initializer.initializer.x+0.5;
#else // ] [
      rect.client.right=rect.client.left+(initializer.initializer.x>>1);
      rect.client.left-=initializer.initializer.x>>1;
#endif // ]
#endif // ]

#if 0 // [
    if (initializer.initializer.y<rect.bottom-rect.top) {
DVWRITELN("!!! %d (%ld)",rect.bottom-rect.top,initializer.initializer.y);
      rect.top+=initializer.initializer.y>>1;
DVWRITELN("!!! %d (%ld)",rect.bottom-rect.top,initializer.initializer.y);
      rect.bottom-=initializer.initializer.y>>1;
DVWRITELN("!!! %d",rect.bottom-rect.top);
    }
    else {
DVWRITELN("!!! %d (%ld)",rect.bottom-rect.top,initializer.initializer.y>>1);
      rect.top-=initializer.initializer.y>>1;
      rect.bottom+=initializer.initializer.y>>1;
DVWRITELN("!!! %d",rect.bottom-rect.top);
    }
#else // ] [
      rect.client.top+=(rect.client.bottom-rect.client.top)>>1;
#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
      rect.client.bottom=rect.client.top+0.5*initializer.initializer.y+0.5;
      rect.client.top-=0.5*initializer.initializer.y+0.5;
#else // ] [
      rect.client.bottom=rect.client.top+(initializer.initializer.y>>1);
      rect.client.top-=initializer.initializer.y>>1;
#endif // ]
#endif // ]

#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
DVWRITELN("!!! cx:%d (%.0lf) cy:%d (%.0lf)",rect.client.right-rect.client.left,initializer.initializer.x+0.5,rect.client.bottom-rect.client.top,initializer.initializer.y+0.5);
#else // ] [
DVWRITELN("!!! cx:%d (%ld) cy:%d (%ld)",rect.client.right-rect.client.left,initializer.initializer.x,rect.client.bottom-rect.client.top,initializer.initializer.y);
#endif // ]

  MapWindowPoints(
    hDlg,                     // HWND    hWndFrom,
    NULL,                     // HWND    hWndTo,
    (LPPOINT)&rect.client,    // LPPOINT lpPoints,
    2                         // UINT    cPoints
  );
#if defined (PIB_WIDGET_INTITALIZER_POSITION_DOUBLE) // [
DVWRITELN("!!! cx:%d (%.0lf) cy:%d (%.0lf)",rect.client.right-rect.client.left,initializer.initializer.x+0.5,rect.client.bottom-rect.client.top,initializer.initializer.y+0.5);
#else // ] [
DVWRITELN("!!! cx:%d (%ld) cy:%d (%ld)",rect.client.right-rect.client.left,initializer.initializer.x,rect.client.bottom-rect.client.top,initializer.initializer.y);
#endif // ]

    SetWindowPos(
      hDlg,                               // HWND hWnd,
      HWND_NOTOPMOST,                     // HWND hWndInsertAfter,
      rect.client.left,                          // int  X,
      rect.client.top,                           // int  Y,
      rect.client.right-rect.client.left,               // int  cx,
      rect.client.bottom-rect.client.top,               // int  cy,
      SWP_NOACTIVATE|SWP_NOZORDER         // UINT uFlags
    );
#endif // ]
    initializer.vmt->destroy(&initializer);
#endif // ]
    return TRUE;
  case WM_SIZE:
DWRITELN("**********************************");
    break;
  case WM_SETFONT:
#if defined (DEBUG) // [
DWRITELN("##################################");
    hFont=(HFONT)wParam;

    GetObjectW (
       hFont,                      // HANDLE h,
       sizeof (LOGFONTW),           // int    c,
       &lf                         // LPVOID pv
    ) ;

DVWRITELNW("### hFont:%p lfFaceName:\"%s\"",hFont,lf.lfFaceName);
#endif // ]

    /*
     * The best time for the owner of a dialog box control to set the
     * font of the control is when it receives the WM_INITDIALOG message.
     * The application should call the DeleteObject function to delete
     * the font when it is no longer needed; for example, after it destroys
     * the control.
     *
     * The size of the control does not change as a result of receiving
     * this message. To avoid clipping text that does not fit within the
     * boundaries of the control, the application should correct the size
     * of the control window before it sets the font.
     *
     * Unfortunately, the WM_SETFONT message arrives in front of the
     * WM_INITDIALOG message. Hence we need to intermediate store hFont.
     */
    SetWindowLongPtrW(
      hDlg,               // HWND     hWnd,
      GWLP_USERDATA,      // int      nIndex,
      wParam              // LONG_PTR dwNewLong
    );

    break;
  case WM_COMMAND:
    switch (wParam) {
    case IDCANCEL:
      EndDialog(hDlg,0);
      return TRUE;
    default:
#if 0 // [
      physical=(PibWidgetPhysical *)GetWindowLongPtrW(
        (HWND)lParam,     // HWND hWnd,
        GWLP_USERDATA     // int  nIndex
      );

      if (!physical)
        break;

      return TRUE;
#else // ] [
      break;
#endif // ]
    }
  default:
    break;
  }

  return FALSE;
}
