/*
 * pib_widget_sizer.h
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if 0 // [
typedef const struct _PibWidgetRealizerMap PibWidgetRealizerMap;

struct _PibWidgetRealizerMap {
  double *(*tx)(PibWidgetTurtle *turtle);
  double *(*ty)(PibWidgetTurtle *turtle);
  long *(*cx)(SIZE *size);
  long *(*cy)(SIZE *size);
  EPibWidgetMode *(*mx)(PibWidgetMode *mode);
  EPibWidgetMode *(*my)(PibWidgetMode *mode);
};
#endif // ]

static PibWidgetAcceptorVmt vmt;
static PibWidgetRealizerMap pib_map_affine;
static PibWidgetRealizerMap pib_map_twisted;

#if defined (PIB_WIDGET_REALIZER2_GEOMETRY) // [
int pib_widget_sizer_create(PibWidgetAcceptor *acceptor, long cx, long cy,
    long border, HWND hDlg, HFONT hFont, PibWidget *root)
#else // ] [
int pib_widget_sizer_create(PibWidgetAcceptor *acceptor, int border,
    HWND hDlg, HFONT hFont, PibWidget *root)
#endif // ]
{
  memset(acceptor,0,sizeof *acceptor);
  acceptor->vmt=&vmt;

#if defined (PIB_WIDGET_REALIZER2_GEOMETRY) // [
  if (pib_widget_sizer_core_create(&acceptor->core,cx,cy,border,hDlg,hFont)<0) {
    DMESSAGE("creating core");
    goto e_core;
  }
#else // ] [
  if (pib_widget_sizer_core_create(&acceptor->core,border,hDlg,hFont)<0) {
    DMESSAGE("creating core");
    goto e_core;
  }
#endif // ]

  acceptor->sizer.root=root;
  acceptor->sizer.realizing=0;

  return 0;
//cleanup:
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  acceptor->core.vmt->destroy(&acceptor->core);
#else // ] [
  pib_widget_sizer_core_destroy(&acceptor->core);
#endif // ]
e_core:
  return -1;
}

void pib_widget_sizer_phase1(PibWidgetAcceptor *acceptor, PibWidget *widget)
{
  PibWidgetRealizerCore *core=&acceptor->core;

  core->turtle.x=core->border*core->unit.cx/4;
  core->turtle.y=core->border*core->unit.cy/8;
  pib_widget_accept(widget,acceptor);
  core->turtle.x+=core->border*core->unit.cx/4;
  core->turtle.y+=core->border*core->unit.cy/8;
}

void pib_widget_sizer_phase2(PibWidgetAcceptor *acceptor, PibWidget *widget)
{
  PibWidgetRealizerCore *core=&acceptor->core;

  if (!acceptor->sizer.realizing)
    ++acceptor->sizer.realizing;

  core->turtle.x=core->border*core->unit.cx/4;
  core->turtle.y=core->border*core->unit.cy/8;
  pib_widget_accept(widget,acceptor);
  core->turtle.x+=core->border*core->unit.cx/4;
  core->turtle.y+=core->border*core->unit.cy/8;
}

#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
static void pib_widget_sizer_dispatch_treeview(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
  acceptor->core.vmt->dispatch_treeview(acceptor,widget);
}

static void pib_widget_sizer_dispatch_combobox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
  acceptor->core.vmt->dispatch_combobox(acceptor,widget);
}

static void pib_widget_sizer_dispatch_button(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
  acceptor->core.vmt->dispatch_button(acceptor,widget);
}
#else // ] [
static void pib_widget_sizer_dispatch_physical(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
  PibWidgetRealizerCore *core=&acceptor->core;
  PibWidgetSizer *sizer=&acceptor->sizer;
  SIZE *geometry=widget->vmt->geometry(widget);

  if (sizer->realizing) {
    SetWindowPos(
      PIB_WIDGET_PHYSICAL(widget)->hWnd,  // HWND hWnd,
      HWND_NOTOPMOST,                     // HWND hWndInsertAfter,
      core->turtle.x+0.5,                 // int  X,
      core->turtle.y+0.5,                 // int  Y,
      geometry->cx,                       // int  cx,
      geometry->cy,                       // int  cy,
      SWP_NOACTIVATE|SWP_NOZORDER         // UINT uFlags
    );
  }
  else {
    PIB_WIDGET_PHYSICAL(widget)->hWnd
        =GetDlgItem(core->hDlg,PIB_WIDGET_PHYSICAL(widget)->idc);
    widget->vmt->extent(widget,acceptor,geometry);
  }

  core->turtle.x+=geometry->cx;
  core->turtle.y+=geometry->cy;
}

static void pib_widget_sizer_dispatch_combobox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
  PibWidgetComboboxMap *map;

  pib_widget_sizer_dispatch_physical(acceptor,widget);

  if (!acceptor->sizer.realizing&&PIB_WIDGET_COMBOBOX(widget)->map) {
    for (map=PIB_WIDGET_COMBOBOX(widget)->map;map->label;++map) {
      SendMessageW(PIB_WIDGET_COMBOBOX(widget)->hWnd,CB_INSERTSTRING,-1,
          (LPARAM)map->label);
    }

    SendMessageW(PIB_WIDGET_COMBOBOX(widget)->hWnd,CB_SETCURSEL,0,0);
  }
}

static void pib_widget_sizer_dispatch_box(PibWidgetAcceptor *acceptor,
    PibWidget *widget, PibWidgetRealizerMap *map)
{
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  PibWidgetRealizerCore *core=&acceptor->core;
#endif // ]
  PibWidgetRealizer *realizer=&acceptor->realizer;
  SIZE *geometry=widget->vmt->geometry(widget);
  int realizing UNUSED=realizer->buf.head?1:0;
  long delta=PIB_WIDGET_BOX(widget)->delta;
  PibWidgetNode *cur;
  double x,y,cx;

  if (!realizing) {
    // initialize the box's size.
    *map->cx(geometry)=0;
    *map->cy(geometry)=0;
  }

  // remember "y".
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  y=*map->ty(&core->turtle);
#else // ] [
  y=*map->ty(&realizer->turtle);
#endif // ]

  for (cur=PIB_WIDGET_LIST(widget)->head;cur;cur=cur->next) {
    // remember "x".
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    x=*map->tx(&core->turtle); // {
#else // ] [
    x=*map->tx(&realizer->turtle); // {
#endif // ]

    if (realizing) {
      cx=*map->cx(widget->vmt->geometry(widget));
      cx-=*map->cx(cur->vmt->geometry(PIB_WIDGET(cur)));

      switch (*map->mx(&cur->mode)) {
      case PIB_WIDGET_NONE:
        break;
      case PIB_WIDGET_BEGIN:
        break;
      case PIB_WIDGET_CENTER:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
        *map->tx(&core->turtle)+=cx/2.0;
#else // ] [
        *map->tx(&realizer->turtle)+=cx/2.0;
#endif // ]
        break;
      case PIB_WIDGET_END:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
        *map->tx(&core->turtle)+=cx;
#else // ] [
        *map->tx(&realizer->turtle)+=cx;
#endif // ]
        break;
      default:
        break;
      }
    }

    cur->vmt->accept(PIB_WIDGET(cur),acceptor);

    if (realizing) {
      switch (*map->mx(&cur->mode)) {
      case PIB_WIDGET_NONE:
        break;
      case PIB_WIDGET_BEGIN:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
        *map->tx(&core->turtle)+=cx;
#else // ] [
        *map->tx(&realizer->turtle)+=cx;
#endif // ]
        break;
      case PIB_WIDGET_CENTER:
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
        *map->tx(&core->turtle)+=cx/2.0;
#else // ] [
        *map->tx(&realizer->turtle)+=cx/2.0;
#endif // ]
        break;
      case PIB_WIDGET_END:
        break;
      default:
        DMESSAGE("something went wrong");
        break;
      }
    }

    // in x-direction, we take into account just the maximum.
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    cx=*map->tx(&core->turtle)-x;
#else // ] [
    cx=*map->tx(&realizer->turtle)-x;
#endif // ]

    if (*map->cx(geometry)<cx)
      *map->cx(geometry)=cx+0.5;

    // in y-direction, the box is growing by each step.
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    *map->cy(geometry)=(*map->ty(&core->turtle)-y)+0.5;
#else // ] [
    *map->cy(geometry)=(*map->ty(&realizer->turtle)-y)+0.5;
#endif // ]

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
    if (cur->next)
      *map->ty(&core->turtle)+=delta;

    // restore "x".
    *map->tx(&core->turtle)=x; // }
#else // ] [
    if (cur->next)
      *map->ty(&realizer->turtle)+=delta;

    // restore "x".
    *map->tx(&realizer->turtle)=x; // }
#endif // ]
  }

  // "x" is still restored to it's initial value, hence in x-direction we
  // need to add "cx".
#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
  *map->tx(&core->turtle)+=*map->cx(geometry);
#else // ] [
  *map->tx(&realizer->turtle)+=*map->cx(geometry);
#endif // ]
}
#endif // ]

static void pib_widget_sizer_dispatch_hbox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  pib_widget_realizer_core_dispatch_box(acceptor,widget,&pib_map_twisted);
#else // ] [
  pib_widget_sizer_dispatch_box(acceptor,widget,&pib_map_twisted);
#endif // ]
}

static void pib_widget_sizer_dispatch_vbox(PibWidgetAcceptor *acceptor,
    PibWidget *widget)
{
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  pib_widget_realizer_core_dispatch_box(acceptor,widget,&pib_map_affine);
#else // ] [
  pib_widget_sizer_dispatch_box(acceptor,widget,&pib_map_affine);
#endif // ]
}

static void pib_widget_sizer_destroy(PibWidgetAcceptor *acceptor)
{
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  acceptor->core.vmt->destroy(&acceptor->core);
#else // ] [
  pib_widget_sizer_core_destroy(&acceptor->core);
#endif // ]
}

static PibWidgetAcceptorVmt vmt={
  .id="PIB_SIZER",
  .destroy=pib_widget_sizer_destroy,
  .dispatch_combobox=pib_widget_sizer_dispatch_combobox,
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  .dispatch_treeview=pib_widget_sizer_dispatch_treeview,
  .dispatch_button=pib_widget_sizer_dispatch_button,
#else // ] [
  .dispatch_treeview=pib_widget_sizer_dispatch_physical,
  .dispatch_button=pib_widget_sizer_dispatch_physical,
#endif // ]
  .dispatch_hbox=pib_widget_sizer_dispatch_hbox,
  .dispatch_vbox=pib_widget_sizer_dispatch_vbox,
};

////////
static double *_pib_map_affine_tx(PibWidgetTurtle *turtle)
{
  return &turtle->x;
}

static double *_pib_map_affine_ty(PibWidgetTurtle *turtle)
{
  return &turtle->y;
}

static long *_pib_map_affine_cx(SIZE *size)
{
  return &size->cx;
}

static long *_pib_map_affine_cy(SIZE *size)
{
  return &size->cy;
}

static EPibWidgetMode *_pib_map_affine_mx(PibWidgetMode *mode)
{
  return &mode->x;
}

static EPibWidgetMode *_pib_map_affine_my(PibWidgetMode *mode)
{
  return &mode->y;
}

static PibWidgetRealizerMap pib_map_affine={
  .tx=_pib_map_affine_tx,
  .ty=_pib_map_affine_ty,
  .cx=_pib_map_affine_cx,
  .cy=_pib_map_affine_cy,
  .mx=_pib_map_affine_mx,
  .my=_pib_map_affine_my,
};

////////
static double *_pib_map_twisted_tx(PibWidgetTurtle *turtle)
{
  return &turtle->y;
}

static double *_pib_map_twisted_ty(PibWidgetTurtle *turtle)
{
  return &turtle->x;
}

static long *_pib_map_twisted_cx(SIZE *size)
{
  return &size->cy;
}

static long *_pib_map_twisted_cy(SIZE *size)
{
  return &size->cx;
}

static EPibWidgetMode *_pib_map_twisted_mx(PibWidgetMode *mode)
{
  return &mode->y;
}

static EPibWidgetMode *_pib_map_twisted_my(PibWidgetMode *mode)
{
  return &mode->x;
}

static PibWidgetRealizerMap pib_map_twisted={
  .tx=_pib_map_twisted_tx,
  .ty=_pib_map_twisted_ty,
  .cx=_pib_map_twisted_cx,
  .cy=_pib_map_twisted_cy,
  .mx=_pib_map_twisted_mx,
  .my=_pib_map_twisted_my,
};
#endif // ]
