/*
 * pib_widget_sizer_core.h
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

#if defined (PIB_WIDGET_REALIZER2_MORPH) // [
#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
static PibWidgetRealizerCoreVmt vmt;
#endif // ]

#if defined (PIB_WIDGET_REALIZER2_GEOMETRY) // [
int pib_widget_sizer_core_create(PibWidgetRealizerCore *core, long cx,
    long cy, long border, HWND hDlg, HFONT hFont)
#else // ] [
int pib_widget_sizer_core_create(PibWidgetRealizerCore *core, long border,
    HWND hDlg, HFONT hFont)
#endif // ]
{
  RECT rect;

#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
  core->vmt=&vmt;
#endif // ]
#if defined (PIB_WIDGET_REALIZER2_GEOMETRY) // [
  core->geometry.cx=cx;
  core->geometry.cy=cy;
#endif // ]
  core->border=border;
  core->hDlg=hDlg;

  /*
   * After painting with a common DC, the ReleaseDC function must be called
   * to release the DC
   */
  core->hDc=GetDC(hDlg);

  if (!core->hDc) {
    DMESSAGE("getting device context");
    goto e_hdc;
  }

  core->font.hCur=hFont;
  core->font.hPrev=(HFONT)SelectObject(core->hDc,core->font.hCur);

  if (!core->font.hPrev) {
    DMESSAGE("selecting font");
    goto e_font_prev;
  }

  /*
   * Q145994: HOWTO: Calculate Dialog Units When Not Using the System Font
   *   https://jeffpar.github.io/kbarchive/kb/145/Q145994/
   */
  rect.left=0;
  rect.top=0;
  rect.right=4000;
  rect.bottom=8000;
  MapDialogRect(hDlg,&rect);
  core->unit.cx=(double)(rect.right-rect.left)/1000;
  core->unit.cy=(double)(rect.bottom-rect.top)/1000;

  return 0;
//cleanup:
  SelectObject(core->hDc,core->font.hPrev);
e_font_prev:
  ReleaseDC(core->hDlg,core->hDc);
e_hdc:
  return -1;
}

#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
static void pib_widget_sizer_core_destroy(PibWidgetRealizerCore *core)
#else // ] [
void pib_widget_sizer_core_destroy(PibWidgetRealizerCore *core)
#endif // ]
{
  SelectObject(core->hDc,core->font.hPrev);
  ReleaseDC(core->hDlg,core->hDc);
}

#if defined (PIB_WIDGET_REALIZER_CORE_VMT) // [
static int pib_widget_sizer_core_realizing(PibWidgetAcceptor *acceptor)
{
  return acceptor->sizer.realizing;
}

static void pib_widget_sizer_core_dispatch_physical(
    PibWidgetAcceptor *acceptor, PibWidget *widget)
{
  PibWidgetRealizerCore *core=&acceptor->core;
  PibWidgetSizer *sizer=&acceptor->sizer;
  SIZE *geometry=widget->vmt->geometry(widget);

  if (sizer->realizing) {
    SetWindowPos(
      PIB_WIDGET_PHYSICAL(widget)->hWnd,  // HWND hWnd,
      HWND_NOTOPMOST,                     // HWND hWndInsertAfter,
      core->turtle.x+0.5,                 // int  X,
      core->turtle.y+0.5,                 // int  Y,
      geometry->cx,                       // int  cx,
      geometry->cy,                       // int  cy,
      SWP_NOACTIVATE|SWP_NOZORDER         // UINT uFlags
    );
  }
  else {
    PIB_WIDGET_PHYSICAL(widget)->hWnd
        =GetDlgItem(core->hDlg,PIB_WIDGET_PHYSICAL(widget)->idc);
    widget->vmt->extent(widget,acceptor,geometry);
  }

  core->turtle.x+=geometry->cx;
  core->turtle.y+=geometry->cy;
}

static void pib_widget_sizer_core_dispatch_combobox(
    PibWidgetAcceptor *acceptor, PibWidget *widget)
{
  PibWidgetComboboxMap *map;

  pib_widget_sizer_core_dispatch_physical(acceptor,widget);

  if (!acceptor->sizer.realizing&&PIB_WIDGET_COMBOBOX(widget)->map) {
    for (map=PIB_WIDGET_COMBOBOX(widget)->map;map->label;++map) {
      SendMessageW(PIB_WIDGET_COMBOBOX(widget)->hWnd,CB_INSERTSTRING,-1,
          (LPARAM)map->label);
    }

    SendMessageW(PIB_WIDGET_COMBOBOX(widget)->hWnd,CB_SETCURSEL,0,0);
  }
}

static PibWidgetRealizerCoreVmt vmt={
  .destroy=pib_widget_sizer_core_destroy,
  .realizing=pib_widget_sizer_core_realizing,
  .dispatch_treeview=pib_widget_sizer_core_dispatch_physical,
  .dispatch_combobox=pib_widget_sizer_core_dispatch_combobox,
  .dispatch_button=pib_widget_sizer_core_dispatch_physical,
};
#endif // ]
#endif // ]
