/*
 * pib_widget_treeview.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib_widget.h>

#if defined (PIB_WIDGET_REALIZER2) // [
PibWidget *pib_widget_treeview_new(long cx, long cy)
{
  PibWidget *widget=pib_widget_physical_new(pib_widget_treeview_vmt(),
      -1,-1,NULL);

  PIB_WIDGET_TREEVIEW(widget)->design.cx=cx;
  PIB_WIDGET_TREEVIEW(widget)->design.cy=cy;

  return widget;
}

static void _pib_widget_treeview_extent(PibWidget *widget,
    PibWidgetAcceptor *acceptor, SIZE *size)
{
  RECT rect;

  if (acceptor->core.hDlg) {
    rect.left=0;
    rect.top=0;
    rect.right=PIB_WIDGET_TREEVIEW(widget)->design.cx;
    rect.bottom=PIB_WIDGET_TREEVIEW(widget)->design.cy;
    MapDialogRect(acceptor->core.hDlg,&rect);
    size->cx=rect.right-rect.left;
    size->cy=rect.bottom-rect.top;
  }
  else {
    size->cx=(double)PIB_WIDGET_TREEVIEW(widget)->design.cx
        *acceptor->core.unit.cx/4;
    size->cy=(double)PIB_WIDGET_TREEVIEW(widget)->design.cy
        *acceptor->core.unit.cy/8;
  }
}
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
static void _pib_widget_treeview_extenti(PibWidget *widget,
    PibWidgetPetzold *petzold, SIZE *size)
{
  const SIZE *geometry=&PIB_WIDGET_PHYSICAL(widget)->geometry;
  RECT rect;

  *size=*geometry;
  rect.left=0;              // DLU
  rect.top=0;               // DLU
  rect.right=size->cx;      // DLU
  rect.bottom=size->cy;     // DLU

  /*
   * The MapDialogRect function assumes that the initial coordinates in
   * the RECT structure represent dialog box units. 
   */
  MapDialogRect(
    petzold->hDlg,          // HWND   hDlg,
    &rect                   // LPRECT lpRect
  );

  size->cx=rect.right-rect.left;
  size->cy=rect.bottom-rect.top;
}
#endif // ]

#if defined (PIB_WIDGET_PETZOLD2) // [
static void _pib_widget_treeview_extent(PibWidget *widget,
    PibWidgetPetzold *petzold, SIZE *size)
#elif defined (PIB_WIDGET_EXTENT_POINT_SIZE) // ] [
static void _pib_widget_treeview_extent(PibWidget *widget, int point_size,
    SIZE *size)
#else // ] [
static void _pib_widget_treeview_extent(PibWidget *widget, HDC hDc, 
    int point_size, SIZE *size)
#endif // ]
{
  const SIZE *geometry=&PIB_WIDGET_PHYSICAL(widget)->geometry;

  if (geometry!=size)
    *size=*geometry;
}
#endif // ]

static void _pib_widget_treeview_accept(PibWidget *widget,
    PibWidgetAcceptor *acceptor)
{
  acceptor->vmt->dispatch_treeview(acceptor,widget);
}

PibWidgetVmt *pib_widget_treeview_vmt(void)
{
  static PibWidgetVmt vmt;
  PibWidgetVmt *prev;

  if (!vmt.destroy) {
    prev=pib_widget_physical_vmt();
    vmt=*prev;
    vmt.id=PIB_WIDGET_ID_TREEVIEW;
#if defined (PIB_WIDGET_REALIZER2) // [
    vmt.size=sizeof (PibWidgetTreeview);
#endif // ]
    vmt.prev=prev;
#if defined (PIB_WIDGET_REALIZER2) // [
    vmt.extent=_pib_widget_treeview_extent;
#else // ] [
#if defined (PIB_WIDGET_INTITALIZER_ARRANGE) // [
    vmt.extenti=_pib_widget_treeview_extenti;
#endif // ]
    vmt.extent=_pib_widget_treeview_extent;
#endif // ]
    vmt.accept=_pib_widget_treeview_accept;
  }

  return &vmt;
}
