/*
 * gen_rand.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
/*
 * Plug-in Developer
 *   http://wiki.winamp.com/wiki/Plug-in_Developer
 */
//#define GEN_YAS_LOCALE
#if defined (GEN_YAS_LOCALE) // [
#include <locale.h>
#endif // ]
#include <yas.h>
#include <yas/resource.h>

#if ! defined (PIB_GEN) // [
///////////////////////////////////////////////////////////////////////////////
__declspec (dllexport)
winampGeneralPurposePlugin *winampGetGeneralPurposePlugin();
#endif // ]

///////////////////////////////////////////////////////////////////////////////
Yas yas;

#if 0 // [
///////////////////////////////////////////////////////////////////////////////
BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call,
    LPVOID lpReserved)
{
  return TRUE;
}
#endif // ]

/*
 * play and stop wm_command
 *   http://forums.winamp.com/showthread.php?t=307877
 * List of Winamp's CMD + WM_COMMAND
 *   http://forums.winamp.com/showpost.php?p=3175906&postcount=2
 */
static yas_rand_t yas_pos(Yas *yas, HWND hWnd)
{
  yas_rand_t size;
  yas_rand_t pos;
  int playing;
  int visualizing;

  size=(yas_rand_t)CallWindowProcW(
    yas->wndproc.wa,            // _In_ WNDPROC lpPrevWndFunc,
    hWnd,                       // _In_ HWND    hWnd,
    WM_WA_IPC,                  // _In_ UINT    Msg,
    0,                          // _In_ WPARAM  wParam,
    IPC_GETLISTLENGTH           // _In_ LPARAM  lParam
  );

  if (!yas->shuffle||size!=yas->size) {
DWRITELN("!!! HERE WE GO !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
DVWRITELN("!!! shuffle:%p",yas->shuffle);
DVWRITELN("!!! size:%u %u",size,yas->size);
    yas_reload(yas,size,yas_cons(yas)->mode);
    pos=yas->shuffle[yas->cur];

    /*
     * SendMessage(hwnd_winamp,WM_WA_IPC,position,IPC_SETPLAYLISTPOS)
     * IPC_SETPLAYLISTPOS sets the playlist position to the specified
     * 'position'. It will not change playback status or anything else.
     * It will just set the current position in the playlist and will
     * update the playlist view if necessary.
     *
     * Unfortunately, we're doing something wrong.
     */
    CallWindowProcW(
      yas->wndproc.wa,          // _In_ WNDPROC lpPrevWndFunc,
      hWnd,                     // _In_ HWND    hWnd,
      WM_WA_IPC,                // _In_ UINT    Msg,
      pos,                      // _In_ WPARAM  wParam,
      IPC_SETPLAYLISTPOS        // _In_ LPARAM  lParam
    );

    /*
     * following is the way we've found to work around the above limitation.
     *
     * 1) are we playing?
     */
    playing=CallWindowProcW(
      yas->wndproc.wa,          // _In_ WNDPROC lpPrevWndFunc,
      hWnd,                     // _In_ HWND    hWnd,
      WM_WA_IPC,                // _In_ UINT    Msg,
      0,                        // _In_ WPARAM  wParam,
      IPC_ISPLAYING             // _In_ LPARAM  lParam
    );

    if (playing) {
      /*
       * 1) in case we're playing stop playing.
       */
      CallWindowProcW(
        yas->wndproc.wa,        // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                   // _In_ HWND    hWnd,
        WM_COMMAND,             // _In_ UINT    Msg,
        WINAMP_STOP,            // _In_ WPARAM  wParam,
        0                       // _In_ LPARAM  lParam
      );

      /*
       * 2) enqueue start playing again.
       *
       * NOTE: when processing the message we've to position again!
       */
      PostMessageW(
        hWnd,                   // _In_ HWND    hWnd,
        WM_COMMAND,             // _In_ UINT    Msg,
        WINAMP_PLAY,            // _In_ WPARAM  wParam,
        0                       // _In_ LPARAM  lParam
      );
    }
    else {
      /*
       * 1) start playing.
       */
      CallWindowProcW(
        yas->wndproc.wa,        // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                   // _In_ HWND    hWnd,
        WM_COMMAND,             // _In_ UINT    Msg,
        WINAMP_PLAY,            // _In_ WPARAM  wParam,
        0                       // _In_ LPARAM  lParam
      );

      /*
       * 2) is a visualizer running?
       */
      visualizing=CallWindowProcW(
        yas->wndproc.wa,        // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                   // _In_ HWND    hWnd,
        WM_WA_IPC,              // _In_ UINT    Msg,
        0,                      // _In_ WPARAM  wParam,
        IPC_ISVISRUNNING        // _In_ LPARAM  lParam
      );

      /*
       * 3) immediatly stop playing.
       */
      CallWindowProcW(
        yas->wndproc.wa,        // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                   // _In_ HWND    hWnd,
        WM_COMMAND,             // _In_ UINT    Msg,
        WINAMP_STOP,            // _In_ WPARAM  wParam,
        0                       // _In_ LPARAM  lParam
      );

      if (visualizing) {
        /*
         * 4) if a visualizer was started enqueue stopping it.
         */
        PostMessageW(
          hWnd,                 // _In_ HWND    hWnd,
          WM_COMMAND,           // _In_ UINT    Msg,
          WINAMP_VISTOGGLE,     // _In_ WPARAM  wParam,
          0                     // _In_ LPARAM  lParam
        );
      }
    }
  }
  else
    pos=yas->shuffle[yas->cur];

  return pos;
}

static LRESULT CALLBACK YasWaProc(HWND hWnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  int active=yas.variables.cons.active;

  active&=CallWindowProcW(
    yas.wndproc.wa,               // _In_ WNDPROC lpPrevWndFunc,
    hWnd,                         // _In_ HWND    hWnd,
    WM_WA_IPC,                    // _In_ UINT    Msg,
    0,                            // _In_ WPARAM  wParam,
    IPC_GET_SHUFFLE               // _In_ LPARAM  lParam
  );

  if (!active)
    goto e_active;

  switch (uMsg) {
  case WM_PIB_IPC:
    switch (wParam) {
    case YAS_INIT: {
      int visualizing=CallWindowProcW(
        yas.wndproc.wa,           // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                     // _In_ HWND    hWnd,
        WM_WA_IPC,                // _In_ UINT    Msg,
        0,                        // _In_ WPARAM  wParam,
        IPC_ISVISRUNNING          // _In_ LPARAM  lParam
      );

      if (visualizing) {
        PostMessageW(
          hWnd,                   // _In_ HWND    hWnd,
          WM_COMMAND,             // _In_ UINT    Msg,
          WINAMP_VISTOGGLE,       // _In_ WPARAM  wParam,
          0                       // _In_ LPARAM  lParam
        );
      }

      break;
    }
    default:
DVWRITELN("%u",wParam);
      break;
    }

    break;
  case WM_COMMAND:
    switch (wParam) {
    case 40045:
      if (yas.shuffle) {
        yas_rand_t pos=yas.shuffle[yas.cur];

        CallWindowProcW(
          yas.wndproc.wa,         // _In_ WNDPROC lpPrevWndFunc,
          hWnd,                   // _In_ HWND    hWnd,
          WM_WA_IPC,              // _In_ UINT    Msg,
          pos,                    // _In_ WPARAM  wParam,
          IPC_SETPLAYLISTPOS      // _In_ LPARAM  lParam
        );
      }

      break;
    default:
      if (40000<=wParam)
DVWRITELN("wParam:%u",wParam);
      break;
    }

    break;
  case WM_WA_IPC:
    switch (lParam) {
    case IPC_GETLISTPOS: {
      /*
       * this is most essential in the first place to get everything correct.
       */
#if defined (DEBUG) // [
      yas_rand_t ref=yas.shuffle?yas.shuffle[yas.cur]:UINT32_MAX;
#endif // ]
      yas_rand_t pos=yas_pos(&yas,hWnd);

#if defined (DEBUG) // [
      if (ref!=pos)
DVWRITELN("*** IPC_GETLISTPOS: %u",pos);
#endif // ]

      return pos;
    }
    case IPC_GET_PLAYING_FILENAME: {
      /*
       * this is also essential in order to make weverything work smooth when
       * start with playing.
       */
#if defined (DEBUG) // [
      enum { NCHARS=(sizeof yas.file)/(sizeof yas.file[0])-1 };
#endif // ]
      yas_rand_t pos=yas_pos(&yas,hWnd);
      wchar_t *path;

      path=(wchar_t *)CallWindowProcW(
        yas.wndproc.wa,         // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                   // _In_ HWND    hWnd,
        WM_WA_IPC,              // _In_ UINT    Msg,
        pos,                    // _In_ WPARAM  wParam,
        IPC_GETPLAYLISTFILEW    // _In_ LPARAM  lParam
      );

#if defined (DEBUG) // [
      if (wcscmp(yas.file,path)) {
DVWRITELNW("*** IPC_GET_PLAYING_FILENAME: %u %u \"%s\"",yas.cur,wParam,path);
        wcsncpy(yas.file,path,NCHARS);
        yas.file[NCHARS]=L'\0';
      }
#endif // ]

      return (LRESULT)path;
    }
    case IPC_PLAYLIST_MODIFIED:
DWRITELNW("!!! IPC_PLAYLIST_MODIFIED");
      break;
    case IPC_GET_NEXT_PLITEM: {
      int playing;
      yas_rand_t next;

      playing=CallWindowProcW(
        yas.wndproc.wa,           // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                     // _In_ HWND    hWnd,
        WM_WA_IPC,                // _In_ UINT    Msg,
        0,                        // _In_ WPARAM  wParam,
        IPC_ISPLAYING             // _In_ LPARAM  lParam
      );

      if (!yas.variables.cons.play&&!playing) {
        PostMessageW(
          hWnd,                   // _In_ HWND    hWnd,
          WM_COMMAND,             // _In_ UINT    Msg,
          WINAMP_STOP,            // _In_ WPARAM  wParam,
          0                       // _In_ LPARAM  lParam
        );
      }

      yas_pos(&yas,hWnd);
      next=yas_next(&yas);
DVWRITELN("*** IPC_GET_NEXT_PLITEM: %u",next);
      return next;
    }
    case IPC_GET_PREVIOUS_PLITEM: {
      int playing;
      yas_rand_t prev;

      playing=CallWindowProcW(
        yas.wndproc.wa,           // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                     // _In_ HWND    hWnd,
        WM_WA_IPC,                // _In_ UINT    Msg,
        0,                        // _In_ WPARAM  wParam,
        IPC_ISPLAYING             // _In_ LPARAM  lParam
      );

      if (!yas.variables.cons.play&&!playing) {
        PostMessageW(
          hWnd,                   // _In_ HWND    hWnd,
          WM_COMMAND,             // _In_ UINT    Msg,
          WINAMP_STOP,            // _In_ WPARAM  wParam,
          0                       // _In_ LPARAM  lParam
        );
      }

      yas_pos(&yas,hWnd);
      prev=yas_prev(&yas);
DVWRITELN("*** IPC_GET_PREVIOUS_PLITEM: %u",prev);
      return prev;
    }
    case IPC_PLAYING_FILE:
      /*
       * just a notification.
       */
DWRITELN("*** IPC_PLAYING_FILE");
      break;
    case IPC_PLAYING_FILEW:
      /*
       * just a notification.
       */
DWRITELN("*** IPC_PLAYING_FILEW");
      break;
    case IPC_GETLISTLENGTH:
      break;
    case IPC_GET_PLAYING_TITLE: {
#if 0 // [
      yas_rand_t pos=yas_pos(&yas,hWnd);
      wchar_t *title;

      title=(wchar_t *)CallWindowProcW(
        yas.wndproc.wa,         // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                   // _In_ HWND    hWnd,
        WM_WA_IPC,              // _In_ UINT    Msg,
        pos,                    // _In_ WPARAM  wParam,
        IPC_GETPLAYLISTTITLEW   // _In_ LPARAM  lParam
      );

DVWRITELNW("*** IPC_GET_PLAYING_TITLE: %u %u \"%s\"",yas.cur,wParam,title);
      return (LRESULT)title;
#else // ] [
DWRITELN("*** IPC_GET_PLAYING_TITLE");
      break;
#endif // ]
    }
    case IPC_GETPLAYLISTFILE: {
#if 0 // [
      yas_rand_t pos=yas_pos(&yas,hWnd);
      char *path;

      path=(char *)CallWindowProcW(
        yas.wndproc.wa,         // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                   // _In_ HWND    hWnd,
        WM_WA_IPC,              // _In_ UINT    Msg,
        pos,                    // _In_ WPARAM  wParam,
        IPC_GETPLAYLISTFILE     // _In_ LPARAM  lParam
      );

DVWRITELN("*** IPC_GETPLAYLISTFILE: \"%s\" (%lu/%lu)",path,wParam,yas.cur);
      return (LPARAM)path;
#else // ] [
DWRITELN("*** IPC_GETPLAYLISTFILE");
      break;
#endif // ]
    }
    case IPC_GETPLAYLISTFILEW: {
#if 0 // [
      /*
       * we assume that we're going here if and only if the playlist
       * was modified.
       */
      yas_rand_t pos=yas_pos(&yas,hWnd);
      wchar_t *path;

      path=(wchar_t *)CallWindowProcW(
        yas.wndproc.wa,         // _In_ WNDPROC lpPrevWndFunc,
        hWnd,                   // _In_ HWND    hWnd,
        WM_WA_IPC,              // _In_ UINT    Msg,
        pos,                    // _In_ WPARAM  wParam,
        IPC_GETPLAYLISTFILEW    // _In_ LPARAM  lParam
      );

DVWRITELNW("*** IPC_GETPLAYLISTFILEW: %u %u \"%s\"",yas.cur,wParam,path);
      return (LPARAM)path;
#else // ] [
DWRITELN("*** IPC_GETPLAYLISTFILEW");
      break;
#endif // ]
    }
    case IPC_GETNEXTLISTPOS: {
#if 0 // [
      yas_rand_t pos;

      pos=yas_pos(&yas,hWnd);
      pos=yas_next_preview(&yas);
DVWRITELN("*** IPC_GETNEXTLISTPOS: %u",pos);
      return pos;
#else // ] [
DWRITELN("*** IPC_GETNEXTLISTPOS");
      break;
#endif // ]
    }
    case IPC_HOOK_OKTOQUIT:
DWRITELN("*** IPC_HOOK_OKTOQUIT");
      SendMessageW(yas.hDlg,WM_PIB_IPC,PIB_CLOSE,0);
      break;
    case 2003:
    case 2005:
    case IPC_SETDIALOGBOXPARENT:
    case IPC_GETPLAYLISTTITLE:
    case IPC_GETPLAYLISTTITLEW:
    case IPC_SETPLEDITCOLORS:
    case IPC_GET_API_SERVICE:
    case IPC_CB_MISC:
    case IPC_GET_BASIC_FILE_INFOW:
    case IPC_GET_EXTENDED_FILE_INFOW_HOOKABLE:
    case IPC_CB_PEINFOTEXT:
    case IPC_HOOK_TITLES:
    case IPC_HOOK_TITLESW:
    case IPC_GETOUTPUTTIME:
    case IPC_IS_FULLSCREEN:
    case IPC_ISPLAYING:
    case IPC_CB_VISRANDOM:
    case IPC_GET_GENSKINBITMAP:
    case IPC_EMBED_ISVALID:
    case IPC_IS_AOT:
    case IPC_CB_ONSHOWWND:
    case IPC_GETLANGUAGEPACKINSTANCE:
    case IPC_GET_HMENU:
    case IPC_CB_ONTOGGLEAOT:
    case IPC_CB_GETTOOLTIP:
    case IPC_CB_GETTOOLTIPW:
    case IPC_GETTIMEDISPLAYMODE:
    case IPC_GETOUTPUTPLUGIN:
    case IPC_CB_OUTPUTCHANGED:
    case IPC_UPDATEDIALOGBOXPARENT:
    case IPC_EMBED_ENUM:
    case IPC_CB_ONHIDEWND:
    case IPC_ISWNDVISIBLE:
    case IPC_ADJUST_OPTIONSMENUPOS:
    case IPC_ADJUST_FFWINDOWSMENUPOS:
    case IPC_ADJUST_FFOPTIONSMENUPOS:
    case IPC_GETDIALOGBOXPARENT:
    case IPC_SETIDEALVIDEOSIZE:
    case IPC_STATS_LIBRARY_ITEMCNT:
    case IPC_CB_RESETFONT:
    case IPC_SETVOLUME:
    case IPC_SETPANNING:
    case IPC_GETINFO:
    case IPC_IS_PLAYING_VIDEO:
    case IPC_GETRATING:
    case IPC_SETVISWND:
    case IPC_GET_EMBEDIF:
    case IPC_GET_D3DX9:
    case IPC_GETINIDIRECTORY:
    case IPC_GETINIDIRECTORYW:
    case IPC_GETPLUGINDIRECTORY:
    case IPC_GETPLUGINDIRECTORYW:
    case IPC_GETVERSION:
    case IPC_GETWND:
    case IPC_ISVISRUNNING:
    case IPC_STOPPLAYING:
    case IPC_SPAWNFILEMENU:
    case IPC_SPAWNOPTIONSMENU:
    case IPC_WRITECONFIG:
    case IPC_GETM3UDIRECTORYW:
    case IPC_REGISTER_WINAMP_IPCMESSAGE:
    case IPC_GET_RANDFUNC:
      break;
    default:
DVWRITELN("%lu",lParam);
      break;
    }

    break;
  default:
    break;
  }
e_active:
  return CallWindowProcW(
    yas.wndproc.wa,      // _In_ WNDPROC lpPrevWndFunc,
    hWnd,                   // _In_ HWND    hWnd,
    uMsg,                   // _In_ UINT    Msg,
    wParam,                 // _In_ WPARAM  wParam,
    lParam                  // _In_ LPARAM  lParam
  );
}

///////////////////////////////////////////////////////////////////////////////
static int init()
{
  INITCOMMONCONTROLSEX iccex={
    .dwSize=sizeof iccex,
    .dwICC=ICC_TAB_CLASSES,
  };

  DCREATE();
#if defined (GEN_YAS_LOCALE) // [
	_wsetlocale(LC_ALL,LC_ALL,L"en-US");
#endif // ]
//DWRITELN("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
  InitCommonControlsEx(&iccex); 

  if (yas_create(&yas,YasWaProc)<0) {
    DMESSAGE("creating yas");
    goto e_yas;
  }

  return 0;
//cleanup:
e_yas:
  return -1;
}

static void quit()
{
  yas_destroy(&yas);
  DDESTROY();
}

#if defined (GEN_YAS_WINDOW) // [
LRESULT CALLBACK yas_main_window_procw(HWND hWnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  enum { BORDER=4 };
  RECT rect;

  switch (uMsg) {
  case WM_CREATE:
    GetClientRect(hWnd,&rect);

    CreateWindowExW(
      0,                            // DWORD     dwExStyle,
      WC_TABCONTROLW,               // LPCWSTR   lpClassName,
      L"",                          // LPCWSTR   lpWindowName,
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE, 
                                    // DWORD     dwStyle,
      BORDER,                       // int       X,
      BORDER,                       // int       Y,
      rect.right-rect.left-(BORDER<<1),
                                    // int       nWidth,
      rect.bottom-rect.top-(BORDER<<1),
                                    // int       nHeight,
      hWnd,                         // HWND      hWndParent,
      NULL,                         // HMENU     hMenu,
      plugin_hinstance(),           // HINSTANCE hInstance,
      NULL                          // LPVOID    lpParam
    );

    break;
  case WM_CLOSE:
    ShowWindow(hWnd,FALSE);
    return 0;
  case WM_COMMAND:
    switch(LOWORD(wParam)) {
    case IDOK:
      //pib_properties_write(form->properties);
      ShowWindow(hWnd,FALSE);
      return 0;
    case IDCANCEL:
      //yas_form_main_rollback(form,hDlg);
      ShowWindow(hWnd,FALSE);
      return 0;
    default:
      break;
    }

    break;
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_CLOSE:
      PostQuitMessage(0);
      break;
    default:
      break;
    }

    break;
  default:
    break;
  }

  return DefWindowProcW(
    hWnd,                           // HWND   hWnd,
    uMsg,                           // UINT   Msg,
    wParam,                         // WPARAM wParam,
    lParam                          // LPARAM lParam
  );
}

WNDCLASSW *yas_main_window_wndclassw(void)
{
  static WNDCLASSW wndclassw;

  if (!wndclassw.lpszClassName) {
#if 0 // [
    wndclassw.style=0;              // UINT      style;
#endif // ]
    wndclassw.lpfnWndProc=yas_main_window_procw;
                                    // WNDPROC   lpfnWndProc;
#if 0 // [
    wndclassw.cbClsExtra=0;         // int       cbClsExtra;
    wndclassw.cbWndExtra=0;         // int       cbWndExtra;
#endif // ]
    wndclassw.hInstance=plugin_hinstance();
                                    // HINSTANCE hInstance;
#if 0 // [
    wndclassw.hIcon=NULL;           // HICON     hIcon;
    wndclassw.hCursor=NULL;         // HCURSOR   hCursor;
#endif // ]
#if 0 // [
    wndclassw.hbrBackground=(HBRUSH)GetStockObject(LTGRAY_BRUSH);
                                    // HBRUSH    hbrBackground;
#else // ] [
    wndclassw.hbrBackground=(HBRUSH)(COLOR_BTNFACE+1),
                                    // HBRUSH    hbrBackground;
#endif // ]
#if 0 // [
    wndclassw.lpszMenuName=NULL;    // LPCWSTR   lpszMenuName;
#endif // ]
    wndclassw.lpszClassName=L"YAS_WINDOW";
                                    // LPCWSTR   lpszClassName;
    RegisterClassW(&wndclassw);
  }

  return &wndclassw;
}
#endif // ]

static void config()
{
  /*
   * Raymond Chen: The dialog manager, part 1: Warm-ups
   *   https://devblogs.microsoft.com/oldnewthing/20050329-00/?p=36043
   */
  if (!yas.hDlg) {
#if defined (GEN_YAS_WINDOW) // [
#if 1 // [
    PibFormInit init={
      .form_vmt=&yas_form_main_vmt,
      .form=&yas.form.main,
      // the following is propagated to all sub-forms:
      .properties=&yas.properties,
      .data=&yas,
    };

    yas.hDlg=CreateWindowExW(
      0,                                  // DWORD     dwExStyle,
      yas_main_window_wndclassw()->lpszClassName,
                                          // LPCWSTR   lpClassName,
      YAS_TITLEW,                         // LPCWSTR   lpWindowName,
      WS_OVERLAPPEDWINDOW,                // DWORD     dwStyle,
      CW_USEDEFAULT,                      // int       X,
      CW_USEDEFAULT,                      // int       Y,
      400,                                // int       nWidth,
      300,                                // int       nHeight,
      NULL,                               // HWND      hWndParent,
      NULL,                               // HMENU     hMenu,
      plugin_hinstance(),                 // HINSTANCE hInstance,
      &init                               // LPVOID    lpParam
    );

#if 0 // [
/*
 * Wouldn't creating an ownership relationship do the trick?
 *   https://stackoverflow.com/questions/812686/can-a-window-be-always-on-top-of-just-one-other-window
 *
 * You must not call SetWindowLong with the GWL_HWNDPARENT index to change
 * the parent of a child window. Instead, use the SetParent function.
 *  https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-setwindowlongw
 */
SetWindowLong(yas.hDlg,GWL_HWNDPARENT,(LONG)plugin_hwnd_main());
#endif // ]
#elif 0 // ] [
    yas.hDlg=yas_main_window_create(&yas);
#else // ] [
    /*
     * Dialog Template
     *   https://www.flipcode.com/archives/Dialog_Template.shtml
     */
    DLGTEMPLATE header={
      .style=/*WS_CAPION|*/WS_SYSMENU|DS_CENTER,
      .dwExtendedStyle=0,
      .cdit=0,
      .x=0,
      .y=0,
      .cx=300,
      .cy=400,
    };

    wchar_t menu[]=L"";
    wchar_t cls[]=L"";

#if 0 // [
    } template UNUSED= {
      .header={
      .style=/*WS_CAPION|*/WS_SYSMENU|DS_CENTER,
      .dwExtendedStyle=0,
      .cdit=0,
      .x=0,
      .y=0,
      .cx=300,
      .cy=400,
      },
      .menu={ 0 },
    };
#endif // ]
#endif // ]

    ShowWindow(yas.hDlg,TRUE);
#else // ] [
    PibFormInit init={
      .form_vmt=&yas_form_main_vmt,
      .form=&yas.form.main,
      // the following is propagated to all sub-forms:
      .properties=&yas.properties,
      .data=&yas,
    };

    yas.hDlg=CreateDialogParamW(
      plugin_hinstance(),                 // HINSTANCE hInstance,
      MAKEINTRESOURCEW(IDD_CONFIG_TAB),   // LPCSTR    lpTemplateName,
      pib_hwnd_parent(),                  // HWND      hWndParent,
      init.form_vmt->dlgproc,             // DLGPROC   lpDialogFunc,
      (LPARAM)&init                       // LPARAM    dwInitParam
    );
#endif // ]
  }
  else if (!IsWindowVisible(yas.hDlg))
    ShowWindow(yas.hDlg,TRUE);
}

HINSTANCE plugin_hinstance(void)
{
  return winampGetGeneralPurposePlugin()->hDllInstance;
}

const char *plugin_titlea(void)
{
   return YAS_TITLEA;
}

const wchar_t *plugin_titlew(void)
{
   return YAS_TITLEW;
}

HWND plugin_hwnd_main(void)
{
  return winampGetGeneralPurposePlugin()->hwndParent;
}

__declspec (dllexport)
winampGeneralPurposePlugin *winampGetGeneralPurposePlugin()
{
  static winampGeneralPurposePlugin gen_yas={
#if defined (GPPHDR_VER_U) // [
    .version=GPPHDR_VER_U,
    .description=(char *)YAS_TITLEW,
#else // ] [
    .version=GPPHDR_VER,
    .description=YAS_TITLEA,
#endif // ]
    .init=init,
    .config=config,
    .quit=quit,
    .hwndParent=NULL,
    .hDllInstance=NULL,
  };

  return &gen_yas;
}
