/*
 * yas_form_algorithms.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>
#if defined (YAS_FORM_ALGORITHMS_VMT) // [
#include <yas/resource.h>

PibFormVmt yas_form_algorithms_vmt;

#define BR L"\r\n"
#define P BR BR
#define BULL L"\x2022 "
#define TAB L"   "
static wchar_t TEXT[]=
  // {
  BULL L"Fisher\x2212Yates shuffle Algorithm:"
  BR L"        "
  L"<a href=\"https://www.geeksforgeeks.org/shuffle-a-given-array-using-fisher-yates-shuffle-algorithm/\">"
  L"https://www.geeksforgeeks.org/</a>"
  BR L"        "
  L"<a href=\"https://www.geeksforgeeks.org/shuffle-a-given-array-using-fisher-yates-shuffle-algorithm/\">"
  L"shuffle-a-given-array-using-fisher-yates-shuffle-algorithm/</a>"
  // }
  // {
  BR BULL L"CryptGenRandom:"
  BR L"        "
  L"<a href=\"https://docs.microsoft.com/en-us/windows/win32/api/wincrypt/nf-wincrypt-cryptgenrandom\">"
  L"https://docs.microsoft.com/en-us/windows/win32/api/</a>"
  BR L"        "
  L"<a href=\"https://docs.microsoft.com/en-us/windows/win32/api/wincrypt/nf-wincrypt-cryptgenrandom\">"
  L"wincrypt/nf-wincrypt-cryptgenrandom</a>"
  // }
  // {
  BR BULL L"WELL Algorithm:"
  BR L"        "
  L"<a href=\"http://lomont.org/papers/2008/Lomont_PRNG_2008.pdf\">"
  L"http://lomont.org/papers/2008/Lomont_PRNG_2008.pdf</a>"
  // }
  // {
  BR BULL L"SIMD-oriented Fast Mersenne Twister (SFMT):"
  BR L"        "
  L"<a href=\"http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/SFMT/\">"
  L"http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/SFMT/</a>"
  // }
  // {
  BR BULL L"C Run-Time Library:"
  BR L"        "
  L"<a href=\"https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/rand?view=vs-2019\">"
  L"https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/</a>"
  BR L"        "
  L"<a href=\"https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/rand?view=vs-2019\">"
  L"rand?view=vs-2019</a>"
  // }
  // {
  BR BULL L"George Marsaglia CMWC4096:"
  BR L"        "
  L"<a href=\"https://groups.google.com/forum/#!msg/comp.lang.c/qZFQgKRCQGg/rmPkaRHqxOMJ\">"
  L"https://groups.google.com/forum/#!msg/comp.lang.c/</a>"
  BR L"        "
  L"<a href=\"https://groups.google.com/forum/#!msg/comp.lang.c/qZFQgKRCQGg/rmPkaRHqxOMJ\">"
  L"qZFQgKRCQGg/rmPkaRHqxOMJ</a>"
  // }
  ;

static PibControlConfig yas_form_algorithms_config[]={
#if 0 // [
  { pib_control_sys_link_new,offsetof(YasProperty,progress.cur),
    .sys_link={ .idc=IDC_STATIC_ALGORITMS,.text=TEXT } },
#else // ] [
  { pib_control_sys_link_new,0,
    .sys_link={ .idc=IDC_STATIC_ALGORITMS,.text=TEXT } },
#endif // ]
};

static PibForm *yas_form_algorithms_create(PibFormInit *init, HWND hDlg)
{
  PibForm *form=init->form;

  if (init->form_vmt!=&yas_form_algorithms_vmt) {
    DMESSAGE("something went wrong");
    goto e_vmt;
  }

  if (pib_form_common_create(init,hDlg)<0) {
    DMESSAGE("creating form");
    goto e_form;
  }

  pib_form_refresh(form,hDlg);

  return form;
e_form:
e_vmt:
  return NULL;
}

static void yas_form_algorithms_refresh(HWND hDlg, PibForm *form)
{
  pib_form_refresh(form,hDlg);
}

#define LINK
static BOOL CALLBACK yas_form_algorithms_proc(HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  PibForm *form=(PibForm *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);

  switch(uMsg) {
  case WM_INITDIALOG:
    form=yas_form_algorithms_create((PibFormInit *)lParam,hDlg);

    SetWindowLongPtrW(
      hDlg,               // HWND     hWnd,
      GWLP_USERDATA,      // int      nIndex,
      (LONG_PTR)form      // LONG_PTR dwNewLong
    );

    return TRUE;
#if defined (SHOWWINDOW) // [
  case WM_SHOWWINDOW:
    if (!wParam)
      pib_form_refresh(form,hDlg);

    break;
#endif // ]
#if defined (LINK) // [
  case WM_NOTIFY:
    switch (((LPNMHDR)lParam)->code) {
    case NM_CLICK:
    case NM_RETURN: {
      ShellExecuteW(
        NULL,
        L"open",
        ((PNMLINK)lParam)->item.szUrl,
        NULL,
        NULL,
        SW_SHOW
      );

      break;
    }
    default:
      break;
    }

    break;
#endif // ]
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      yas_form_algorithms_refresh(hDlg,form);
      break;
    default:
      break;
    }

    break;
#if 0 // [
  case WM_LBUTTONDOWN:
  case WM_LBUTTONUP:
  case WM_MBUTTONDOWN:
  case WM_MBUTTONUP:
  case WM_RBUTTONDOWN:
  case WM_RBUTTONUP:
  case WM_XBUTTONDOWN:
  case WM_XBUTTONUP:
    return TRUE;
#endif // ]
  default:
    break;
  }

  return FALSE;
}

PibFormVmt yas_form_algorithms_vmt={
#if defined (DEBUG) // [
  .id=__FILE__ ":" PIB_STRA(__LINE__) ":yas_form_algorithms_vmt",
#endif // ]
  .label=L"Algorithms",
  .idd=IDD_PAGE_ALGORITHMS,
  .ncontrols=(sizeof yas_form_algorithms_config)
      /(sizeof yas_form_algorithms_config[0]),
  .config=yas_form_algorithms_config,
  .destroy=pib_form_common_destroy,
  .dlgproc=yas_form_algorithms_proc,
};
#endif // ]
