/*
 * yas_form_license.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>
#include <yas/resource.h>

PibFormVmt yas_form_license_vmt;

#define BR L"\r\n"
#define P BR BR
#define BULL L"\x2022 "
#define TAB L"   "
static wchar_t TEXT[]=
  L"YAS is free software; you can redistribute it and/or modify it"
  L" under the terms of the GNU General Public License as"
  L" published by the Free Software Foundation; either version 2.0"
  L" of the License, or (at your option) any later version."
  P L"YAS is distributed in the hope that it will be useful, but"
  L" WITHOUT ANY WARRANTY; without even the implied"
  L" warranty of MERCHANTABILITY or FITNESS FOR A"
  L" PARTICULAR PURPOSE. See the GNU General Public"
  L" License for more details."
  P L"You should have received a copy of the GNU General Public"
  L" License along with this library; if not, write to the Free Software"
  L" Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,"
  L" MA  02110-1301  USA."
  P L"<a href=\"https://www.gnu.org/licenses/old-licenses/gpl-2.0.html\">"
  L"https://www.gnu.org/licenses/old-licenses/gpl-2.0.html"
  L"</a>"
  ;

static PibControlConfig yas_form_license_config[]={
  { pib_control_sys_link_new,0,
    .sys_link={ .idc=IDC_STATIC_LICENSE,.text=TEXT } },
};

static PibForm *yas_form_license_create(PibFormInit *init, HWND hDlg)
{
  PibForm *form=init->form;

  if (init->form_vmt!=&yas_form_license_vmt) {
    DMESSAGE("something went wrong");
    goto e_vmt;
  }

  if (pib_form_common_create(init,hDlg)<0) {
    DMESSAGE("creating form");
    goto e_form;
  }

  pib_form_refresh(form,hDlg);

  return form;
e_form:
e_vmt:
  return NULL;
}

static void yas_form_license_refresh(HWND hDlg, PibForm *form)
{
  pib_form_refresh(form,hDlg);
}

static BOOL CALLBACK yas_form_license_proc(HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  PibForm *form=(PibForm *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);

  switch(uMsg) {
  case WM_INITDIALOG:
    form=yas_form_license_create((PibFormInit *)lParam,hDlg);

    SetWindowLongPtrW(
      hDlg,               // HWND     hWnd,
      GWLP_USERDATA,      // int      nIndex,
      (LONG_PTR)form      // LONG_PTR dwNewLong
    );

    return TRUE;
#if defined (SHOWWINDOW) // [
  case WM_SHOWWINDOW:
    if (!wParam)
      pib_form_refresh(form,hDlg);

    break;
#endif // ]
  case WM_NOTIFY:
    switch (((LPNMHDR)lParam)->code) {
    case NM_CLICK:
    case NM_RETURN: {
      ShellExecuteW(
        NULL,
        L"open",
        ((PNMLINK)lParam)->item.szUrl,
        NULL,
        NULL,
        SW_SHOW
      );

      break;
    }
    default:
      break;
    }

    break;
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      yas_form_license_refresh(hDlg,form);
      break;
    default:
      break;
    }

    break;
#if 0 // [
  case WM_LBUTTONDOWN:
  case WM_LBUTTONUP:
  case WM_MBUTTONDOWN:
  case WM_MBUTTONUP:
  case WM_RBUTTONDOWN:
  case WM_RBUTTONUP:
  case WM_XBUTTONDOWN:
  case WM_XBUTTONUP:
    return TRUE;
#endif // ]
  default:
    break;
  }

  return FALSE;
}

PibFormVmt yas_form_license_vmt={
#if defined (DEBUG) // [
  .id=__FILE__ ":" PIB_STRA(__LINE__) ":yas_form_license_vmt",
#endif // ]
  .label=L"License",
  .idd=IDD_PAGE_LICENSE,
  .ncontrols
      =(sizeof yas_form_license_config)/(sizeof yas_form_license_config[0]),
  .config=yas_form_license_config,
  .destroy=pib_form_common_destroy,
  .dlgproc=yas_form_license_proc,
};
