/*
 * yas_form_license.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>
#include <yas/resource.h>

PibFormVmt yas_form_license_vmt;

/*
"Just to name a few, we'd like to thank (in no particular order)\r\n\r\n* the engineers at Nullsoft who once created Winamp,\r\n * the engineers at Microsoft for providing community editions of their\r\n   compiler and tools,\r\n * the Mingw-w64 project for providing the GNU C compiler and tools on\r\n   Windows,\r\n * Alexey Pavlov et.al. for providing the MSYS2 environment,\r\n * Bram Moolenaar for crafting the gvim editor, \r\n * Richard Stallman for crafting the GPL, and\r\n * SourceForge for providing this great platform for publishing open source\r\n   software.\r\n\r\nThe project's web page is at http://yapib.sourceforge.net/. The download page is at https://sourceforge.net/projects/yapib/files/yapib."
*/
#define BR L"\r\n"
#define P BR BR
#define BULL L"\x2022 "
#define TAB L"   "
static wchar_t TEXT[]=
  L"Just to name a few, we'd like to thank (in no particular order)"
  P BULL L"the engineers at"
    L" <a href=\"https://en.wikipedia.org/wiki/Nullsoft\">Nullsoft</a>"
    L" who once created"
    L" <a href=\"https://en.wikipedia.org/wiki/Winamp\">Winamp</a>,"
  BR BULL L"the engineers at"
    L" <a href=\"https://www.microsoft.com/\">Microsoft</a>"
    L" for providing"
    L" <a href=\"https://www.visualstudio.com/vs/older-downloads/\">"
    L"community editions"
    L"</a>"
    L" of their"
    BR TAB L"compiler and tools,"
  BR BULL L"the"
    L" <a href=\"https://en.wikipedia.org/wiki/Mingw-w64\">Mingw-w64</a> project"
    L" for providing the"
    L" <a href=\"https://gcc.gnu.org/\">GNU C compiler and tools</a>"
    L" on"
    BR TAB L"Windows,"
  BR BULL L"Alexey Pavlov et.al. for providing the"
    L" <a href=\"http://www.msys2.org/\">MSYS2</a>"
    L" environment,"
  BR BULL L"Bram Moolenaar for crafting the"
    L" <a href=\"http://www.vim.org/\">gvim</a>"
    L" editor,"
  BR BULL
    L"<a href=\"https://en.wikipedia.org/wiki/Richard_Stallman\">Richard"
    L" Stallman</a> for crafting the "
    L" <a href=\"https://www.gnu.org/licenses/licenses.html\">GPL</a>, and"
  BR BULL L"<a href=\"https://sourceforge.net/\">SourceForge</a>"
    L" for providing this great platform for publishing open source"
    BR TAB L"software."
  P L"The project's web page is at"
    L" <a href=\"http://yapib.sourceforge.net/\">"
    L"http://yapib.sourceforge.net/"
    L"</a>."
    L" The download page is at"
    L" <a href=\"https://sourceforge.net/projects/yapib/files/\">"
    L"https://sourceforge.net/projects/yapib/files/yapib"
    L"</a>."
  ;

static PibControlConfig yas_form_license_config[]={
  { pib_control_sys_link_new,offsetof(YasProperty,progress.cur),
    .sys_link={ .idc=IDC_STATIC_LICENSE,.text=TEXT } },
};

static PibForm *yas_form_license_create(PibFormInit *init, HWND hDlg)
{
  PibForm *form=init->form;

  if (init->form_vmt!=&yas_form_license_vmt) {
    DMESSAGE("something went wrong");
    goto e_vmt;
  }

  if (pib_form_common_create(init,hDlg)<0) {
    DMESSAGE("creating form");
    goto e_form;
  }

  pib_form_refresh(form,hDlg);

  return form;
e_form:
e_vmt:
  return NULL;
}

static void yas_form_license_refresh(HWND hDlg, PibForm *form)
{
  pib_form_refresh(form,hDlg);
}

#define LINK
static BOOL CALLBACK yas_form_license_proc(HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  PibForm *form=(PibForm *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);

  switch(uMsg) {
  case WM_INITDIALOG:
    form=yas_form_license_create((PibFormInit *)lParam,hDlg);

    SetWindowLongPtrW(
      hDlg,               // HWND     hWnd,
      GWLP_USERDATA,      // int      nIndex,
      (LONG_PTR)form      // LONG_PTR dwNewLong
    );

    return TRUE;
#if defined (SHOWWINDOW) // [
  case WM_SHOWWINDOW:
    if (!wParam)
      pib_form_refresh(form,hDlg);

    break;
#endif // ]
#if defined (LINK) // [
  case WM_NOTIFY:
    switch (((LPNMHDR)lParam)->code) {
    case NM_CLICK:
    case NM_RETURN: {
      ShellExecuteW(
        NULL,
        L"open",
        ((PNMLINK)lParam)->item.szUrl,
        NULL,
        NULL,
        SW_SHOW
      );

      break;
    }
    default:
      break;
    }

    break;
#endif // ]
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      yas_form_license_refresh(hDlg,form);
      break;
    default:
      break;
    }

    break;
#if 0 // [
  case WM_LBUTTONDOWN:
  case WM_LBUTTONUP:
  case WM_MBUTTONDOWN:
  case WM_MBUTTONUP:
  case WM_RBUTTONDOWN:
  case WM_RBUTTONUP:
  case WM_XBUTTONDOWN:
  case WM_XBUTTONUP:
    return TRUE;
#endif // ]
  default:
    break;
  }

  return FALSE;
}

PibFormVmt yas_form_license_vmt={
#if defined (DEBUG) // [
  .id=__FILE__ ":" PIB_STRA(__LINE__) ":yas_form_license_vmt",
#endif // ]
  .label=L"License",
  .idd=IDD_PAGE_LICENSE,
  .ncontrols=(sizeof yas_form_license_config)
      /(sizeof yas_form_license_config[0]),
  .config=yas_form_license_config,
  .destroy=pib_form_common_destroy,
  .dlgproc=yas_form_license_proc,
};
