/*
 * yas_form_main.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>
#include <commctrl.h>
#include <uxtheme.h>
#include <yas/resource.h>

static PibFormVmt *tab_main_vmts[]={
  &yas_form_configuration_vmt,
#if defined (YAS_FORM_ALGORITHMS_VMT) // [
  &yas_form_algorithms_vmt,
#endif // ]
  &yas_form_monitor_vmt,
  &yas_form_license_vmt,
#if ! defined (YAS_FORM_ALGORITHMS_VMT) // [
  &yas_form_sfmt_vmt,
#endif // ]
  &yas_form_references_vmt,
  &yas_form_notes_vmt,
};

enum {
  TAB_MAIN_NPAGES=(sizeof tab_main_vmts)/(sizeof tab_main_vmts[0]),
};

static PibControlConfig yas_form_main_config[]={
  { pib_control_tab_new,YAS_PROPERTY_PAGE,.tab={
    .idc=IDC_TAB_CONTOL_MAIN,
    .npages=TAB_MAIN_NPAGES,
    .form_vmts=tab_main_vmts },
  },
};

static PibForm *yas_form_main_create(PibFormInit *init, HWND hDlg)
{
  PibForm *form=init->form;

  if (init->form_vmt!=&yas_form_main_vmt) {
    DMESSAGE("something went wrong");
    goto e_vmt;
  }

  if (pib_form_common_create(init,hDlg)<0) {
    DMESSAGE("creating form");
    goto e_form;
  }

  pib_form_refresh(form,hDlg);

  return init->form;
e_form:
e_vmt:
  return NULL;
}

///////////////////////////////////////////////////////////////////////////////
static void rand_config_done(PibForm *form, int reset, HWND hDlg)
{
  RECT rect;
  int page;

  if (reset) {
    page=yas_form_trans(form)->page;
    *yas_form_trans(form)=*yas_form_cons(form);
    yas_form_trans(form)->page=page;
  }

  GetWindowRect(hDlg,&rect);
  yas_form_trans(form)->x=rect.left;
  yas_form_trans(form)->y=rect.top;
  pib_properties_write(form->properties);
  PostQuitMessage(0);
}

static void yas_form_main_rollback(PibForm *form, HWND hDlg)
{
  int page;

  page=yas_form_trans(form)->page;
  *yas_form_trans(form)=*yas_form_cons(form);
  yas_form_trans(form)->page=page;
  pib_form_refresh(form,hDlg);
}

#if defined (GEN_YAS_WINDOW) // [
static LRESULT CALLBACK yas_main_window_procw(HWND hWnd, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  enum { BORDER=4 };
  PibForm *form=(PibForm *)GetWindowLongPtrW(hWnd,GWLP_USERDATA);
  PibControl *control;

  switch (uMsg) {
  case WM_CREATE: {
    ////////
    PibFormInit *init=((CREATESTRUCTW *)lParam)->lpCreateParams;
    RECT rect;
    HWND hTab;
    HFONT hFont;

    GetClientRect(hWnd,&rect);

    hTab=CreateWindowExW(
      0,                            // DWORD     dwExStyle,
      WC_TABCONTROLW,               // LPCWSTR   lpClassName,
      L"",                          // LPCWSTR   lpWindowName,
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE, 
                                    // DWORD     dwStyle,
      BORDER,                       // int       X,
      BORDER,                       // int       Y,
      rect.right-rect.left-(BORDER<<1),
                                    // int       nWidth,
      rect.bottom-rect.top-(BORDER<<1),
                                    // int       nHeight,
      hWnd,                         // HWND      hWndParent,
      (HMENU)IDC_TAB_CONTOL_MAIN,   // HMENU     hMenu,
      plugin_hinstance(),           // HINSTANCE hInstance,
      NULL                          // LPVOID    lpParam
    );

    /*
     * How to change text size in basic text window win32 c++
     *   https://stackoverflow.com/questions/17347324/how-to-change-text-size-in-basic-text-window-win32-c
     */
    hFont=CreateFontW(
#if 0 // [
      20,                       // int    cHeight,
#else // ] [
      12,                       // int    cHeight,
#endif // ]
      0,                        // int    cWidth,
      0,                        // int    cEscapement,
      0,                        // int    cOrientation,
      FW_DONTCARE,              // int    cWeight,
      FALSE,                    // DWORD  bItalic,
      FALSE,                    // DWORD  bUnderline,
      FALSE,                    // DWORD  bStrikeOut,
      ANSI_CHARSET,             // DWORD  iCharSet,
      OUT_DEFAULT_PRECIS,       // DWORD  iOutPrecision,
      CLIP_DEFAULT_PRECIS,      // DWORD  iClipPrecision,
      DEFAULT_QUALITY,          // DWORD  iQuality,
      DEFAULT_PITCH|FF_SWISS,   // DWORD  iPitchAndFamily,
#if 0 // [
      L"Arial"                  // LPCSTR pszFaceName
#else // ] [
      L"Ms Shell Dlg"           // LPCSTR pszFaceName
#endif // ]
    );

    SendMessageW(hTab,WM_SETFONT,(WPARAM)hFont,TRUE);

    ////////
    form=yas_form_main_create(init,hWnd);

    SetWindowLongPtrW(
      hWnd,                       // HWND     hWnd,
      GWLP_USERDATA,              // int      nIndex,
      (LONG_PTR)form              // LONG_PTR dwNewLong
    );

    SetWindowTextW(
      hWnd,                       // HWND   hWnd,
      L"Configure " YAS_TITLEW    // LPCSTR lpString
    );

    if (0<=yas_form_cons(form)->x&&0<=yas_form_cons(form)->y) {
      SetWindowPos(
        hWnd,                     // HWND hWnd,
        HWND_TOPMOST,             // HWND hWndInsertAfter,
        yas_form_cons(form)->x,   // int  X,
        yas_form_cons(form)->y,   // int  Y,
        0,                        // int  cx,
        0,                        // int  cy,
        SWP_NOZORDER|SWP_NOSIZE   // UINT uFlags
      );
    }

    break;
  }
  case WM_CLOSE:
    yas_form_main_rollback(form,hWnd);
    ShowWindow(hWnd,FALSE);
    return 0;
  case WM_COMMAND:
    switch(LOWORD(wParam)) {
    case IDOK:
      pib_properties_write(form->properties);
      ShowWindow(hWnd,FALSE);
      return 0;
    case IDCANCEL:
      yas_form_main_rollback(form,hWnd);
      ShowWindow(hWnd,FALSE);
      return 0;
#if 0 // [
    case IDOK:
      pib_properties_write(form->properties);
      ShowWindow(hDlg,FALSE);
      return 0;
    case IDCANCEL:
      yas_form_main_rollback(form,hDlg);
      ShowWindow(hDlg,FALSE);
      return 0;
#endif // ]
    case IDC_BUTTON_COMMIT:
      pib_properties_write(form->properties);
      return TRUE;
    case IDC_BUTTON_ROLLBACK:
      yas_form_main_rollback(form,hWnd);
      return 0;
    default:
      break;
    }

    break;
  case WM_NOTIFY:
    control=(PibControl *)GetWindowLongPtrW(GetDlgItem(hWnd,
        ((NMHDR *)lParam)->idFrom),GWLP_USERDATA);

    if (control) {
DWRITELN("WM_NOTIFY");
      pib_control_notify(control,hWnd,uMsg,wParam,lParam);
    }

    break;
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_CLOSE:
DWRITELN("WM_PIB_IPC/PIB_CLOSE");
      rand_config_done(form,1,hWnd);
      break;
    case PIB_REFRESH:
DWRITELN("WM_PIB_IPC/PIB_REFRESH");
      pib_form_post(form,hWnd,uMsg,wParam,lParam);
      break;
    default:
      break;
    }

    break;
  default:
    break;
  }

  return DefWindowProcW(
    hWnd,                           // HWND   hWnd,
    uMsg,                           // UINT   Msg,
    wParam,                         // WPARAM wParam,
    lParam                          // LPARAM lParam
  );
}

static WNDCLASSW *yas_main_window_wndclassw(void)
{
  static WNDCLASSW wndclassw;

  if (!wndclassw.lpszClassName) {
#if 0 // [
    wndclassw.style=0;              // UINT      style;
#endif // ]
    wndclassw.lpfnWndProc=yas_main_window_procw;
                                    // WNDPROC   lpfnWndProc;
#if 0 // [
    wndclassw.cbClsExtra=0;         // int       cbClsExtra;
    wndclassw.cbWndExtra=0;         // int       cbWndExtra;
#endif // ]
    wndclassw.hInstance=plugin_hinstance();
                                    // HINSTANCE hInstance;
#if 0 // [
    wndclassw.hIcon=NULL;           // HICON     hIcon;
    wndclassw.hCursor=NULL;         // HCURSOR   hCursor;
#endif // ]
#if 0 // [
    wndclassw.hbrBackground=(HBRUSH)GetStockObject(LTGRAY_BRUSH);
                                    // HBRUSH    hbrBackground;
#else // ] [
    wndclassw.hbrBackground=(HBRUSH)(COLOR_BTNFACE+1),
                                    // HBRUSH    hbrBackground;
#endif // ]
#if 0 // [
    wndclassw.lpszMenuName=NULL;    // LPCWSTR   lpszMenuName;
#endif // ]
    wndclassw.lpszClassName=L"YAS_WINDOW";
                                    // LPCWSTR   lpszClassName;
    RegisterClassW(&wndclassw);
  }

  return &wndclassw;
}

HWND yas_main_window_create(Yas *yas)
{
  PibFormInit init={
    .form_vmt=&yas_form_main_vmt,
    .form=&yas->form.main,
    // the following is propagated to all sub-forms:
    .properties=&yas->properties,
    .data=yas,
  };

  return CreateWindowExW(
    WS_EX_DLGMODALFRAME,                // DWORD     dwExStyle,
    yas_main_window_wndclassw()->lpszClassName,
                                        // LPCWSTR   lpClassName,
    YAS_TITLEW,                         // LPCWSTR   lpWindowName,
    //WS_OVERLAPPEDWINDOW,                // DWORD     dwStyle,
    WS_DLGFRAME,                        // DWORD     dwStyle,
    CW_USEDEFAULT,                      // int       X,
    CW_USEDEFAULT,                      // int       Y,
    400,                                // int       nWidth,
    300,                                // int       nHeight,
    plugin_hwnd_main(),                 // HWND      hWndParent,
    NULL,                               // HMENU     hMenu,
    plugin_hinstance(),                 // HINSTANCE hInstance,
    &init                               // LPVOID    lpParam
  );
}
#else // ] [
static BOOL CALLBACK yas_form_main_proc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  PibForm *form=(PibForm *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);
  PibControl *control;

  switch(uMsg) {
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_CLOSE:
DWRITELN("WM_PIB_IPC/PIB_CLOSE");
      rand_config_done(form,1,hDlg);
      break;
    case PIB_REFRESH:
DWRITELN("WM_PIB_IPC/PIB_REFRESH");
      pib_form_post(form,hDlg,uMsg,wParam,lParam);
      break;
    default:
      break;
    }

    break;
  case WM_INITDIALOG:
DWRITELN("WM_INITDIALOG");
//SetWindowTheme(hDlg,L" ",L" ");
    form=yas_form_main_create((PibFormInit *)lParam,hDlg);

    SetWindowLongPtrW(
      hDlg,                       // HWND     hWnd,
      GWLP_USERDATA,              // int      nIndex,
      (LONG_PTR)form              // LONG_PTR dwNewLong
    );

    SetWindowTextW(
      hDlg,                       // HWND   hWnd,
      L"Configure " YAS_TITLEW    // LPCSTR lpString
    );

    if (0<=yas_form_cons(form)->x&&0<=yas_form_cons(form)->y) {
      SetWindowPos(
        hDlg,                     // HWND hWnd,
        HWND_TOPMOST,             // HWND hWndInsertAfter,
        yas_form_cons(form)->x,   // int  X,
        yas_form_cons(form)->y,   // int  Y,
        0,                        // int  cx,
        0,                        // int  cy,
        SWP_NOZORDER|SWP_NOSIZE   // UINT uFlags
      );
    }

    return TRUE;
  case WM_COMMAND:
    switch(LOWORD(wParam)) {
    case IDOK:
      pib_properties_write(form->properties);
      ShowWindow(hDlg,FALSE);
      return TRUE;
    case IDCANCEL:
      yas_form_main_rollback(form,hDlg);
      ShowWindow(hDlg,FALSE);
      return TRUE;
    case IDC_BUTTON_COMMIT:
      pib_properties_write(form->properties);
      return TRUE;
    case IDC_BUTTON_ROLLBACK:
      yas_form_main_rollback(form,hDlg);
      return TRUE;
    default:
      break;
    }

    break;
#if defined (SHOWWINDOW) // [
  case WM_SHOWWINDOW:
    // TODO??? possibly there's nothing left to do because the child
    //   windows are managed by their respective parent window.
    pib_form_post(form,hDlg,uMsg,lParam,wParam);
    break;
#endif // ]
  case WM_NOTIFY:
    control=(PibControl *)GetWindowLongPtrW(GetDlgItem(hDlg,
        ((NMHDR *)lParam)->idFrom),GWLP_USERDATA);

    if (control) {
DWRITELN("WM_NOTIFY");
      pib_control_notify(control,hDlg,uMsg,wParam,lParam);
    }

    break;
  default:
    break;
  }

  return FALSE;
}
#endif // ]

PibFormVmt yas_form_main_vmt={
#if defined (DEBUG) // [
  .id=__FILE__ ":" PIB_STRA(__LINE__) ":yas_form_main_vmt",
#endif // ]
  .label=L"Main",
  .idd=IDD_CONFIG_TAB,
  .ncontrols=(sizeof yas_form_main_config)/(sizeof yas_form_main_config[0]),
  .config=yas_form_main_config,
  .destroy=pib_form_common_destroy,
#if ! defined (GEN_YAS_WINDOW) // [
  .dlgproc=yas_form_main_proc,
#endif // ]
};
