/*
 * yas_form_monitor.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>
#include <yas/resource.h>

PibFormVmt yas_form_monitor_vmt;

#define YAS_PICTURE_NOTIFY
#define DIFF_BASE
//#define YAS_PICTURE_ROUND
//#define YAS_PICTURE_RECT
/*
 * GetSysColorBrush function
 * https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-getsyscolorbrush
 */
#if defined (YAS_PICTURE_RECT) // [
static void picture_paint(PibControl *control, HWND hWnd)
{
  PibForm *form=pib_control_form(control);
  Yas *yas=yas_form_yas(form);
  RECT rect;
  double scale;
  double dx,x,y;
  HBRUSH hBrush;
  struct { int i; } cur;
  struct { double dy; } base;
  struct { double dy; } incr;
  int i;
  PAINTSTRUCT ps;
  HDC hDc;

  GetClientRect(hWnd,&rect);
  dx=(double)(rect.right-rect.left)/yas->hist.nbins;
  y=rect.bottom-rect.top;

  cur.i=yas_histogram_index(&yas->hist,yas->shuffle[yas->cur],yas->size);

  hDc=BeginPaint(hWnd,&ps); // {

  for (i=0,x=0.0;i<yas->hist.nbins;++i,x+=dx) {
    if (cur.i==i) {
      scale=yas->hist.bin[i].base+yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      base.dy=scale*y;

      rect.left=x;
      rect.top=y-base.dy;
      rect.right=x+dx;
      rect.bottom=y;
      hBrush=CreateSolidBrush(RGB(204,196,0));

      FillRect(
        hDc,      // HDC        hDC,
        &rect,    // const RECT *lprc,
        hBrush    // HBRUSH     hbr
      );
    }
#if defined (YAS_PICTURE_NOTIFY) // [
    else if (yas->variables.trans.histogram) {
#else // ] [
    else if (yas->variables.cons.histogram) {
#endif // ]
      scale=yas->hist.bin[i].base;
      scale/=yas->hist.max;
      base.dy=scale*y;

      rect.left=x;
      rect.top=y-base.dy;
      rect.right=x+dx;
      rect.bottom=y;
      hBrush=CreateSolidBrush(RGB(0,208,39));

      FillRect(
        hDc,      // HDC        hDC,
        &rect,    // const RECT *lprc,
        hBrush    // HBRUSH     hbr
      );

      scale=yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      incr.dy=scale*y;

      rect.left=x;
      rect.top=y-base.dy-incr.dy;
      rect.right=x+dx;
      rect.bottom=y-base.dy;
      hBrush=CreateSolidBrush(RGB(204,196,0));

      FillRect(
        hDc,      // HDC        hDC,
        &rect,    // const RECT *lprc,
        hBrush    // HBRUSH     hbr
      );
#if ! defined (YAS_PICTURE_NOTIFY) // [
    }
#else // ] [
    }
#endif // ]
    else {
      scale=yas->hist.bin[i].base+yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      base.dy=scale*y;

      rect.left=x;
      rect.top=y-base.dy;
      rect.right=x+dx;
      rect.bottom=y;
      hBrush=CreateSolidBrush(RGB(0,208,39));

      FillRect(
        hDc,      // HDC        hDC,
        &rect,    // const RECT *lprc,
        hBrush    // HBRUSH     hbr
      );
    }
  }

  EndPaint(hWnd,&ps); // }
}
#else // ] [
static int picture_region_or_rect(HRGN *hRgn, int x1, int y1, int x2, int y2)
{
  HRGN hRgnRect;

  if (*hRgn) {
    hRgnRect=CreateRectRgn(
      x1,                   // int x1,
      y1,                   // int y1,
      x2,                   // int x2,
      y2                    // int y2
    );

    if (!hRgnRect) {
      DMESSAGE("creating region");
      goto e_region;
    }

    CombineRgn(
      *hRgn,                // HRGN hRgnDst,
      *hRgn,                // HRGN hRgnSrc1,
      hRgnRect,             // HRGN hRgnSrc2,
      RGN_OR                // int  iMode
    );

    DeleteObject(
      hRgnRect              // HGDIOBJ ho
    );
  }
  else {
    *hRgn=CreateRectRgn(
      x1,                   // int x1,
      y1,                   // int y1,
      x2,                   // int x2,
      y2                    // int y2
    );

    if (!*hRgn) {
      DMESSAGE("creating region");
      goto e_region;
    }
  }

  return 0;
e_region:
  return -1;
}

static void picture_paint(PibControl *control, HWND hWnd)
{
  PibForm *form=pib_control_form(control);
  Yas *yas=yas_form_yas(form);
  HRGN hRgnBase=NULL;
  HRGN hRgnHighlight=NULL;
  RECT rect;
  double scale;
  double dx,x,y;
  HBRUSH hBrush;
  struct { int i; } cur;
  struct { double dy; } base;
  struct { double dy; } incr;
  int i;
  PAINTSTRUCT ps;
  HDC hDc;
  int err;

  GetClientRect(hWnd,&rect);
  dx=(double)(rect.right-rect.left)/yas->hist.nbins;
  y=rect.bottom-rect.top;

  cur.i=yas_histogram_index(&yas->hist,yas->shuffle[yas->cur],yas->size);
DVWRITELNW("### cur.i:%d",cur.i);

  for (i=0,x=0.0;i<yas->hist.nbins;++i,x+=dx) {
    if (cur.i==i) {
      scale=yas->hist.bin[i].base+yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      base.dy=scale*y;

#if defined (YAS_PICTURE_ROUND) // [
      err=picture_region_or_rect(
        &hRgnHighlight,             // HRGN *hRgn,
        x+0.5,                      // int x1,
        (y-base.dy)+0.5,            // int y1,
        (x+dx)+0.5,                 // int x2,
        y+0.5                       // int y2
      );
#else // ] [
DVWRITELNW("### x1:%.0f y1:%.0f x2:%.0f y2:%.0f",
        x,                          // int x1,
        y-base.dy,                  // int y1,
        x+dx,                       // int x2,
        y                           // int y2
      );
      err=picture_region_or_rect(
        &hRgnHighlight,             // HRGN *hRgn,
        x,                          // int x1,
        y-base.dy,                  // int y1,
        x+dx,                       // int x2,
        y                           // int y2
      );
#endif // ]

      if (err<0)
        goto e_region;
    }
#if defined (YAS_PICTURE_NOTIFY) // [
    else if (yas->variables.trans.histogram) {
#else // ] [
    else if (yas->variables.cons.histogram) {
#endif // ]
      scale=yas->hist.bin[i].base;
      scale/=yas->hist.max;
      base.dy=scale*y;

#if defined (YAS_PICTURE_ROUND) // [
      err=picture_region_or_rect(
        &hRgnBase,                  // HRGN *hRgn,
        x+0.5,                      // int x1,
        (y-base.dy)+0.5,            // int y1,
        (x+dx)+0.5,                 // int x2,
        y+0.5                       // int y2
      );
#else // ] [
      err=picture_region_or_rect(
        &hRgnBase,                  // HRGN *hRgn,
        x,                          // int x1,
        y-base.dy,                  // int y1,
        x+dx,                       // int x2,
        y                           // int y2
      );
#endif // ]

      if (err<0)
        goto e_region;

      scale=yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      incr.dy=scale*y;

#if defined (YAS_PICTURE_ROUND) // [
      err=picture_region_or_rect(
          &hRgnHighlight,           // HRGN *hRgn,
          x+0.5,                    // int x1,
          (y-base.dy-incr.dy)+0.5,  // int y1,
          (x+dx)+0.5,               // int x2,
          y-base.dy+0.5             // int y2
      );
#else // ] [
      err=picture_region_or_rect(
          &hRgnHighlight,           // HRGN *hRgn,
          x,                        // int x1,
          y-base.dy-incr.dy,        // int y1,
          x+dx,                     // int x2,
          y-base.dy                 // int y2
      );
#endif // ]

      if (err<0)
        goto e_region;
#if ! defined (YAS_PICTURE_NOTIFY) // [
    }
#else // ] [
    }
#endif // ]
    else {
      scale=yas->hist.bin[i].base+yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      base.dy=scale*y;

#if defined (YAS_PICTURE_ROUND) // [
      err=picture_region_or_rect(
        &hRgnBase,                  // HRGN *hRgn,
        x+0.5,                      // int x1,
        (y-base.dy)+0.5,            // int y1,
        (x+dx)+0.5,                 // int x2,
        y+0.5                       // int y2
      );
#else // ] [
      err=picture_region_or_rect(
        &hRgnBase,                  // HRGN *hRgn,
        x,                          // int x1,
        y-base.dy,                  // int y1,
        x+dx,                       // int x2,
        y                           // int y2
      );
#endif // ]

      if (err<0)
        goto e_region;
    }
  }

  hDc=BeginPaint(hWnd,&ps); // {

  // All painting occurs here, between BeginPaint and EndPaint.
  if (hRgnBase) {
#if defined (DIFF_BASE) // [
    if (hRgnHighlight) {
      CombineRgn(
        hRgnBase,                   // HRGN hRgnDst,
        hRgnBase,                   // HRGN hRgnSrc1,
        hRgnHighlight,              // HRGN hRgnSrc2,
        RGN_DIFF                    // int  iMode
      );
    }
#endif // ]

    hBrush=CreateSolidBrush(RGB(0,208,39));

    FillRgn(
      hDc,                          // HDC    hdc,
      hRgnBase,                     // HRGN   hrgn,
      hBrush                        // HBRUSH hbr
    );
  }
  else {
    DVMESSAGE("something went wrong: %i: %lu %lu %lu %lu",
        cur.i,yas->cur,yas->shuffle[yas->cur],yas->size);
  }

  if (hRgnHighlight) {
DWRITELN("**************************************************");
    hBrush=CreateSolidBrush(RGB(204,196,0));

    FillRgn(
      hDc,                          // HDC    hdc,
      hRgnHighlight,                // HRGN   hrgn,
      hBrush                        // HBRUSH hbr
    );
  }

  EndPaint(hWnd,&ps); // }
e_region:
  if (hRgnBase) {
    DeleteObject(
      hRgnBase                      // HGDIOBJ ho
    );
  }

  if (hRgnHighlight) {
    DeleteObject(
      hRgnHighlight                 // HGDIOBJ ho
    );
  }
}
#endif // ]

static void label_changed(PibControl *control, HWND hDlg)
{
  enum { SIZE=128 };
  HWND hLabel=GetDlgItem(hDlg,control->label.idc);
  PibForm *form=pib_control_form(control);
  double max=yas_form_trans(form)->progress.max;
  double cur=yas_form_trans(form)->progress.cur;
  wchar_t buf[SIZE];

  if (max) {
#if 0 // [
    swprintf(buf,SIZE,L"%.0f/%.0f ≈ %.1f%%",cur,max,100.0f*cur/max);
#else // ] [
    /*
     * Unicode Character “≈” (U+2248)
     *   https://www.compart.com/en/unicode/U+2248
     */
    swprintf(buf,SIZE,L"%.0f/%.0f \x2248 %.1f%%",cur,max,100.0f*cur/max);
#endif // ]
  }
  else
    buf[0]=L'\0';

  SetWindowTextW(hLabel,buf);
}

enum {
  OFFS_PROGRESS_BAR,
  OFFS_LABEL,
  OFFS_PICTURE,
#if defined (YAS_SWAP_HISTOGRAM) // [
  OFFS_CHECK_BOX,
#endif // ]
};

static void label_refresh(PibControl *control, HWND hDlg)
{
  PibForm *form;

  form=pib_control_form(control);
  pib_control_refresh(form->min[OFFS_LABEL],hDlg);
}

#if defined (YAS_SWAP_HISTOGRAM) // [
static void picture_refresh(PibControl *control, HWND hDlg)
{
  PibForm *form;

  form=pib_control_form(control);
  pib_control_refresh(form->min[OFFS_PICTURE],hDlg);
}
#endif // ]

static PibControlConfig yas_form_monitor_config[]={
  { pib_control_progress_bar_new,YAS_PROPERTY_PROGRESS_CUR,label_refresh,
      .progress_bar={
    .idc=IDC_PROGRESS_BAR_STATE,
    .max_offs=YAS_PROPERTY_PROGRESS_MAX,
#if 0 // [
    .state=PBST_PAUSED,
    //.state=PBST_ERROR,
#endif // ]
  } },
  { pib_control_label_new,YAS_PROPERTY_PROGRESS_CUR,label_changed,.label={
      .idc=IDC_STATIC_STATE }},
  { pib_control_picture_new,.picture={
    .idc=IDC_HISTOGRAM,.paint=picture_paint }},
#if defined (YAS_SWAP_HISTOGRAM) // [
  { pib_control_check_box_new,YAS_PROPERTY_HISTOGRAM,picture_refresh,
      .check_box={ .idc=IDC_CHECK_BOX_HISTOGRAM }},
#endif // ]
};

enum {
  MASK_STATE_NITEMS
      =(sizeof yas_form_monitor_config)/(sizeof yas_form_monitor_config[0]),
};

static PibForm *yas_form_monitor_create(PibFormInit *init, HWND hDlg)
{
  PibForm *form=init->form;

  if (init->form_vmt!=&yas_form_monitor_vmt) {
    DMESSAGE("something went wrong");
    goto e_vmt;
  }

  if (pib_form_common_create(init,hDlg)<0) {
    DMESSAGE("creating form");
    goto e_form;
  }

#if 0 // [
  /*
   * TODO: alter the following potentially dangerous line of code attaching
   *   the label to the progress bar.
   */
  init->form->min[0]->common.attached=init->form->min[1];
#endif // ]
  pib_form_refresh(form,hDlg);

  return form;
e_form:
e_vmt:
  return NULL;
}

static void yas_form_monitor_refresh(HWND hDlg, PibForm *form)
{
  Yas *yas=yas_form_yas(form);

  yas_form_trans(form)->progress.cur=yas->cur+1u;
  yas_form_trans(form)->progress.max=yas->size;
  pib_form_refresh(form,hDlg);
  yas_form_cons(form)->progress=yas_form_trans(form)->progress;
}

BOOL CALLBACK yas_form_monitor_proc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  PibForm *form=(PibForm *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);
  PibControl *control;

  switch(uMsg) {
  case WM_INITDIALOG:
    form=yas_form_monitor_create((PibFormInit *)lParam,hDlg);

    SetWindowLongPtrW(
      hDlg,               // HWND     hWnd,
      GWLP_USERDATA,      // int      nIndex,
      (LONG_PTR)form      // LONG_PTR dwNewLong
    );

    return TRUE;
  case WM_COMMAND:
    control=(PibControl *)GetWindowLongPtrW(GetDlgItem(hDlg,LOWORD(wParam)),
        GWLP_USERDATA);

    if (control)
      pib_control_notify(control,hDlg,uMsg,wParam,lParam);

    break;
  case WM_NOTIFY:
    control=(PibControl *)GetWindowLongPtrW(GetDlgItem(hDlg,LOWORD(wParam)),
        GWLP_USERDATA);

    if (control)
      pib_control_notify(control,hDlg,uMsg,wParam,lParam);

    break;
#if defined (SHOWWINDOW) // [
  case WM_SHOWWINDOW:
    if (!wParam)
      pib_form_refresh(form,hDlg);

    break;
#endif // ]
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      yas_form_monitor_refresh(hDlg,form);
      break;
    default:
      break;
    }

    break;
#if 0 // [
  case WM_LBUTTONDOWN:
  case WM_LBUTTONUP:
  case WM_MBUTTONDOWN:
  case WM_MBUTTONUP:
  case WM_RBUTTONDOWN:
  case WM_RBUTTONUP:
  case WM_XBUTTONDOWN:
  case WM_XBUTTONUP:
    return TRUE;
#endif // ]
  default:
    break;
  }

  return FALSE;
}

PibFormVmt yas_form_monitor_vmt={
#if defined (DEBUG) // [
  .id=__FILE__ ":" PIB_STRA(__LINE__) ":yas_form_monitor_vmt",
#endif // ]
  .label=L"Monitor",
  .idd=IDD_PAGE_MONITOR,
  .ncontrols
      =(sizeof yas_form_monitor_config)/(sizeof yas_form_monitor_config[0]),
  .config=yas_form_monitor_config,
  .destroy=pib_form_common_destroy,
  .dlgproc=yas_form_monitor_proc,
};
