/*
 * yas_form_monitor.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>
#include <yas/resource.h>

PibFormVmt yas_form_monitor_vmt;

static void picture_paint(PibControl *control, HWND hWnd)
{
  PibForm *form=pib_control_form(control);
  Yas *yas=yas_form_yas(form);
  HRGN hRgnBase=NULL;
  HRGN hRgnIncr=NULL;
  RECT rect;
  double scale;
  double dx,x,y;
#if 0 // [
  HBRUSH hBrushBase;
  HBRUSH hBrushIncr;
#else // ] [
  HBRUSH hBrush;
#endif // ]

  struct {
    double dy;
  } base;

  struct {
    double dy;
  } incr;

  HRGN hRgnRect;
  int i;
  PAINTSTRUCT ps;
  HDC hDc;

  GetClientRect(hWnd,&rect);
  dx=(double)(rect.right-rect.left)/yas->hist.nbins;
  y=rect.bottom-rect.top;

  for (i=0,x=0.0;i<yas->hist.nbins;++i,x+=dx) {
#if 0 // [
    if (yas->variables.cons.histogram) {
#endif // ]
      scale=(double)yas->hist.bin[i].base/yas->hist.max;
      base.dy=scale*y;
      scale=(double)yas->hist.bin[i].incr/yas->hist.max;
      incr.dy=scale*y;
#if 0 // [
    }
    else {
      scale=(double)yas->hist.bin[i].base/yas->hist.max;
      scale+=(double)yas->hist.bin[i].incr/yas->hist.max;
      base.dy=scale*y;
    }
#endif // ]

    if (hRgnBase) {
      hRgnRect=CreateRectRgn(
        x+0.5,            // int x1,
        (y-base.dy)+0.5,  // int y1,
        (x+dx)+0.5,       // int x2,
        y+0.5             // int y2
      );

      if (!hRgnRect) {
        DMESSAGEW("creating region");
        goto e_region;
      }

      CombineRgn(
        hRgnBase,         // HRGN hRgnDst,
        hRgnBase,         // HRGN hRgnSrc1,
        hRgnRect,         // HRGN hRgnSrc2,
        RGN_OR            // int  iMode
      );

      DeleteObject(
        hRgnRect          // HGDIOBJ ho
      );

#if 0 // [
      if (yas->variables.cons.histogram) {
#endif // ]
        hRgnRect=CreateRectRgn(
          x+0.5,            // int x1,
          (y-base.dy-incr.dy)+0.5,  // int y1,
          (x+dx)+0.5,       // int x2,
          y-base.dy+0.5             // int y2
        );

        if (!hRgnRect) {
          DMESSAGEW("creating region");
          goto e_region;
        }

        CombineRgn(
          hRgnIncr,         // HRGN hRgnDst,
          hRgnIncr,         // HRGN hRgnSrc1,
          hRgnRect,         // HRGN hRgnSrc2,
          RGN_OR            // int  iMode
        );

        DeleteObject(
          hRgnRect          // HGDIOBJ ho
        );
#if 0 // [
      }
#endif // ]
    }
    else {
      hRgnBase=CreateRectRgn(
        x+0.5,          // int x1,
        (y-base.dy)+0.5,  // int y1,
        (x+dx)+0.5,       // int x2,
        y+0.5             // int y2
      );

#if 0 // [
      if (yas->variables.cons.histogram) {
#endif // ]
        hRgnIncr=CreateRectRgn(
          x+0.5,            // int x1,
          (y-base.dy-incr.dy)+0.5,  // int y1,
          (x+dx)+0.5,       // int x2,
          y-base.dy+0.5             // int y2
        );
#if 0 // [
      }
#endif // ]
    }
  }

  if (!hRgnBase) {
    DMESSAGEW("missing region");
    goto e_region;
  }

#if 0 // [
  if (yas->variables.cons.histogram&&!hRgnIncr) {
    DMESSAGEW("missing region");
    goto e_region;
  }
#else // ]
  if (!hRgnIncr) {
    DMESSAGEW("missing region");
    goto e_region;
  }
#endif // ]

  hDc=BeginPaint(hWnd,&ps); // {

  // All painting occurs here, between BeginPaint and EndPaint.
#if 0 // [
  hBrushBase=CreateSolidBrush(RGB(0,208,39));

  FillRgn(
    hDc,                // HDC    hdc,
    hRgnBase,           // HRGN   hrgn,
    hBrushBase          // HBRUSH hbr
  );

  if (yas->variables.cons.histogram) {
    hBrushIncr=CreateSolidBrush(RGB(204,196,0));

    FillRgn(
      hDc,                // HDC    hdc,
      hRgnIncr,           // HRGN   hrgn,
      hBrushIncr          // HBRUSH hbr
    );
  }
#else // ] [
  hBrush=CreateSolidBrush(RGB(0,208,39));

  FillRgn(
    hDc,                // HDC    hdc,
    hRgnBase,           // HRGN   hrgn,
    hBrush              // HBRUSH hbr
  );

  if (yas->variables.cons.histogram) {
    hBrush=CreateSolidBrush(RGB(204,196,0));

  FillRgn(
    hDc,                // HDC    hdc,
    hRgnIncr,           // HRGN   hrgn,
    hBrush              // HBRUSH hbr
  );
#endif // ]

  EndPaint(hWnd,&ps); // }
e_region:
  if (hRgnIncr) {
    DeleteObject(
      hRgnIncr          // HGDIOBJ ho
    );
  }

  if (hRgnBase) {
    DeleteObject(
      hRgnBase          // HGDIOBJ ho
    );
  }

  return;
}

static void label_changed(PibControl *control, HWND hDlg)
{
  enum { SIZE=128 };
  HWND hLabel=GetDlgItem(hDlg,control->label.idc);
  PibForm *form=pib_control_form(control);
  double max=yas_form_trans(form)->progress.max;
  double cur=yas_form_trans(form)->progress.cur;
  wchar_t buf[SIZE];

  if (max) {
#if 0 // [
    swprintf(buf,SIZE,L"%.0f/%.0f ≈ %.1f%%",cur,max,100.0f*cur/max);
#else // ] [
    /*
     * Unicode Character “≈” (U+2248)
     *   https://www.compart.com/en/unicode/U+2248
     */
    swprintf(buf,SIZE,L"%.0f/%.0f \x2248 %.1f%%",cur,max,100.0f*cur/max);
#endif // ]
	}
  else
    buf[0]=L'\0';

  SetWindowTextW(hLabel,buf);
}

static PibControlConfig yas_form_monitor_config[]={
  { pib_control_progress_bar_new,YAS_PROPERTY_PROGRESS_CUR,NULL,
      .progress_bar={
    .idc=IDC_PROGRESS_BAR_STATE,
    .max_offs=YAS_PROPERTY_PROGRESS_MAX,
#if 0 // [
    .state=PBST_PAUSED,
#endif // ]
  } },
  { pib_control_label_new,YAS_PROPERTY_PROGRESS_CUR,label_changed,.label={
      .idc=IDC_STATIC_STATE }},
  { pib_control_picture_new,.picture={
    .idc=IDC_HISTOGRAM,.paint=picture_paint }},
};

enum {
  MASK_STATE_NITEMS
      =(sizeof yas_form_monitor_config)/(sizeof yas_form_monitor_config[0]),
};

static PibForm *yas_form_monitor_create(PibFormInit *init, HWND hDlg)
{
  PibForm *form=init->form;

  if (init->form_vmt!=&yas_form_monitor_vmt) {
    DMESSAGEW("something went wrong");
    goto e_vmt;
  }

  if (pib_form_common_create(init,hDlg)<0) {
    DMESSAGEW("creating form");
    goto e_form;
  }

  /*
   * TODO: alter the following potentially dangerous line of code attaching
   *   the label to the progress bar.
   */
  init->form->min[0]->common.attached=init->form->min[1];
  pib_form_refresh(form,hDlg);

  return form;
e_form:
e_vmt:
  return NULL;
}

static void yas_form_monitor_refresh(HWND hDlg, PibForm *form)
{
  Yas *yas=yas_form_yas(form);

  yas_form_trans(form)->progress.cur=yas->cur+1u;
  yas_form_trans(form)->progress.max=yas->size;
  pib_form_refresh(form,hDlg);
  yas_form_cons(form)->progress=yas_form_trans(form)->progress;
}

BOOL CALLBACK yas_form_monitor_proc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  PibForm *form=(PibForm *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);

  switch(uMsg) {
  case WM_INITDIALOG:
    form=yas_form_monitor_create((PibFormInit *)lParam,hDlg);

    SetWindowLongPtrW(
      hDlg,               // HWND     hWnd,
      GWLP_USERDATA,      // int      nIndex,
      (LONG_PTR)form      // LONG_PTR dwNewLong
    );

    return TRUE;
#if defined (SHOWWINDOW) // [
  case WM_SHOWWINDOW:
    if (!wParam)
      pib_form_refresh(form,hDlg);

    break;
#endif // ]
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      yas_form_monitor_refresh(hDlg,form);
      break;
    default:
      break;
    }

    break;
#if 0 // [
  case WM_LBUTTONDOWN:
  case WM_LBUTTONUP:
  case WM_MBUTTONDOWN:
  case WM_MBUTTONUP:
  case WM_RBUTTONDOWN:
  case WM_RBUTTONUP:
  case WM_XBUTTONDOWN:
  case WM_XBUTTONUP:
    return TRUE;
#endif // ]
  default:
    break;
  }

  return FALSE;
}

PibFormVmt yas_form_monitor_vmt={
#if defined (DEBUG) // [
  .id=__FILE__ ":" PIB_STRA(__LINE__) ":yas_form_monitor_vmt",
#endif // ]
  .label=L"Monitor",
  .idd=IDD_PAGE_MONITOR,
  .ncontrols
      =(sizeof yas_form_monitor_config)/(sizeof yas_form_monitor_config[0]),
  .config=yas_form_monitor_config,
  .destroy=pib_form_common_destroy,
  .dlgproc=yas_form_monitor_proc,
};
