/*
 * yas_form_sfmt.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>
#if ! defined (YAS_FORM_ALGORITHMS_VMT) // [
#include <yas/resource.h>

PibFormVmt yas_form_sfmt_vmt;

/*
"SFMT is a new variant of Mersenne Twister (MT) introduced by Mutsuo Saito and Makoto Matsumoto in 2006. The algorithm was reported at MCQMC 2006. The article published in the proceedings of MCQMC2006. (see Prof. Matsumoto's Papers on random number generation.) SFMT is a Linear Feedbacked Shift Register (LFSR) generator that generates a 128-bit pseudorandom integer at one step. SFMT is designed with recent parallelism of modern CPUs, such as multi-stage pipelining and SIMD (e.g. 128-bit integer) instructions. It supports 32-bit and 64-bit integers, as well as double precision floating point as output.\r\n\r\nSFMT is much faster than MT, in most platforms. Not only the speed, but also the dimensions of equidistributions at v-bit precision are improved. In addition, recovery from 0-excess initial state is much faster. See Master's Thesis of Mutsuo Saito for detail.\r\n\r\nhttp://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/SFMT/index.html"
*/
static wchar_t TEXT[]=
  L"SFMT is a new variant of Mersenne Twister (MT) introduced by"
  L" Mutsuo Saito and"
  L" <a href=\"http://www.math.sci.hiroshima-u.ac.jp/~m-mat/eindex.html\">"
  L"Makoto Matsumoto</a> in 2006. The algorithm was"
  L" reported at MCQMC 2006. The article published in the proceedings"
  L" of MCQMC2006. (see Prof. Matsumoto's"
  L" <a href=\"http://www.math.sci.hiroshima-u.ac.jp/"
  L"~m-mat/MT/ARTICLES/earticles.html#sfmt\">Papers on random number</a>"
  L" generation.) SFMT is a Linear Feedbacked Shift Register (LFSR)"
  L" generator that generates a 128-bit pseudorandom integer at one"
  L" step. SFMT is designed with recent parallelism of modern CPUs,"
  L" such as multi-stage pipelining and SIMD (e.g. 128-bit integer)"
  L" instructions. It supports 32-bit and 64-bit integers, as well"
  L" as double precision floating point as output.\r\n"
  L"\r\n"
#if 1 // [
  L"SFMT is much faster than MT, in most platforms. Not only the"
  L" speed, but also the dimensions of equidistributions at v-bit"
  L" precision are improved. In addition, recovery from 0-excess"
  L" initial state is much faster. See"
	L" <a href=\"http://www.math.sci.hiroshima-u.ac.jp/"
	L"~m-mat/MT/SFMT/M062821.pdf\">Master's Thesis of Mutsuo Saito</a>"
	L" for detail.\r\n"
  L"\r\n"
#endif // ]
  L"<a href=\"http://www.math.sci.hiroshima-u.ac.jp/"
  L"~m-mat/MT/SFMT/index.html\">"
  L"http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/SFMT/index.html</a>"
  ;

static PibControlConfig yas_form_sfmt_config[]={
  { pib_control_sys_link_new,offsetof(YasProperty,progress.cur),
    .sys_link={ .idc=IDC_STATIC_SFMT,.text=TEXT } },
};

static PibForm *yas_form_sfmt_create(PibFormInit *init, HWND hDlg)
{
  PibForm *form=init->form;

  if (init->form_vmt!=&yas_form_sfmt_vmt) {
    DMESSAGE("something went wrong");
    goto e_vmt;
  }

  if (pib_form_common_create(init,hDlg)<0) {
    DMESSAGE("creating form");
    goto e_form;
  }

  pib_form_refresh(form,hDlg);

  return form;
e_form:
e_vmt:
  return NULL;
}

static void yas_form_sfmt_refresh(HWND hDlg, PibForm *form)
{
  pib_form_refresh(form,hDlg);
}

#define LINK
static BOOL CALLBACK yas_form_sfmt_proc(HWND hDlg, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  PibForm *form=(PibForm *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);

  switch(uMsg) {
  case WM_INITDIALOG:
    form=yas_form_sfmt_create((PibFormInit *)lParam,hDlg);

    SetWindowLongPtrW(
      hDlg,               // HWND     hWnd,
      GWLP_USERDATA,      // int      nIndex,
      (LONG_PTR)form      // LONG_PTR dwNewLong
    );

    return TRUE;
#if defined (SHOWWINDOW) // [
  case WM_SHOWWINDOW:
    if (!wParam)
      pib_form_refresh(form,hDlg);

    break;
#endif // ]
#if defined (LINK) // [
  case WM_NOTIFY:
    switch (((LPNMHDR)lParam)->code) {
    case NM_CLICK:
    case NM_RETURN: {
      ShellExecuteW(
        NULL,
        L"open",
        ((PNMLINK)lParam)->item.szUrl,
        NULL,
        NULL,
        SW_SHOW
      );

      break;
    }
    default:
      break;
    }

    break;
#endif // ]
  case WM_PIB_IPC:
    switch (wParam) {
    case PIB_REFRESH:
      yas_form_sfmt_refresh(hDlg,form);
      break;
    default:
      break;
    }

    break;
#if 0 // [
  case WM_LBUTTONDOWN:
  case WM_LBUTTONUP:
  case WM_MBUTTONDOWN:
  case WM_MBUTTONUP:
  case WM_RBUTTONDOWN:
  case WM_RBUTTONUP:
  case WM_XBUTTONDOWN:
  case WM_XBUTTONUP:
    return TRUE;
#endif // ]
  default:
    break;
  }

  return FALSE;
}

PibFormVmt yas_form_sfmt_vmt={
#if defined (DEBUG) // [
  .id=__FILE__ ":" PIB_STRA(__LINE__) ":yas_form_sfmt_vmt",
#endif // ]
  .label=L"About SFMT",
  .idd=IDD_PAGE_SFMT,
  .ncontrols=(sizeof yas_form_sfmt_config)/(sizeof yas_form_sfmt_config[0]),
  .config=yas_form_sfmt_config,
  .destroy=pib_form_common_destroy,
  .dlgproc=yas_form_sfmt_proc,
};
#endif // ]
