/*
 * yas_histogram.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>

int yas_histogram_create(YasHistogram *hist)
{
  return 0;
}

void yas_histogram_destroy(YasHistogram *hist)
{
}

void yas_histogram_reset(YasHistogram *hist, yas_rand_t nbins)
{
  hist->nbins=nbins<YAS_HIST_NBINS?nbins:YAS_HIST_NBINS;
#if 0 // [
  memset(hist->bin,0,sizeof hist->bin);
#else // ] [
  memset(hist->bin,0,hist->nbins*sizeof hist->bin[0]);
#endif // ]
  hist->max=0u;
}

int yas_histogram_index(YasHistogram *hist, yas_rand_t pos, yas_rand_t size)
{
  return hist->nbins*pos/size;
}

static void yas_histogram_add(YasHistogram *hist, yas_rand_t pos,
    yas_rand_t size, void bin_incr(YasHistogramBin *bin))
{
  int i=yas_histogram_index(hist,pos,size);
  yas_rand_t total;

  if (i<0) {
    DVMESSAGE("index out of range: %d (%u) %u (%u)",i,hist->nbins,pos,size);
    return;
  }
  else if (hist->nbins<=i) {
    DVMESSAGE("index out of range: %d (%u) %u (%u)",i,hist->nbins,pos,size);
    return;
  }

  bin_incr(hist->bin+i);
  total=hist->bin[i].base+hist->bin[i].incr;

  if (hist->max<total)
    hist->max=total;
}

static void yas_histogram_bin_incr_base(YasHistogramBin *bin)
{
  ++bin->base;
}

void yas_histogram_base_add(YasHistogram *hist, yas_rand_t pos,
    yas_rand_t size)
{
  yas_histogram_add(hist,pos,size,yas_histogram_bin_incr_base);
}

static void yas_histogram_bin_incr_incr(YasHistogramBin *bin)
{
  ++bin->incr;
}

void yas_histogram_incr_add(YasHistogram *hist, yas_rand_t pos,
    yas_rand_t size)
{
  yas_histogram_add(hist,pos,size,yas_histogram_bin_incr_incr);
}
