/*
 * yas_rand.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>

/*
 * Shuffle a given array using Fisher–Yates shuffle Algorithm
 *   https://www.geeksforgeeks.org/shuffle-a-given-array-using-fisher-yates-shuffle-algorithm/
 */
#if defined (YAS_RAND_SHUFFLE_LOOP) // [
int yas_rand_shuffle_loop(YasRand *rand, yas_rand_t *array, yas_rand_t size,
    int nloops)
#else // ] [
int yas_rand_shuffle(YasRand *rand, yas_rand_t *array, yas_rand_t size)
#endif // ]
{
  yas_rand_t i,j,k;
#if defined (YAS_RAND_SHUFFLE_LOOP) // [
  int n;
#endif // ]

  for (i=0;i<size;++i)
    array[i]=i;

#if defined (YAS_RAND_SHUFFLE_LOOP) // [
  for (n=0;n<nloops;++n) {
#endif // ]
    for (i=size-1;0<i;--i) {
      if (rand->vmt->next(rand,&j)<0)
        goto exit;

#if 1 // [
      j%=i+1;
#else // ] [
      /*
       * How can I get random integers in a certain range?
       *   http://c-faq.com/lib/randrange.html
       */
      j/=(rand->vmt->max/(i+1)+1);
#endif // ]
      /*
       * swap
       */
      k=array[i];
      array[i]=array[j];
      array[j]=k;
    }
#if defined (YAS_RAND_SHUFFLE_LOOP) // [
  }
#endif // ]

  return 0;
exit:
  return -1;
}
