/*
 * yas_rand_crypt.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>

static YasRandVmt yas_rand_crypt_vmt;

int yas_rand_crypt_create(YasRand *rand)
{
  int bCode;

  rand->vmt=&yas_rand_crypt_vmt;

  bCode=CryptAcquireContextW(
    &rand->hProv,         // HCRYPTPROV *phProv,
    NULL,                 // LPCSTR     szContainer,
    NULL,                 // LPCSTR     szProvider,
    PROV_RSA_FULL,        // DWORD      dwProvType,
    CRYPT_VERIFYCONTEXT   // DWORD      dwFlags
  );

  if (!bCode) {
    DMESSAGE("acquiring context");
    goto e_context;
  }

  return 0;
e_context:
  return -1;
}

static void yas_rand_crypt_destroy(YasRand *rand)
{
  /*bCode=*/CryptReleaseContext(
    rand->hProv,          // HCRYPTPROV hProv,
    0ul                   // DWORD      dwFlags
  );
}

static int yas_rand_crypt_next(YasRand *rand, yas_rand_t *result)
{
  int bCode;

  bCode=CryptGenRandom(
    rand->hProv,          // HCRYPTPROV hProv,
    sizeof *result,       // DWORD      dwLen,
    (BYTE *)result        // BYTE       *pbBuffer
  );

  if (!bCode)
    goto exit;

  return 0;
exit:
  return -1;
}

static YasRandVmt yas_rand_crypt_vmt={
  .id="crypt",
  .max=UINT32_MAX,
  .destroy=yas_rand_crypt_destroy,
  .next=yas_rand_crypt_next,
};
