/*
 * yas_rand_sfmt.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <time.h>
#include <yas.h>

static YasRandVmt yas_rand_sfmt_vmt;

#define YAS_RAND_SFMT_SEED_CRYPT
int yas_rand_sfmt_create(YasRand *rand)
{
#if defined (YAS_RAND_SFMT_SEED_CRYPT) // [
  HCRYPTPROV hProv;
  int bCode;
  uint32_t seed;
#else // ] [
  uint32_t seed=time(NULL);
#endif // ]

  rand->vmt=&yas_rand_sfmt_vmt;
#if defined (YAS_RAND_SFMT_SEED_CRYPT) // [
  bCode=CryptAcquireContextW(
    &hProv,                     // HCRYPTPROV *phProv,
    NULL,                       // LPCSTR     szContainer,
    NULL,                       // LPCSTR     szProvider,
    PROV_RSA_FULL,              // DWORD      dwProvType,
    CRYPT_VERIFYCONTEXT         // DWORD      dwFlags
  );

  if (!bCode) {
    DMESSAGE("acquiring context");
    goto e_context;
  }

  bCode=CryptGenRandom(
    hProv,                      // HCRYPTPROV hProv,
    sizeof seed,                // DWORD      dwLen,
    (BYTE *)&seed               // BYTE       *pbBuffer
  );

  if (!bCode) {
    DMESSAGE("generating state");
    goto e_seed;
  }
#endif // ]

  sfmt_init_gen_rand(&rand->sfmt,seed);

  return 0;
#if defined (YAS_RAND_SFMT_SEED_CRYPT) // [
e_context:
  CryptReleaseContext(
    hProv,                      // HCRYPTPROV hProv,
    0ul                         // DWORD      dwFlags
  );
e_seed:
  return -1;
#endif // ]
}

static void yas_rand_sfmt_destroy(YasRand *rand)
{
}

static int yas_rand_sfmt_next(YasRand *rand, yas_rand_t *result)
{
  *result=sfmt_genrand_uint32(&rand->sfmt);

  return 0;
}

static YasRandVmt yas_rand_sfmt_vmt={
  .id="sfmt",
  .max=UINT32_MAX,
  .destroy=yas_rand_sfmt_destroy,
  .next=yas_rand_sfmt_next,
};
