/*
 * yas_thread_sync_request_create.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>

static PibThreadRequestVmt vmt;

YasThreadSyncRequestCreate *yas_thread_sync_request_create_new(Yas *yas)
{
  size_t size=yas->size*sizeof yas->shuffle[0];
  YasThreadSyncRequestCreate *request;

  ////////
  request=pib_thread_request_common_new(&vmt,&yas->sync,
      sizeof request->yas);

  if (!request) {
    DMESSAGE("allocating request");
    goto e_request;
  }

  ////////
  request->yas.path=wcsdup(yas->state.path);

  if (!request->yas.path) {
    DMESSAGE("duplicating path");
    goto e_path;
  }

  ////////
  request->yas.size=yas->size;
  request->yas.shuffle=malloc(size);

  if (!request->yas.shuffle) {
    DMESSAGE("allocating shuffle");
    goto e_shuffle;
  }

  memcpy(request->yas.shuffle,yas->shuffle,size);

  ////////
  request->yas.cur=yas->cur;
#if defined (YAS_NEG) // [
  request->yas.neg=yas->neg;
#endif // ]
#if defined (YAS_PILOT) // [
  request->yas.pilot=yas->pilot;
#endif // ]

  return request;
//cleanup:
  free(request->yas.shuffle);
e_shuffle:
  free(request->yas.path);
e_path:
  pib_thread_request_common_destroy(&request->common);
e_request:
  return NULL;
}

static void yas_thread_sync_request_create_destroy(PibThreadRequest *request)
{
  free(((YasThreadSyncRequestCreate *)request)->yas.shuffle);
  free(((YasThreadSyncRequestCreate *)request)->yas.path);
  pib_thread_request_common_destroy(request);
}

static void yas_thread_sync_request_create_execute(PibThreadRequest *request)
{
  wchar_t *path=((YasThreadSyncRequestCreate *)request)->yas.path;
  yas_rand_t *shuffle=((YasThreadSyncRequestCreate *)request)->yas.shuffle;
  size_t size=((YasThreadSyncRequestCreate *)request)->yas.size;
  FILE *f;
  yas_rand_t i;
  char patha[MAX_PATH];

  f=_wfopen(
    path,     // const wchar_t *filename,
    L"w"      // const wchar_t *mode
  );

  if (f) {
    fprintf(f,"%0" PIB_STRA(YAS_WIDTH) "u\n",
        ((YasThreadSyncRequestCreate *)request)->yas.cur);
#if defined (YAS_NEG) // [
    fprintf(f,"%0" PIB_STRA(YAS_WIDTH) "u\n",
        ((YasThreadSyncRequestCreate *)request)->yas.neg);
#endif // ]
#if defined (YAS_PILOT) // [
    fprintf(f,"%0" PIB_STRA(YAS_WIDTH) "u\n",
        ((YasThreadSyncRequestCreate *)request)->yas.pilot);
#endif // ]

    for (i=0;i<size;++i)
      fprintf(f,"%0" PIB_STRA(YAS_WIDTH) "u\n",shuffle[i]);

    fclose(f);
  }
  else
    DVMESSAGE("opening file \"%s\"",PIB_WS2CS(path,patha));
}

static PibThreadRequestVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=yas_thread_sync_request_create_destroy,
  .execute=yas_thread_sync_request_create_execute,
};
