/*
 * yas_thread_sync_request_update.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>

static PibThreadRequestVmt vmt;

YasThreadSyncRequestUpdate *yas_thread_sync_request_update_new(Yas *yas)
{
  YasThreadSyncRequestUpdate *yas_request;

  ////////
  yas_request=pib_thread_request_common_new(&vmt,&yas->sync,
      sizeof yas_request->yas);

  if (!yas_request) {
    DMESSAGE("allocating request");
    goto e_request;
  }

  ////////
  yas_request->yas.path=wcsdup(yas->state.path);

  if (!yas_request->yas.path) {
    DMESSAGE("duplicating path");
    goto e_path;
  }

  ////////
  yas_request->yas.cur=yas->cur;
#if defined (YAS_NEG) // [
  yas_request->yas.neg=yas->neg;
#endif // ]
#if defined (YAS_PILOT) // [
  yas_request->yas.pilot=yas->pilot;
#endif // ]

  return yas_request;
//cleanup:
  free(yas_request->yas.path);
e_path:
  pib_thread_request_common_destroy(&yas_request->common);
e_request:
  return NULL;
}

static void yas_thread_sync_request_update_destroy(PibThreadRequest *request)
{
  free(((YasThreadSyncRequestUpdate *)request)->yas.path);
  pib_thread_request_common_destroy(request);
}

static void yas_thread_sync_request_update_execute(PibThreadRequest *request)
{
  wchar_t *path=((YasThreadSyncRequestUpdate *)request)->yas.path;
  FILE *f;
  char patha[MAX_PATH];

  f=_wfopen(
    path,     // const wchar_t *filename,
    L"r+"     // const wchar_t *mode
  );

  if (f) {
    fprintf(f,"%0" PIB_STRA(YAS_WIDTH) "u\n",
        ((YasThreadSyncRequestUpdate *)request)->yas.cur);
#if defined (YAS_NEG) // [
    fprintf(f,"%0" PIB_STRA(YAS_WIDTH) "u\n",
        ((YasThreadSyncRequestUpdate *)request)->yas.neg);
#endif // ]
#if defined (YAS_PILOT) // [
    fprintf(f,"%0" PIB_STRA(YAS_WIDTH) "u\n",
        ((YasThreadSyncRequestUpdate *)request)->yas.pilot);
#endif // ]
    fclose(f);
  }
  else
    DVMESSAGE("opening file \"%s\"",PIB_WS2CS(path,patha));
}

static PibThreadRequestVmt vmt={
#if defined (DEBUG) // [
  .id=__FILE__,
#endif // ]
  .destroy=yas_thread_sync_request_update_destroy,
  .execute=yas_thread_sync_request_update_execute,
};
